searchData={"items":[{"type":"module","title":"AshAuthentication","doc":"AshAuthentication provides a turn-key authentication solution for folks using\n[Ash](https://www.ash-hq.org/).","ref":"AshAuthentication.html"},{"type":"module","title":"Usage - AshAuthentication","doc":"This package assumes that you have [Ash](https://ash-hq.org/) installed and\nconfigured.  See the Ash documentation for details.\n\nOnce installed you can easily add support for authentication by configuring\nthe `AshAuthentication` extension on your resource:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n    attribute :hashed_password, :string, allow_nil?: false, sensitive?: true\n  end\n\n  authentication do\n    strategies do\n      password :password do\n        identity_field :email\n        hashed_password_field :hashed_password\n      end\n    end\n  end\n\n  identities do\n    identity :unique_email, [:email]\n  end\nend\n```\n\nIf you plan on providing authentication via the web, then you will need to\ndefine a plug using `AshAuthentication.Plug` which builds a `Plug.Router` that\nroutes incoming authentication requests to the correct provider and provides\ncallbacks for you to manipulate the conn after success or failure.\n\nIf you're using AshAuthentication with Phoenix, then check out\n[`ash_authentication_phoenix`](https://github.com/team-alembic/ash_authentication_phoenix)\nwhich provides route helpers, a controller abstraction and LiveView components\nfor easy set up.","ref":"AshAuthentication.html#module-usage"},{"type":"module","title":"Authentication Strategies - AshAuthentication","doc":"Currently supported strategies:\n\n1. `AshAuthentication.Strategy.Password`\n   - authenticate users against your local database using a unique identity\n   (such as username or email address) and a password.\n2. `AshAuthentication.Strategy.OAuth2`\n   - authenticate using local or remote [OAuth 2.0](https://oauth.net/2/) compatible services.\n   - also includes:\n     - `AshAuthentication.Strategy.Apple`\n     - `AshAuthentication.Strategy.Auth0`\n     - `AshAuthentication.Strategy.Github`\n     - `AshAuthentication.Strategy.Google`\n     - `AshAuthentication.Strategy.Oidc`\n     - `AshAuthentication.Strategy.Slack`\n3. `AshAuthentication.Strategy.MagicLink`\n   - authenticate by sending a single-use link to the user.","ref":"AshAuthentication.html#module-authentication-strategies"},{"type":"module","title":"HTTP client settings - AshAuthentication","doc":"Most of the authentication strategies based on `OAuth2` wrap the [`assent`](https://hex.pm/packages/assent) package.\n\nIf you needs to customize the behavior of the http client used by `assent`, define a custom `http_adapter` in the\napplication settings:\n\n`config :ash_authentication, :http_adapter, {Assent.HTTPAdapter.Finch, supervisor: MyApp.CustomFinch}`\n\nSee [`assent's documentation`](https://hexdocs.pm/assent/README.html#http-client) for more details on the supported\nhttp clients and their configuration.","ref":"AshAuthentication.html#module-http-client-settings"},{"type":"module","title":"Add-ons - AshAuthentication","doc":"Add-ons are like strategies, except that they don't actually provide\nauthentication - they just provide features adjacent to authentication.\nCurrent add-ons:\n\n1. `AshAuthentication.AddOn.Confirmation`\n   - allows you to force the user to confirm changes using a confirmation\n     token (eg. sending a confirmation email when a new user registers).","ref":"AshAuthentication.html#module-add-ons"},{"type":"module","title":"Supervisor - AshAuthentication","doc":"Some add-ons or strategies may require processes to be started which manage\ntheir state over the lifetime of the application (eg periodically deleting\nexpired token revocations).  Because of this you should add\n`{AshAuthentication.Supervisor, otp_app: :my_app}` to your application's\nsupervision tree.  See [the Elixir\ndocs](https://hexdocs.pm/elixir/Application.html#module-the-application-callback-module)\nfor more information.","ref":"AshAuthentication.html#module-supervisor"},{"type":"function","title":"AshAuthentication.authenticated_resources/1","doc":"Find all resources which support authentication for a given OTP application.\n\nReturns a list of resource modules.","ref":"AshAuthentication.html#authenticated_resources/1"},{"type":"macro","title":"AshAuthentication.authentication/1","doc":"","ref":"AshAuthentication.html#authentication/1"},{"type":"function","title":"AshAuthentication.do_subject_to_user/3","doc":"","ref":"AshAuthentication.html#do_subject_to_user/3"},{"type":"type","title":"AshAuthentication.resource_config/0","doc":"","ref":"AshAuthentication.html#t:resource_config/0"},{"type":"type","title":"AshAuthentication.subject/0","doc":"","ref":"AshAuthentication.html#t:subject/0"},{"type":"function","title":"AshAuthentication.subject_to_user/3","doc":"Given a subject string, attempt to retrieve a user record.\n\n    iex> %{id: user_id} = build_user()\n    ...> {:ok, %{id: ^user_id}} = subject_to_user(\"user?id=#{user_id}\", Example.User)\n\nAny options passed will be passed to the underlying `Domain.read/2` callback.","ref":"AshAuthentication.html#subject_to_user/3"},{"type":"function","title":"AshAuthentication.user_to_subject/1","doc":"Return a subject string for user.\n\nThis is done by concatenating the resource's subject name with the resource's\nprimary key field(s) to generate a uri-like string.\n\nExample:\n\n    iex> build_user(id: \"ce7969f9-afa5-474c-bc52-ac23a103cef6\") |> user_to_subject()\n    \"user?id=ce7969f9-afa5-474c-bc52-ac23a103cef6\"","ref":"AshAuthentication.html#user_to_subject/1"},{"type":"module","title":"AshAuthentication.TokenResource","doc":"This is an Ash resource extension which generates the default token resource.\n\nThe token resource is used to store information about tokens that should not\nbe shared with the end user.  It does not actually contain any tokens.\n\nFor example:\n\n  * When an authentication token has been revoked\n  * When a confirmation token has changes to apply","ref":"AshAuthentication.TokenResource.html"},{"type":"module","title":"Storage - AshAuthentication.TokenResource","doc":"The information stored in this resource is essentially ephemeral - all tokens\nhave an expiry date, so it doesn't make sense to keep them after that time has\npassed.  However, if you have any tokens with very long expiry times then we\nsuggest you store this resource in a resilient data-layer such as Postgres.","ref":"AshAuthentication.TokenResource.html#module-storage"},{"type":"module","title":"Usage - AshAuthentication.TokenResource","doc":"There is no need to define any attributes or actions (although you can if you\nwant).  The extension will wire up everything that's needed for the token\nsystem to function.\n\n```\ndefmodule MyApp.Accounts.Token do\n  use Ash.Resource,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshAuthentication.TokenResource],\n    domain: MyApp.Accounts\n\n  postgres do\n    table \"tokens\"\n    repo MyApp.Repo\n  end\nend\n```\n\nWhilst it is possible to have multiple token resources, there is no need to do\nso.","ref":"AshAuthentication.TokenResource.html#module-usage"},{"type":"module","title":"Removing expired records - AshAuthentication.TokenResource","doc":"Once a token has expired there's no point in keeping the information it refers\nto, so expired tokens can be automatically removed by adding the\n`AshAuthentication.Supervisor` to your application supervision tree.  This\nwill start the `AshAuthentication.TokenResource.Expunger` `GenServer` which\nperiodically scans and removes any expired records.","ref":"AshAuthentication.TokenResource.html#module-removing-expired-records"},{"type":"function","title":"AshAuthentication.TokenResource.expunge_expired/2","doc":"Remove all expired records.","ref":"AshAuthentication.TokenResource.html#expunge_expired/2"},{"type":"function","title":"AshAuthentication.TokenResource.jti_revoked?/3","doc":"Has the token been revoked?\n\nSimilar to `token-revoked?/2..3` except that rather than extracting the JTI\nfrom the token, assumes that it's being passed in directly.","ref":"AshAuthentication.TokenResource.html#jti_revoked?/3"},{"type":"function","title":"AshAuthentication.TokenResource.revoke/3","doc":"Revoke a token.\n\nExtracts the JTI from the provided token and uses it to generate a revocation\nrecord.","ref":"AshAuthentication.TokenResource.html#revoke/3"},{"type":"macro","title":"AshAuthentication.TokenResource.token/1","doc":"","ref":"AshAuthentication.TokenResource.html#token/1"},{"type":"function","title":"AshAuthentication.TokenResource.token_revoked?/3","doc":"Has the token been revoked?\n\nSimilar to `jti_revoked?/2..3` except that it extracts the JTI from the token,\nrather than relying on it to be passed in.","ref":"AshAuthentication.TokenResource.html#token_revoked?/3"},{"type":"module","title":"AshAuthentication.UserIdentity","doc":"An Ash extension which generates the default user identities resource.\n\nIf you plan to support multiple different strategies at once (eg giving your\nusers the choice of more than one authentication provider, or signing them into\nmultiple services simultaneously) then you will want to create a resource with\nthis extension enabled. It is used to keep track of the links between your\nlocal user records and their many remote identities.\n\nThe user identities resource is used to store information returned by remote\nauthentication strategies (such as those provided by OAuth2) and maps them to\nyour user resource(s).  This provides the following benefits:\n\n  1. A user can be signed in to multiple authentication strategies at once.\n  2. For those providers that support it, AshAuthentication can handle\n     automatic refreshing of tokens.","ref":"AshAuthentication.UserIdentity.html"},{"type":"module","title":"Storage - AshAuthentication.UserIdentity","doc":"User identities are expected to be relatively long-lived (although they're\ndeleted on log out), so should probably be stored using a permanent data layer\nsush as `ash_postgres`.","ref":"AshAuthentication.UserIdentity.html#module-storage"},{"type":"module","title":"Usage - AshAuthentication.UserIdentity","doc":"There is no need to define any attributes, etc.  The extension will generate\nthem all for you.  As there is no other use-case for this resource it's\nunlikely that you will need to customise it.\n\n```elixir\ndefmodule MyApp.Accounts.UserIdentity do\n  use Ash.Resource,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshAuthentication.UserIdentity],\n    domain: MyApp.Accounts\n\n  user_identity do\n    user_resource MyApp.Accounts.User\n  end\n\n  postgres do\n    table \"user_identities\"\n    repo MyApp.Repo\n  end\nend\n```\n\nIf you intend to operate with multiple user resources, you will need to define\nmultiple user identity resources.","ref":"AshAuthentication.UserIdentity.html#module-usage"},{"type":"macro","title":"AshAuthentication.UserIdentity.user_identity/1","doc":"","ref":"AshAuthentication.UserIdentity.html#user_identity/1"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation","doc":"Confirmation support.\n\nSometimes when creating a new user, or changing a sensitive attribute (such as\ntheir email address) you may want to wait for the user to confirm by way of\nsending them a confirmation token to prove that it was really them that took\nthe action.\n\nIn order to add confirmation to your resource, it must been the following\nminimum requirements:\n\n1. Have a primary key\n2. Have at least one attribute you wish to confirm\n3. Tokens must be enabled","ref":"AshAuthentication.AddOn.Confirmation.html"},{"type":"module","title":"Example - AshAuthentication.AddOn.Confirmation","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n  end\n\n  authentication do\n    add_ons do\n      confirmation :confirm do\n        monitor_fields [:email]\n        sender MyApp.ConfirmationSender\n      end\n    end\n\n    strategies do\n      # ...\n    end\n  end\n\n  identities do\n    identity :email, [:email]\n  end\nend\n```","ref":"AshAuthentication.AddOn.Confirmation.html#module-example"},{"type":"module","title":"Attributes - AshAuthentication.AddOn.Confirmation","doc":"A `confirmed_at` attribute will be added to your resource if it's not already\npresent (see `confirmed_at_field` in the DSL documentation).","ref":"AshAuthentication.AddOn.Confirmation.html#module-attributes"},{"type":"module","title":"Actions - AshAuthentication.AddOn.Confirmation","doc":"By default confirmation will add an action which updates the `confirmed_at`\nattribute as well as retrieving previously stored changes and applying them to\nthe resource.\n\nIf you wish to perform the confirm action directly from your code you can do\nso via the `AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.AddOn.Confirmation.html#module-actions"},{"type":"module","title":"Example - AshAuthentication.AddOn.Confirmation","doc":"iex> strategy = Info.strategy!(Example.User, :confirm)\n    ...> {:ok, user} = Strategy.action(strategy, :confirm, %{\"confirm\" => confirmation_token()})\n    ...> user.confirmed_at >= one_second_ago()\n    true","ref":"AshAuthentication.AddOn.Confirmation.html#module-example"},{"type":"module","title":"Plugs - AshAuthentication.AddOn.Confirmation","doc":"Confirmation provides a single endpoint for the `:confirm` phase.  If you wish\nto interact with the plugs directly, you can do so via the\n`AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.AddOn.Confirmation.html#module-plugs"},{"type":"module","title":"Example - AshAuthentication.AddOn.Confirmation","doc":"iex> strategy = Info.strategy!(Example.User, :confirm)\n    ...> conn = conn(:get, \"/user/confirm\", %{\"confirm\" => confirmation_token()})\n    ...> conn = Strategy.plug(strategy, :confirm, conn)\n    ...> {_conn, {:ok, user}} = Plug.Helpers.get_authentication_result(conn)\n    ...> user.confirmed_at >= one_second_ago()\n    true","ref":"AshAuthentication.AddOn.Confirmation.html#module-example"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.confirmation_token/3","doc":"Generate a confirmation token for a changeset.\n\nThis will generate a token with the `\"act\"` claim set to the confirmation\naction for the strategy, and the `\"chg\"` claim will contain any changes.","ref":"AshAuthentication.AddOn.Confirmation.html#confirmation_token/3"},{"type":"type","title":"AshAuthentication.AddOn.Confirmation.t/0","doc":"","ref":"AshAuthentication.AddOn.Confirmation.html#t:t/0"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.transform/2","doc":"","ref":"AshAuthentication.AddOn.Confirmation.html#transform/2"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.verify/2","doc":"","ref":"AshAuthentication.AddOn.Confirmation.html#verify/2"},{"type":"protocol","title":"AshAuthentication.Strategy","doc":"The protocol used for interacting with authentication strategies.\n\nAny new Authentication strategy must implement this protocol.","ref":"AshAuthentication.Strategy.html"},{"type":"type","title":"AshAuthentication.Strategy.action/0","doc":"The name of an individual action supported by the strategy.\n\nThis maybe not be the action name on the underlying resource, which may be\ngenerated, but the name that the strategy itself calls the action.","ref":"AshAuthentication.Strategy.html#t:action/0"},{"type":"function","title":"AshAuthentication.Strategy.action/4","doc":"Perform an named action.\n\nDifferent strategies are likely to implement a number of different actions\ndepending on their configuration.  Calling them via this function will ensure\nthat the context is correctly set, etc.\n\nSee `actions/1` for a list of actions provided by the strategy.\n\nAny options passed to the action will be passed to the underlying `Ash.Domain` function.","ref":"AshAuthentication.Strategy.html#action/4"},{"type":"function","title":"AshAuthentication.Strategy.actions/1","doc":"Return a list of actions supported by the strategy.","ref":"AshAuthentication.Strategy.html#actions/1"},{"type":"function","title":"Example - AshAuthentication.Strategy.actions/1","doc":"iex> strategy = Info.strategy!(Example.User, :password)\n    ...> actions(strategy)\n    [:sign_in_with_token, :register, :sign_in, :reset_request, :reset]","ref":"AshAuthentication.Strategy.html#actions/1-example"},{"type":"type","title":"AshAuthentication.Strategy.http_method/0","doc":"","ref":"AshAuthentication.Strategy.html#t:http_method/0"},{"type":"function","title":"AshAuthentication.Strategy.method_for_phase/2","doc":"Return the HTTP method for a phase.","ref":"AshAuthentication.Strategy.html#method_for_phase/2"},{"type":"function","title":"Example - AshAuthentication.Strategy.method_for_phase/2","doc":"iex> strategy = Info.strategy!(Example.User, :oauth2)\n    ...> method_for_phase(strategy, :request)\n    :get","ref":"AshAuthentication.Strategy.html#method_for_phase/2-example"},{"type":"function","title":"AshAuthentication.Strategy.name/1","doc":"The \"short name\" of the strategy, used for genererating routes, etc.\n\nThis is most likely the same value that you use for the entity's `name`\nargument.","ref":"AshAuthentication.Strategy.html#name/1"},{"type":"type","title":"AshAuthentication.Strategy.path/0","doc":"A path to match in web requests","ref":"AshAuthentication.Strategy.html#t:path/0"},{"type":"type","title":"AshAuthentication.Strategy.phase/0","doc":"The \"phase\" of the request.\n\nUsually `:request` or `:callback` but can be any atom.","ref":"AshAuthentication.Strategy.html#t:phase/0"},{"type":"function","title":"AshAuthentication.Strategy.phases/1","doc":"Return a list of phases supported by the strategy.","ref":"AshAuthentication.Strategy.html#phases/1"},{"type":"function","title":"Example - AshAuthentication.Strategy.phases/1","doc":"iex> strategy = Info.strategy!(Example.User, :password)\n    ...> phases(strategy)\n    [:sign_in_with_token, :register, :sign_in, :reset_request, :reset]","ref":"AshAuthentication.Strategy.html#phases/1-example"},{"type":"function","title":"AshAuthentication.Strategy.plug/3","doc":"Handle requests routed to the strategy.\n\nEach phase will be an atom (ie the second element in the route tuple).\n\nSee `phases/1` for a list of phases supported by the strategy.","ref":"AshAuthentication.Strategy.html#plug/3"},{"type":"type","title":"AshAuthentication.Strategy.route/0","doc":"An individual route.\n\nEg: `{\"/user/password/sign_in\", :sign_in}`","ref":"AshAuthentication.Strategy.html#t:route/0"},{"type":"function","title":"AshAuthentication.Strategy.routes/1","doc":"Used to build the routing table to route web requests to request phases for\neach strategy.","ref":"AshAuthentication.Strategy.html#routes/1"},{"type":"function","title":"Example - AshAuthentication.Strategy.routes/1","doc":"iex> strategy = Info.strategy!(Example.User, :password)\n    ...> routes(strategy)\n    [\n      {\"/user/password/sign_in_with_token\", :sign_in_with_token},\n      {\"/user/password/register\", :register},\n      {\"/user/password/sign_in\", :sign_in},\n      {\"/user/password/reset_request\", :reset_request},\n      {\"/user/password/reset\", :reset}\n    ]","ref":"AshAuthentication.Strategy.html#routes/1-example"},{"type":"type","title":"AshAuthentication.Strategy.t/0","doc":"All the types that implement this protocol.","ref":"AshAuthentication.Strategy.html#t:t/0"},{"type":"function","title":"AshAuthentication.Strategy.tokens_required?/1","doc":"Indicates that the strategy creates or consumes tokens.","ref":"AshAuthentication.Strategy.html#tokens_required?/1"},{"type":"module","title":"AshAuthentication.Strategy.Apple","doc":"Strategy for authenticating using [Apple Sign In](https://developer.apple.com/sign-in-with-apple/)\n\nThis strategy builds on-top of `AshAuthentication.Strategy.Oidc` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use Apple Sign In you need to provide the following minimum configuration:\n\n  - `client_id`\n  - `team_id`\n  - `private_key_id`\n  - `private_key_path`\n  - `redirect_uri`","ref":"AshAuthentication.Strategy.Apple.html"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Apple","doc":"- The [Apple Sign In Documentation](https://developer.apple.com/documentation/sign_in_with_apple).\n- The [OIDC documentation](`AshAuthentication.Strategy.Oidc`)","ref":"AshAuthentication.Strategy.Apple.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Apple.transform/2","doc":"","ref":"AshAuthentication.Strategy.Apple.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Apple.verify/2","doc":"","ref":"AshAuthentication.Strategy.Apple.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Auth0","doc":"Strategy for authenticating using [Auth0](https://auth0.com).\n\nThis strategy builds on-top of `AshAuthentication.Strategy.OAuth2` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use Auth0 you need to provide the following minimum configuration:\n\n  - `client_id`\n  - `redirect_uri`\n  - `client_secret`\n  - `site`","ref":"AshAuthentication.Strategy.Auth0.html"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Auth0","doc":"- The [Auth0 Tutorial](/documentation/tutorial/auth0.md).\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)","ref":"AshAuthentication.Strategy.Auth0.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Auth0.transform/2","doc":"","ref":"AshAuthentication.Strategy.Auth0.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Auth0.verify/2","doc":"","ref":"AshAuthentication.Strategy.Auth0.html#verify/2"},{"type":"behaviour","title":"AshAuthentication.Strategy.Custom","doc":"Define your own custom authentication strategy.\n\nSee [the Custom Strategies guide](/documentation/topics/custom-strategy.md)\nfor more information.","ref":"AshAuthentication.Strategy.Custom.html"},{"type":"type","title":"AshAuthentication.Strategy.Custom.entity/0","doc":"A Strategy DSL Entity.\n\nSee `Spark.Dsl.Entity` for more information.","ref":"AshAuthentication.Strategy.Custom.html#t:entity/0"},{"type":"function","title":"AshAuthentication.Strategy.Custom.set_defaults/2","doc":"Sets default values for a DSL schema based on a set of defaults.\n\nIf a given default is in the schema, it can be overriden, so we just set the default\nand mark it not required.\n\nIf not, then we add it to `auto_set_fields`, and the user cannot override it.","ref":"AshAuthentication.Strategy.Custom.html#set_defaults/2"},{"type":"type","title":"AshAuthentication.Strategy.Custom.strategy/0","doc":"This is the DSL target for your entity and the struct for which you will\nimplement the `AshAuthentication.Strategy` protocol.\n\nThe only required field is `strategy_module` which is used to keep track of\nwhich custom strategy created which strategy.","ref":"AshAuthentication.Strategy.Custom.html#t:strategy/0"},{"type":"callback","title":"AshAuthentication.Strategy.Custom.transform/2","doc":"If your strategy needs to modify either the entity or the parent resource then\nyou can implement this callback.\n\nThis callback can return one of three results:\n\n  - `{:ok, Entity.t}` - an updated DSL entity - useful if you're just changing\n    the entity itself and not changing the wider DSL state of the resource.\n    If this is the response then the transformer will take care of updating\n    the entity in the DSL state.\n  - `{:ok, Dsl.t}` - an updated DSL state for the entire resource.\n  - `{:error, Exception.t}` - a compilation-stopping problem was found. Any\n    exception can be returned, but we strongly advise you to return a\n    `Spark.Error.DslError`.","ref":"AshAuthentication.Strategy.Custom.html#c:transform/2"},{"type":"callback","title":"AshAuthentication.Strategy.Custom.verify/2","doc":"If your strategy needs to verify either the entity or the parent resource then\nyou can implement this callback.\n\nThis is called post-compilation in the `@after_verify` hook - see `Module` for\nmore information.\n\nThis callback can return one of the following results:\n\n  - `:ok` - everything is A-Okay.\n  - `{:error, Exception.t}` - a compilation-stopping problem was found. Any\n    exception can be returned, but we strongly advise you to return a\n    `Spark.Error.DslError`.","ref":"AshAuthentication.Strategy.Custom.html#c:verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Github","doc":"Strategy for authenticating using [GitHub](https://github.com)\n\nThis strategy builds on-top of `AshAuthentication.Strategy.OAuth2` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use GitHub you need to provide the following minimum configuration:\n\n  - `client_id`\n  - `redirect_uri`\n  - `client_secret`","ref":"AshAuthentication.Strategy.Github.html"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Github","doc":"- The [GitHub Tutorial](/documentation/tutorial/github.md).\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)","ref":"AshAuthentication.Strategy.Github.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Github.transform/2","doc":"","ref":"AshAuthentication.Strategy.Github.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Github.verify/2","doc":"","ref":"AshAuthentication.Strategy.Github.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Google","doc":"Strategy for authenticating using [Google](https://google.com)\n\nThis strategy builds on-top of `AshAuthentication.Strategy.OAuth2` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use Google you need to provide the following minimum configuration:\n\n  - `client_id`\n  - `redirect_uri`\n  - `client_secret`\n  - `site`","ref":"AshAuthentication.Strategy.Google.html"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Google","doc":"- The [Google OAuth 2.0 Overview](https://developers.google.com/identity/protocols/oauth2).\n- The [Google Tutorial](/documentation/tutorial/google.md)\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)","ref":"AshAuthentication.Strategy.Google.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Google.transform/2","doc":"","ref":"AshAuthentication.Strategy.Google.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Google.verify/2","doc":"","ref":"AshAuthentication.Strategy.Google.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink","doc":"Strategy for authentication using a magic link.\n\nIn order to use magic link authentication your resource needs to meet the\nfollowing minimum requirements:\n\n1. Have a primary key.\n2. A uniquely constrained identity field (eg `username` or `email`)\n3. Have tokens enabled.\n\nThere are other options documented in the DSL.","ref":"AshAuthentication.Strategy.MagicLink.html"},{"type":"module","title":"Example - AshAuthentication.Strategy.MagicLink","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n  end\n\n  authentication do\n    strategies do\n      magic_link do\n        identity_field :email\n        sender fn user_or_email, token, _opts ->\n          # will be a user if the token relates to an existing user\n          # will be an email if there is no matching user (such as during sign up)\n          # opts will contain the `tenant` key, use this if you need to alter the link based\n          # on the tenant that requested the token\n          MyApp.Emails.deliver_magic_link(user_or_email, token)\n        end\n      end\n    end\n  end\n\n  identities do\n    identity :unique_email, [:email]\n  end\nend\n```","ref":"AshAuthentication.Strategy.MagicLink.html#module-example"},{"type":"module","title":"Tenancy - AshAuthentication.Strategy.MagicLink","doc":"Note that the tenant is provided to the sender in the `opts` key. Use this if you need\nto modify the url (i.e `tenant.app.com`) based on the tenant that requested the token.","ref":"AshAuthentication.Strategy.MagicLink.html#module-tenancy"},{"type":"module","title":"Actions - AshAuthentication.Strategy.MagicLink","doc":"By default the magic link strategy will automatically generate the request and\nsign-in actions for you, however you're free to define them yourself.  If you\ndo, then the action will be validated to ensure that all the needed\nconfiguration is present.\n\nIf you wish to work with the actions directly from your code you can do so via\nthe `AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.Strategy.MagicLink.html#module-actions"},{"type":"module","title":"Examples - AshAuthentication.Strategy.MagicLink","doc":"Requesting that a magic link token is sent for a user:\n\n    iex> strategy = Info.strategy!(Example.User, :magic_link)\n    ...> user = build_user()\n    ...> Strategy.action(strategy, :request, %{\"username\" => user.username})\n    :ok\n\nSigning in using a magic link token:\n\n    ...> {:ok, token} = MagicLink.request_token_for(strategy, user)\n    ...> {:ok, signed_in_user} = Strategy.action(strategy, :sign_in, %{\"token\" => token})\n    ...> signed_in_user.id == user\n    true","ref":"AshAuthentication.Strategy.MagicLink.html#module-examples"},{"type":"module","title":"Plugs - AshAuthentication.Strategy.MagicLink","doc":"The magic link strategy provides plug endpoints for both request and sign-in\nactions.\n\nIf you wish to work with the plugs directly, you can do so via the\n`AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.Strategy.MagicLink.html#module-plugs"},{"type":"module","title":"Examples: - AshAuthentication.Strategy.MagicLink","doc":"Dispatching to plugs directly:\n\n    iex> strategy = Info.strategy!(Example.User, :magic_link)\n    ...> user = build_user()\n    ...> conn = conn(:post, \"/user/magic_link/request\", %{\"user\" => %{\"username\" => user.username}})\n    ...> conn = Strategy.plug(strategy, :request, conn)\n    ...> {_conn, {:ok, nil}} = Plug.Helpers.get_authentication_result(conn)\n\n    ...> {:ok, token} = MagicLink.request_token_for(strategy, user)\n    ...> conn = conn(:get, \"/user/magic_link\", %{\"token\" => token})\n    ...> conn = Strategy.plug(strategy, :sign_in, conn)\n    ...> {_conn, {:ok, signed_in_user}} = Plug.Helpers.get_authentication_result(conn)\n    ...> signed_in_user.id == user.id\n    true\n\nSee the [Magic Link Tutorial](/documentation/tutorial/magic-links.md) for more information.","ref":"AshAuthentication.Strategy.MagicLink.html#module-examples"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.request_token_for/2","doc":"Generate a magic link token for a user.\n\nUsed by `AshAuthentication.Strategy.MagicLink.RequestPreparation`.","ref":"AshAuthentication.Strategy.MagicLink.html#request_token_for/2"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.request_token_for_identity/2","doc":"Generate a magic link token for an identity field.\n\nUsed by `AshAuthentication.Strategy.MagicLink.RequestPreparation`.","ref":"AshAuthentication.Strategy.MagicLink.html#request_token_for_identity/2"},{"type":"type","title":"AshAuthentication.Strategy.MagicLink.t/0","doc":"","ref":"AshAuthentication.Strategy.MagicLink.html#t:t/0"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.transform/2","doc":"","ref":"AshAuthentication.Strategy.MagicLink.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.verify/2","doc":"","ref":"AshAuthentication.Strategy.MagicLink.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2","doc":"Strategy for authenticating using any OAuth 2.0 server as the source of truth.\n\nThis authentication strategy provides registration and sign-in for users using a\nremote [OAuth 2.0](https://oauth.net/2/) server as the source of truth. You\nwill be required to provide either a \"register\" or a \"sign-in\" action depending\non your configuration, which the strategy will attempt to validate for common\nmisconfigurations.\n\nThis strategy wraps the excellent [`assent`](https://hex.pm/packages/assent)\npackage, which provides OAuth 2.0 capabilities.\n\nIn order to use OAuth 2.0 authentication on your resource, it needs to meet\nthe following minimum criteria:\n\n1. Have a primary key.\n2. Provide a strategy-specific action, either register or sign-in.\n3. Provide configuration for OAuth2 destinations, secrets, etc.","ref":"AshAuthentication.Strategy.OAuth2.html"},{"type":"module","title":"Example: - AshAuthentication.Strategy.OAuth2","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n  end\n\n  authentication do\n    strategies do\n      oauth2 :example do\n        client_id \"OAuth Client ID\"\n        redirect_uri \"https://my.app/\"\n        client_secret \"My Super Secret Secret\"\n        site \"https://auth.example.com/\"\n      end\n    end\n  end\nend\n```","ref":"AshAuthentication.Strategy.OAuth2.html#module-example"},{"type":"module","title":"Secrets and runtime configuration - AshAuthentication.Strategy.OAuth2","doc":"In order to use OAuth 2.0 you need to provide a varying number of secrets and\nother configuration which may change based on runtime environment.  The\n`AshAuthentication.Secret` behaviour is provided to accommodate this.  This\nallows you to provide configuration either directly on the resource (ie as a\nstring), as an anonymous function, or as a module.\n\n>","ref":"AshAuthentication.Strategy.OAuth2.html#module-secrets-and-runtime-configuration"},{"type":"module","title":"Warning {: .warning} - AshAuthentication.Strategy.OAuth2","doc":">\n> We **strongly** urge you not to share actual secrets in your code or\n> repository.","ref":"AshAuthentication.Strategy.OAuth2.html#module-warning-warning"},{"type":"module","title":"Examples: - AshAuthentication.Strategy.OAuth2","doc":"Providing configuration as an anonymous function:\n\n```elixir\noauth2 do\n  client_secret fn _path, resource ->\n    Application.fetch_env(:my_app, resource, :oauth2_client_secret)\n  end\nend\n```\n\nProviding configuration as a module:\n\n```elixir\ndefmodule MyApp.Secrets do\n  use AshAuthentication.Secret\n\n  def secret_for([:authentication, :strategies, :example, :client_secret], MyApp.User, _opts), do: Application.fetch_env(:my_app, :oauth2_client_secret)\nend\n\n# and in your strategies:\n\noauth2 :example do\n  client_secret MyApp.Secrets\nend\n```","ref":"AshAuthentication.Strategy.OAuth2.html#module-examples"},{"type":"module","title":"User identities - AshAuthentication.Strategy.OAuth2","doc":"Because your users can be signed in via multiple providers at once, you can\nspecify an `identity_resource` in the DSL configuration which points to a\nseperate Ash resource which has the `AshAuthentication.UserIdentity` extension\npresent. This resource will be used to store details of the providers in use\nby each user and a relationship will be added to the user resource.\n\nSetting the `identity_resource` will cause extra validations to be applied to\nyour resource so that changes are tracked correctly on sign-in or\nregistration.","ref":"AshAuthentication.Strategy.OAuth2.html#module-user-identities"},{"type":"module","title":"Actions - AshAuthentication.Strategy.OAuth2","doc":"When using an OAuth 2.0 provider you need to declare either a \"register\" or\n\"sign-in\" action.  The reason for this is that it's not possible for us to\nknow ahead of time how you want to manage the link between your user resources\nand the \"user info\" provided by the OAuth server.\n\nBoth actions receive the following two arguments:\n\n1. `user_info` - a map with string keys containing the [OpenID Successful\n   UserInfo\n   response](https://openid.net/specs/openid-connect-core-1_0.html#UserInfoResponse).\n   Usually this will be used to populate your email, nickname or other\n   identifying field.\n2. `oauth_tokens` a map with string keys containing the [OpenID Successful\n   Token\n   response](https://openid.net/specs/openid-connect-core-1_0.html#TokenResponse)\n   (or similar).\n\nThe actions themselves can be interacted with directly via the\n`AshAuthentication.Strategy` protocol, but you are more likely to interact\nwith them via the web/plugs.","ref":"AshAuthentication.Strategy.OAuth2.html#module-actions"},{"type":"module","title":"Sign-in - AshAuthentication.Strategy.OAuth2","doc":"The sign-in action is called when a successful OAuth2 callback is received.\nYou should use it to constrain the query to the correct user based on the\narguments provided.\n\nThis action is only needed when the `registration_enabled?` DSL settings is\nset to `false`.","ref":"AshAuthentication.Strategy.OAuth2.html#module-sign-in"},{"type":"module","title":"Registration - AshAuthentication.Strategy.OAuth2","doc":"The register action is a little more complicated than the sign-in action,\nbecause we cannot tell the difference between a new user and a returning user\n(they all use the same OAuth flow).  In order to handle this your register\naction must be defined as an upsert with a configured `upsert_identity` (see\nexample below).","ref":"AshAuthentication.Strategy.OAuth2.html#module-registration"},{"type":"module","title":"Examples: - AshAuthentication.Strategy.OAuth2","doc":"Providing sign-in to users who already exist in the database (and by extension\nrejecting new users):\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n  end\n\n  actions do\n    read :sign_in_with_example do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      prepare AshAuthentication.Strategy.OAuth2.SignInPreparation\n\n      filter expr(email == get_path(^arg(:user_info), [:email]))\n    end\n  end\n\n  authentication do\n    strategies do\n      oauth2 :example do\n        registration_enabled? false\n      end\n    end\n  end\nend\n```\n\nProviding registration or sign-in to all comers:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n  end\n\n  actions do\n    create :register_with_oauth2 do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      upsert? true\n      upsert_identity :email\n\n      change AshAuthentication.GenerateTokenChange\n      change fn changeset, _ctx ->\n        user_info = Ash.Changeset.get_argument(changeset, :user_info)\n\n        changeset\n        |> Ash.Changeset.change_attribute(:email, user_info[\"email\"])\n      end\n    end\n  end\n\n  authentication do\n    strategies do\n      oauth2 :example do\n      end\n    end\n  end\nend\n```","ref":"AshAuthentication.Strategy.OAuth2.html#module-examples"},{"type":"module","title":"Plugs - AshAuthentication.Strategy.OAuth2","doc":"OAuth 2.0 is (usually) a browser-based flow. This means that you're most\nlikely to interact with this strategy via it's plugs.  There are two phases to\nauthentication with OAuth 2.0:\n\n1. The request phase, where the user's browser is redirected to the remote\n   authentication provider for authentication.\n2. The callback phase, where the provider redirects the user back to your app\n   to create a local database record, session, etc.","ref":"AshAuthentication.Strategy.OAuth2.html#module-plugs"},{"type":"type","title":"AshAuthentication.Strategy.OAuth2.secret/0","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#t:secret/0"},{"type":"type","title":"AshAuthentication.Strategy.OAuth2.secret_keyword/0","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#t:secret_keyword/0"},{"type":"type","title":"AshAuthentication.Strategy.OAuth2.secret_list/0","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#t:secret_list/0"},{"type":"type","title":"AshAuthentication.Strategy.OAuth2.t/0","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#t:t/0"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.transform/2","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.verify/2","doc":"","ref":"AshAuthentication.Strategy.OAuth2.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Oidc","doc":"Strategy for authentication using an [OpenID\nConnect](https://openid.net/connect/) compatible server as the source of\ntruth.\n\nThis strategy builds on-top of `AshAuthentication.Strategy.OAuth2` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use OIDC you need to provide the following minimum configuration:\n\n- `client_id` - The client id, required\n- `site` - The OIDC issuer, required\n- `openid_configuration_uri` - The URI for OpenID Provider, optional, defaults\n  to `/.well-known/openid-configuration`\n- `client_authentication_method` - The Client Authentication method to use,\n  optional, defaults to `client_secret_basic`\n- `client_secret` - The client secret, required if\n  `:client_authentication_method` is `:client_secret_basic`,\n  `:client_secret_post`, or `:client_secret_jwt`\n- `openid_configuration` - The OpenID configuration, optional, the\n  configuration will be fetched from `:openid_configuration_uri` if this is\n  not defined\n- `id_token_signed_response_alg` - The `id_token_signed_response_alg`\n  parameter sent by the Client during Registration, defaults to `RS256`\n- `id_token_ttl_seconds` - The number of seconds from `iat` that an ID Token\n  will be considered valid, optional, defaults to nil\n- `nonce` - The nonce to use for authorization request, optional, MUST be\n  session based and unguessable.","ref":"AshAuthentication.Strategy.Oidc.html"},{"type":"module","title":"Nonce - AshAuthentication.Strategy.Oidc","doc":"`nonce` can be set in the provider config. The `nonce` will be returned in the\n`session_params` along with `state`. You can use this to store the value in\nthe current session e.g. a httpOnly session cookie.\n\nA random value generator can look like this:\n\n```elixir\n16\n|> :crypto.strong_rand_bytes()\n|> Base.encode64(padding: false)\n```\n\nAshAuthentication will dynamically generate one for the session if `nonce` is\nset to `true`.","ref":"AshAuthentication.Strategy.Oidc.html#module-nonce"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Oidc","doc":"- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)","ref":"AshAuthentication.Strategy.Oidc.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Oidc.transform/2","doc":"","ref":"AshAuthentication.Strategy.Oidc.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Oidc.verify/2","doc":"","ref":"AshAuthentication.Strategy.Oidc.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Password","doc":"Strategy for authenticating using local resources as the source of truth.\n\nIn order to use password authentication your resource needs to meet the\nfollowing minimum requirements:\n\n1. Have a primary key.\n2. A uniquely constrained identity field (eg `username` or `email`).\n3. A sensitive string field within which to store the hashed password.\n\nThere are other options documented in the DSL.","ref":"AshAuthentication.Strategy.Password.html"},{"type":"module","title":"Example: - AshAuthentication.Strategy.Password","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false\n    attribute :hashed_password, :string, allow_nil?: false, sensitive?: true\n  end\n\n  authentication do\n    strategies do\n      password :password do\n        identity_field :email\n        hashed_password_field :hashed_password\n      end\n    end\n  end\n\n  identities do\n    identity :unique_email, [:email]\n  end\nend\n```","ref":"AshAuthentication.Strategy.Password.html#module-example"},{"type":"module","title":"Actions - AshAuthentication.Strategy.Password","doc":"By default the password strategy will automatically generate the register,\nsign-in, reset-request and reset actions for you, however you're free to\ndefine them yourself.  If you do, then the action will be validated to ensure\nthat all the needed configuration is present.\n\nIf you wish to work with the actions directly from your code you can do so via\nthe `AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.Strategy.Password.html#module-actions"},{"type":"module","title":"Examples: - AshAuthentication.Strategy.Password","doc":"Interacting with the actions directly:\n\n    iex> strategy = Info.strategy!(Example.User, :password)\n    ...> {:ok, marty} = Strategy.action(strategy, :register, %{\"username\" => \"marty\", \"password\" => \"outatime1985\", \"password_confirmation\" => \"outatime1985\"})\n    ...> marty.username |> to_string()\n    \"marty\"\n\n    ...> {:ok, user} = Strategy.action(strategy, :sign_in, %{\"username\" => \"marty\", \"password\" => \"outatime1985\"})\n    ...> user.username |> to_string()\n    \"marty\"","ref":"AshAuthentication.Strategy.Password.html#module-examples"},{"type":"module","title":"Plugs - AshAuthentication.Strategy.Password","doc":"The password strategy provides plug endpoints for all four actions, although\nonly sign-in and register will be reported by `Strategy.routes/1` if the\nstrategy is not configured as resettable.\n\nIf you wish to work with the plugs directly, you can do so via the\n`AshAuthentication.Strategy` protocol.","ref":"AshAuthentication.Strategy.Password.html#module-plugs"},{"type":"module","title":"Examples: - AshAuthentication.Strategy.Password","doc":"Dispatching to plugs directly:\n\n    iex> strategy = Info.strategy!(Example.User, :password)\n    ...> conn = conn(:post, \"/user/password/register\", %{\"user\" => %{\"username\" => \"marty\", \"password\" => \"outatime1985\", \"password_confirmation\" => \"outatime1985\"}})\n    ...> conn = Strategy.plug(strategy, :register, conn)\n    ...> {_conn, {:ok, marty}} = Plug.Helpers.get_authentication_result(conn)\n    ...> marty.username |> to_string()\n    \"marty\"\n\n    ...> conn = conn(:post, \"/user/password/reset_request\", %{\"user\" => %{\"username\" => \"marty\"}})\n    ...> conn = Strategy.plug(strategy, :reset_request, conn)\n    ...> {_conn, :ok} = Plug.Helpers.get_authentication_result(conn)","ref":"AshAuthentication.Strategy.Password.html#module-examples"},{"type":"module","title":"Testing - AshAuthentication.Strategy.Password","doc":"See the [Testing guide](/documentation/topics/testing.md) for tips on testing resources using this strategy.","ref":"AshAuthentication.Strategy.Password.html#module-testing"},{"type":"function","title":"AshAuthentication.Strategy.Password.reset_token_for/2","doc":"Generate a reset token for a user.\n\nUsed by `AshAuthentication.Strategy.Password.RequestPasswordResetPreparation`.","ref":"AshAuthentication.Strategy.Password.html#reset_token_for/2"},{"type":"type","title":"AshAuthentication.Strategy.Password.t/0","doc":"","ref":"AshAuthentication.Strategy.Password.html#t:t/0"},{"type":"function","title":"AshAuthentication.Strategy.Password.transform/2","doc":"","ref":"AshAuthentication.Strategy.Password.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Password.verify/2","doc":"","ref":"AshAuthentication.Strategy.Password.html#verify/2"},{"type":"module","title":"AshAuthentication.Strategy.Slack","doc":"Strategy for authenticating using [Slack](https://slack.com)\n\nThis strategy builds on-top of `AshAuthentication.Strategy.Oidc` and\n[`assent`](https://hex.pm/packages/assent).\n\nIn order to use GitHub you need to provide the following minimum configuration:\n\n  - `client_id`\n  - `redirect_uri`\n  - `client_secret`","ref":"AshAuthentication.Strategy.Slack.html"},{"type":"module","title":"More documentation: - AshAuthentication.Strategy.Slack","doc":"- The [Slack Tutorial](/documentation/tutorial/slack.md).\n- The [OIDC documentation](`AshAuthentication.Strategy.Oidc`)","ref":"AshAuthentication.Strategy.Slack.html#module-more-documentation"},{"type":"function","title":"AshAuthentication.Strategy.Slack.transform/2","doc":"","ref":"AshAuthentication.Strategy.Slack.html#transform/2"},{"type":"function","title":"AshAuthentication.Strategy.Slack.verify/2","doc":"","ref":"AshAuthentication.Strategy.Slack.html#verify/2"},{"type":"module","title":"AshAuthentication.BcryptProvider","doc":"Provides the default implementation of `AshAuthentication.HashProvider` using `Bcrypt`.","ref":"AshAuthentication.BcryptProvider.html"},{"type":"function","title":"AshAuthentication.BcryptProvider.hash/1","doc":"Given some user input as a string, convert it into it's hashed form using `Bcrypt`.","ref":"AshAuthentication.BcryptProvider.html#hash/1"},{"type":"function","title":"Example - AshAuthentication.BcryptProvider.hash/1","doc":"iex> {:ok, hashed} = hash(\"Marty McFly\")\n    ...> String.starts_with?(hashed, \"$2b$04$\")\n    true","ref":"AshAuthentication.BcryptProvider.html#hash/1-example"},{"type":"function","title":"AshAuthentication.BcryptProvider.simulate/0","doc":"Simulate a password check to help avoid timing attacks.","ref":"AshAuthentication.BcryptProvider.html#simulate/0"},{"type":"function","title":"Example - AshAuthentication.BcryptProvider.simulate/0","doc":"iex> simulate()\n    false","ref":"AshAuthentication.BcryptProvider.html#simulate/0-example"},{"type":"function","title":"AshAuthentication.BcryptProvider.valid?/2","doc":"Check if the user input matches the hash.","ref":"AshAuthentication.BcryptProvider.html#valid?/2"},{"type":"function","title":"Example - AshAuthentication.BcryptProvider.valid?/2","doc":"iex> valid?(\"Marty McFly\", \"$2b$04$qgacrnrAJz8aPwaVQiGJn.PvryldV.NfOSYYvF/CZAGgMvvzhIE7S\")\n    true","ref":"AshAuthentication.BcryptProvider.html#valid?/2-example"},{"type":"behaviour","title":"AshAuthentication.HashProvider","doc":"A behaviour providing password hashing.","ref":"AshAuthentication.HashProvider.html"},{"type":"callback","title":"AshAuthentication.HashProvider.hash/1","doc":"Given some user input as a string, convert it into it's hashed form.","ref":"AshAuthentication.HashProvider.html#c:hash/1"},{"type":"callback","title":"AshAuthentication.HashProvider.simulate/0","doc":"Attempt to defeat timing attacks by simulating a password hash check.\n\nSee [Bcrypt.no_user_verify/1](https://hexdocs.pm/bcrypt_elixir/Bcrypt.html#no_user_verify/1) for more information.","ref":"AshAuthentication.HashProvider.html#c:simulate/0"},{"type":"callback","title":"AshAuthentication.HashProvider.valid?/2","doc":"Check if the user input matches the hash.","ref":"AshAuthentication.HashProvider.html#c:valid?/2"},{"type":"module","title":"AshAuthentication.Jwt","doc":"Uses the excellent `joken` hex package to generate and sign Json Web Tokens.","ref":"AshAuthentication.Jwt.html"},{"type":"module","title":"Configuration - AshAuthentication.Jwt","doc":"There are a few things we need to know in order to generate and sign a JWT:\n\n  * `signing_algorithm` - the crypographic algorithm used to to sign tokens.\n  * `token_lifetime` - how long the token is valid for (in hours).\n  * `signing_secret` - the secret key used to sign the tokens.\n\nThese can be configured in your resource's token DSL:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n\n  authentication do\n    tokens do\n      token_lifetime 32\n      signing_secret fn _, _ ->\n        System.fetch_env(\"TOKEN_SIGNING_SECRET\")\n      end\n    end\n  end\n\n  # ...\nend\n```\n\nThe signing secret is retrieved using the `AshAuthentication.Secret`\nbehaviour, which means that it can be retrieved one of three ways:\n\n1. As a string directly in your resource DSL (please don't do this unless you\n   know why this is a bad idea!), or\n2. a two-arity anonymous function which returns `{:ok, secret}`, or\n3. the name of a module which implements the `AshAuthentication.Secret`\n   behaviour.\n\nAvailable signing algorithms are EdDSA, Ed448ph, Ed448, Ed25519ph, Ed25519, PS512, PS384, PS256, ES512, ES384, ES256, RS512, RS384, RS256, HS512, HS384 or HS256.  Defaults to HS256.\n\nWe strongly advise against storing the signing secret in your mix config or\ndirectly in your resource configuration.  We instead suggest you make use of\n[`runtime.exs`](https://elixir-lang.org/getting-started/mix-otp/config-and-releases.html#configuration)\nand read it from the system environment or other secret store.\n\nThe default token lifetime is 168 and should be\nspecified in integer positive hours.","ref":"AshAuthentication.Jwt.html#module-configuration"},{"type":"type","title":"AshAuthentication.Jwt.claims/0","doc":"\"claims\" are the decoded contents of a JWT.  A map of (short) string keys to\nstring values.","ref":"AshAuthentication.Jwt.html#t:claims/0"},{"type":"function","title":"AshAuthentication.Jwt.default_algorithm/0","doc":"The default signing algorithm","ref":"AshAuthentication.Jwt.html#default_algorithm/0"},{"type":"function","title":"AshAuthentication.Jwt.default_lifetime_hrs/0","doc":"The default token lifetime","ref":"AshAuthentication.Jwt.html#default_lifetime_hrs/0"},{"type":"function","title":"AshAuthentication.Jwt.peek/1","doc":"Given a token, read it's claims without validating.","ref":"AshAuthentication.Jwt.html#peek/1"},{"type":"function","title":"AshAuthentication.Jwt.supported_algorithms/0","doc":"Supported signing algorithms","ref":"AshAuthentication.Jwt.html#supported_algorithms/0"},{"type":"type","title":"AshAuthentication.Jwt.token/0","doc":"A string likely to contain a valid JWT.","ref":"AshAuthentication.Jwt.html#t:token/0"},{"type":"function","title":"AshAuthentication.Jwt.token_for_resource/3","doc":"Given a resource, generate a signed JWT with a set of claims.","ref":"AshAuthentication.Jwt.html#token_for_resource/3"},{"type":"function","title":"AshAuthentication.Jwt.token_for_user/3","doc":"Given a user, generate a signed JWT for use while authenticating.","ref":"AshAuthentication.Jwt.html#token_for_user/3"},{"type":"function","title":"AshAuthentication.Jwt.token_to_resource/2","doc":"Given a token, find a matching resource configuration.","ref":"AshAuthentication.Jwt.html#token_to_resource/2"},{"type":"function","title":"Warning - AshAuthentication.Jwt.token_to_resource/2","doc":"This function *does not* validate the token, so don't rely on it for\nauthentication or authorisation.","ref":"AshAuthentication.Jwt.html#token_to_resource/2-warning"},{"type":"function","title":"AshAuthentication.Jwt.verify/2","doc":"Given a token, verify it's signature and validate it's claims.","ref":"AshAuthentication.Jwt.html#verify/2"},{"type":"module","title":"AshAuthentication.Info","doc":"Generated configuration functions based on a resource's DSL configuration.","ref":"AshAuthentication.Info.html"},{"type":"function","title":"AshAuthentication.Info.authentication_add_ons/1","doc":"authentication.add_ons DSL entities","ref":"AshAuthentication.Info.html#authentication_add_ons/1"},{"type":"function","title":"AshAuthentication.Info.authentication_domain/1","doc":"The name of the Ash domain to use to access this resource when doing anything authentication related.","ref":"AshAuthentication.Info.html#authentication_domain/1"},{"type":"function","title":"AshAuthentication.Info.authentication_domain!/1","doc":"The name of the Ash domain to use to access this resource when doing anything authentication related.","ref":"AshAuthentication.Info.html#authentication_domain!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_get_by_subject_action_name/1","doc":"The name of the read action used to retrieve records. If the action doesn't exist, one will be generated for you.","ref":"AshAuthentication.Info.html#authentication_get_by_subject_action_name/1"},{"type":"function","title":"AshAuthentication.Info.authentication_get_by_subject_action_name!/1","doc":"The name of the read action used to retrieve records. If the action doesn't exist, one will be generated for you.","ref":"AshAuthentication.Info.html#authentication_get_by_subject_action_name!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_options/1","doc":"authentication DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.Info.html#authentication_options/1"},{"type":"function","title":"AshAuthentication.Info.authentication_select_for_senders/1","doc":"A list of fields that we will ensure are selected whenever a sender will be invoked.  Defaults to `[:email]` if there is an `:email` attribute on the resource, and `[]` otherwise.","ref":"AshAuthentication.Info.html#authentication_select_for_senders/1"},{"type":"function","title":"AshAuthentication.Info.authentication_select_for_senders!/1","doc":"A list of fields that we will ensure are selected whenever a sender will be invoked.  Defaults to `[:email]` if there is an `:email` attribute on the resource, and `[]` otherwise.","ref":"AshAuthentication.Info.html#authentication_select_for_senders!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_strategies/1","doc":"authentication.strategies DSL entities","ref":"AshAuthentication.Info.html#authentication_strategies/1"},{"type":"function","title":"AshAuthentication.Info.authentication_subject_name/1","doc":"The subject name is used anywhere that a short version of your resource name is needed.  Must be unique system-wide and will be inferred from the resource name by default (ie `MyApp.Accounts.User` -> `user`).","ref":"AshAuthentication.Info.html#authentication_subject_name/1"},{"type":"function","title":"AshAuthentication.Info.authentication_subject_name!/1","doc":"The subject name is used anywhere that a short version of your resource name is needed.  Must be unique system-wide and will be inferred from the resource name by default (ie `MyApp.Accounts.User` -> `user`).","ref":"AshAuthentication.Info.html#authentication_subject_name!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_enabled?/1","doc":"Should JWTs be generated by this resource?","ref":"AshAuthentication.Info.html#authentication_tokens_enabled?/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_options/1","doc":"authentication.tokens DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.Info.html#authentication_tokens_options/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_require_token_presence_for_authentication?/1","doc":"Require a locally-stored token for authentication. See the [tokens guide](/documentation/topics/tokens.md) for more.","ref":"AshAuthentication.Info.html#authentication_tokens_require_token_presence_for_authentication?/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_signing_algorithm/1","doc":"The algorithm to use for token signing. Available signing algorithms are; EdDSA, Ed448ph, Ed448, Ed25519ph, Ed25519, PS512, PS384, PS256, ES512, ES384, ES256, RS512, RS384, RS256, HS512, HS384 and HS256.","ref":"AshAuthentication.Info.html#authentication_tokens_signing_algorithm/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_signing_algorithm!/1","doc":"The algorithm to use for token signing. Available signing algorithms are; EdDSA, Ed448ph, Ed448, Ed25519ph, Ed25519, PS512, PS384, PS256, ES512, ES384, ES256, RS512, RS384, RS256, HS512, HS384 and HS256.","ref":"AshAuthentication.Info.html#authentication_tokens_signing_algorithm!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_signing_secret/1","doc":"The secret used to sign tokens.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"AshAuthentication.Info.html#authentication_tokens_signing_secret/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_signing_secret!/1","doc":"The secret used to sign tokens.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"AshAuthentication.Info.html#authentication_tokens_signing_secret!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_store_all_tokens?/1","doc":"Store all tokens in the `token_resource`. See the [tokens guide](/documentation/topics/tokens.md) for more.","ref":"AshAuthentication.Info.html#authentication_tokens_store_all_tokens?/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_token_lifetime/1","doc":"How long a token should be valid. See [the tokens guide](/documentation/topics/tokens.md) for more.","ref":"AshAuthentication.Info.html#authentication_tokens_token_lifetime/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_token_lifetime!/1","doc":"How long a token should be valid. See [the tokens guide](/documentation/topics/tokens.md) for more.","ref":"AshAuthentication.Info.html#authentication_tokens_token_lifetime!/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_token_resource/1","doc":"The resource used to store token information, such as in-flight confirmations, revocations, and if `store_all_tokens?` is enabled, authentication tokens themselves.","ref":"AshAuthentication.Info.html#authentication_tokens_token_resource/1"},{"type":"function","title":"AshAuthentication.Info.authentication_tokens_token_resource!/1","doc":"The resource used to store token information, such as in-flight confirmations, revocations, and if `store_all_tokens?` is enabled, authentication tokens themselves.","ref":"AshAuthentication.Info.html#authentication_tokens_token_resource!/1"},{"type":"function","title":"AshAuthentication.Info.domain/1","doc":"Retrieve the domain to use for authentication.\n\nIf the `authentication.domain` DSL option is set, it will be used, otherwise\nit will default to that configured on the resource.","ref":"AshAuthentication.Info.html#domain/1"},{"type":"function","title":"AshAuthentication.Info.domain!/1","doc":"Raising version of `domain/1`","ref":"AshAuthentication.Info.html#domain!/1"},{"type":"type","title":"AshAuthentication.Info.dsl_or_resource/0","doc":"","ref":"AshAuthentication.Info.html#t:dsl_or_resource/0"},{"type":"function","title":"AshAuthentication.Info.find_strategy/3","doc":"Find the underlying strategy that required a change/preparation to be used.\n\nThis is because the `strategy_name` can be passed on the change options, eg:\n\n```elixir\nchange {AshAuthentication.Strategy.Password.HashPasswordChange, strategy_name: :banana_custard}\n```\n\nOr via the action context, eg:\n\n```elixir\nprepare set_context(%{strategy_name: :banana_custard})\nprepare AshAuthentication.Strategy.Password.SignInPreparation\n```\n\nOr via the passed-in context on calling the action.","ref":"AshAuthentication.Info.html#find_strategy/3"},{"type":"function","title":"AshAuthentication.Info.strategy/2","doc":"Retrieve a named strategy from a resource.","ref":"AshAuthentication.Info.html#strategy/2"},{"type":"function","title":"AshAuthentication.Info.strategy!/2","doc":"Retrieve a named strategy from a resource (raising version).","ref":"AshAuthentication.Info.html#strategy!/2"},{"type":"function","title":"AshAuthentication.Info.strategy_for_action/2","doc":"Given an action name, retrieve the strategy it is for from the DSL\nconfiguration.","ref":"AshAuthentication.Info.html#strategy_for_action/2"},{"type":"function","title":"AshAuthentication.Info.strategy_for_action!/2","doc":"Given an action name, retrieve the strategy it is for from the DSL\nconfiguration.","ref":"AshAuthentication.Info.html#strategy_for_action!/2"},{"type":"module","title":"AshAuthentication.TokenResource.Info","doc":"Introspection functions for the `AshAuthentication.TokenResource` Ash\nextension.","ref":"AshAuthentication.TokenResource.Info.html"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_confirmation_get_changes_action_name/1","doc":"The name of the action used to get confirmation changes.","ref":"AshAuthentication.TokenResource.Info.html#token_confirmation_get_changes_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_confirmation_get_changes_action_name!/1","doc":"The name of the action used to get confirmation changes.","ref":"AshAuthentication.TokenResource.Info.html#token_confirmation_get_changes_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_confirmation_options/1","doc":"token.confirmation DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.TokenResource.Info.html#token_confirmation_options/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_confirmation_store_changes_action_name/1","doc":"The name of the action used to store confirmation changes.","ref":"AshAuthentication.TokenResource.Info.html#token_confirmation_store_changes_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_confirmation_store_changes_action_name!/1","doc":"The name of the action used to store confirmation changes.","ref":"AshAuthentication.TokenResource.Info.html#token_confirmation_store_changes_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_domain/1","doc":"The Ash domain to use to access this resource.","ref":"AshAuthentication.TokenResource.Info.html#token_domain/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_domain!/1","doc":"The Ash domain to use to access this resource.","ref":"AshAuthentication.TokenResource.Info.html#token_domain!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_expunge_expired_action_name/1","doc":"The name of the action used to remove expired tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_expunge_expired_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_expunge_expired_action_name!/1","doc":"The name of the action used to remove expired tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_expunge_expired_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_expunge_interval/1","doc":"How often to scan this resource for records which have expired, and thus can be removed.","ref":"AshAuthentication.TokenResource.Info.html#token_expunge_interval/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_expunge_interval!/1","doc":"How often to scan this resource for records which have expired, and thus can be removed.","ref":"AshAuthentication.TokenResource.Info.html#token_expunge_interval!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_get_token_action_name/1","doc":"The name of the action used to retrieve tokens from the store, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"AshAuthentication.TokenResource.Info.html#token_get_token_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_get_token_action_name!/1","doc":"The name of the action used to retrieve tokens from the store, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"AshAuthentication.TokenResource.Info.html#token_get_token_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_options/1","doc":"token DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.TokenResource.Info.html#token_options/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_read_expired_action_name/1","doc":"The name of the action use to find all expired tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_read_expired_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_read_expired_action_name!/1","doc":"The name of the action use to find all expired tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_read_expired_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_revocation_is_revoked_action_name/1","doc":"The name of the action used to check if a token is revoked.","ref":"AshAuthentication.TokenResource.Info.html#token_revocation_is_revoked_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_revocation_is_revoked_action_name!/1","doc":"The name of the action used to check if a token is revoked.","ref":"AshAuthentication.TokenResource.Info.html#token_revocation_is_revoked_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_revocation_options/1","doc":"token.revocation DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.TokenResource.Info.html#token_revocation_options/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_revocation_revoke_token_action_name/1","doc":"The name of the action used to revoke tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_revocation_revoke_token_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_revocation_revoke_token_action_name!/1","doc":"The name of the action used to revoke tokens.","ref":"AshAuthentication.TokenResource.Info.html#token_revocation_revoke_token_action_name!/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_store_token_action_name/1","doc":"The name of the action to use to store a token, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"AshAuthentication.TokenResource.Info.html#token_store_token_action_name/1"},{"type":"function","title":"AshAuthentication.TokenResource.Info.token_store_token_action_name!/1","doc":"The name of the action to use to store a token, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"AshAuthentication.TokenResource.Info.html#token_store_token_action_name!/1"},{"type":"module","title":"AshAuthentication.UserIdentity.Info","doc":"Introspection functions for the `AshAuthentication.UserIdentity` Ash\nextension.","ref":"AshAuthentication.UserIdentity.Info.html"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_access_token_attribute_name/1","doc":"The name of the `access_token` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_access_token_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_access_token_attribute_name!/1","doc":"The name of the `access_token` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_access_token_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_access_token_expires_at_attribute_name/1","doc":"The name of the `access_token_expires_at` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_access_token_expires_at_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_access_token_expires_at_attribute_name!/1","doc":"The name of the `access_token_expires_at` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_access_token_expires_at_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_destroy_action_name/1","doc":"The name of the action used to destroy records.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_destroy_action_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_destroy_action_name!/1","doc":"The name of the action used to destroy records.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_destroy_action_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_domain/1","doc":"The Ash domain to use to access this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_domain/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_domain!/1","doc":"The Ash domain to use to access this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_domain!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_options/1","doc":"user_identity DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_options/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_read_action_name/1","doc":"The name of the action used to query identities.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_read_action_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_read_action_name!/1","doc":"The name of the action used to query identities.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_read_action_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_refresh_token_attribute_name/1","doc":"The name of the `refresh_token` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_refresh_token_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_refresh_token_attribute_name!/1","doc":"The name of the `refresh_token` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_refresh_token_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_strategy_attribute_name/1","doc":"The name of the `strategy` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_strategy_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_strategy_attribute_name!/1","doc":"The name of the `strategy` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_strategy_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_uid_attribute_name/1","doc":"The name of the `uid` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_uid_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_uid_attribute_name!/1","doc":"The name of the `uid` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_uid_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_upsert_action_name/1","doc":"The name of the action used to create and update records.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_upsert_action_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_upsert_action_name!/1","doc":"The name of the action used to create and update records.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_upsert_action_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_id_attribute_name/1","doc":"The name of the `user_id` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_id_attribute_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_id_attribute_name!/1","doc":"The name of the `user_id` attribute on this resource.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_id_attribute_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_relationship_name/1","doc":"The name of the belongs-to relationship between identities and users.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_relationship_name/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_relationship_name!/1","doc":"The name of the belongs-to relationship between identities and users.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_relationship_name!/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_resource/1","doc":"The user resource to which these identities belong.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_resource/1"},{"type":"function","title":"AshAuthentication.UserIdentity.Info.user_identity_user_resource!/1","doc":"The user resource to which these identities belong.","ref":"AshAuthentication.UserIdentity.Info.html#user_identity_user_resource!/1"},{"type":"module","title":"AshAuthentication.Debug","doc":"Allows you to debug authentication failures in development.\n\nSimply add `config :ash_authentication, debug_authentication_failures?: true`\nto your `dev.exs` and get fancy log messages when authentication fails.","ref":"AshAuthentication.Debug.html"},{"type":"function","title":"AshAuthentication.Debug.enabled?/0","doc":"Has authentication debug logging been enabled?","ref":"AshAuthentication.Debug.html#enabled?/0"},{"type":"behaviour","title":"AshAuthentication.Secret","doc":"A module to implement retrieving of secrets.\n\nAllows you to implement secrets access via your method or choice at runtime.","ref":"AshAuthentication.Secret.html"},{"type":"behaviour","title":"Example - AshAuthentication.Secret","doc":"```elixir\ndefmodule MyApp.GetSecret do\n  use AshAuthentication.Secret\n\n  def secret_for([:authentication, :strategies, :oauth2, :client_id], MyApp.User, _opts), do: Application.fetch_env(:my_app, :oauth_client_id)\n  def secret_for([:authentication, :strategies, :oauth2, :client_secret], MyApp.User, _opts), do: Application.fetch_env(:my_app, :oauth_client_secret)\nend\n\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      oauth2 do\n        client_id MyApp.GetSecret\n        client_secret MyApp.GetSecret\n      end\n    end\n  end\nend\n```\n\nYou can also implement it directly as a function:\n\n```elixir\ndefmodule MyApp.User do\n   use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      oauth2 do\n        client_id fn _secret, _resource ->\n          Application.fetch_env(:my_app, :oauth_client_id)\n        end\n      end\n    end\n  end\nend\n```","ref":"AshAuthentication.Secret.html#module-example"},{"type":"behaviour","title":"Secret name - AshAuthentication.Secret","doc":"Because you may wish to reuse this module for a number of different providers\nand resources, the first argument passed to the callback is the \"secret name\",\nit contains the \"path\" to the option being set.  The path is made up of a list\ncontaining the DSL path to the secret.","ref":"AshAuthentication.Secret.html#module-secret-name"},{"type":"callback","title":"AshAuthentication.Secret.secret_for/3","doc":"Secret retrieval callback.\n\nThis function will be called with the \"secret name\", see the module\ndocumentation for more info.","ref":"AshAuthentication.Secret.html#c:secret_for/3"},{"type":"behaviour","title":"AshAuthentication.Sender","doc":"A module to implement sending of a token to a user.\n\nAllows you to glue sending of instructions to\n[swoosh](https://hex.pm/packages/swoosh),\n[ex_twilio](https://hex.pm/packages/ex_twilio) or whatever notification system\nis appropriate for your application.\n\nNote that the return value and any failures are ignored.  If you need retry\nlogic, etc, then you should implement it in your sending system.","ref":"AshAuthentication.Sender.html"},{"type":"behaviour","title":"Example - AshAuthentication.Sender","doc":"Implementing as a module:\n\n```elixir\ndefmodule MyApp.PasswordResetSender do\n  use AshAuthentication.Sender\n  import Swoosh.Email\n\n  def send(user, reset_token, _opts) do\n    new()\n    |> to({user.name, user.email})\n    |> from({\"Doc Brown\", \"emmet@brown.inc\"})\n    |> subject(\"Password reset instructions\")\n    |> html_body(\"\"\"\n       Password reset instructions \n       \n        Hi #{user.name}, \n\n        Someone (maybe you) has requested a password reset for your account.\n        If you did not initiate this request then please ignore this email.\n       \n       \n        Click here to reset\n       \n    \"\"\")\n    |> MyApp.Mailer.deliver()\n  end\nend\n\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      password :password do\n        resettable do\n          sender MyApp.PasswordResetSender\n        end\n      end\n    end\n  end\nend\n```\n\nYou can also implement it directly as a function:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      password :password do\n        resettable do\n          sender fn user, token ->\n            MyApp.Mailer.send_password_reset_email(user, token)\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"AshAuthentication.Sender.html#module-example"},{"type":"callback","title":"AshAuthentication.Sender.send/3","doc":"Sending callback.\n\nThis function will be called with a value representing a user, the token and any options passed\nto the module in the DSL.","ref":"AshAuthentication.Sender.html#c:send/3"},{"type":"module","title":"AshAuthentication.Supervisor","doc":"Starts and manages any processes required by AshAuthentication.\n\nAdd to your application supervisor:","ref":"AshAuthentication.Supervisor.html"},{"type":"module","title":"Example - AshAuthentication.Supervisor","doc":"```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {AshAuthentication.Supervisor, otp_app: :my_app}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n  end\nend\n```","ref":"AshAuthentication.Supervisor.html#module-example"},{"type":"function","title":"AshAuthentication.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AshAuthentication.Supervisor.html#child_spec/1"},{"type":"behaviour","title":"AshAuthentication.Plug","doc":"Generate an authentication plug.\n\nUse in your app by creating a new module called `AuthPlug` or similar:\n\n```elixir\ndefmodule MyAppWeb.AuthPlug do\n  use AshAuthentication.Plug, otp_app: :my_app\n\n  def handle_success(conn, _activity, user, _token) do\n    conn\n    |> store_in_session(user)\n    |> send_resp(200, \"Welcome back #{user.name}\")\n  end\n\n  def handle_failure(conn, _activity, reason) do\n    conn\n    |> send_resp(401, \"Better luck next time\")\n  end\nend\n```","ref":"AshAuthentication.Plug.html"},{"type":"behaviour","title":"Using in Phoenix - AshAuthentication.Plug","doc":"In your Phoenix router you can add it:\n\n```elixir\nscope \"/auth\" do\n  pipe_through :browser\n  forward \"/\", MyAppWeb.AuthPlug\nend\n```\n\nIn order to load any authenticated users for either web or API users you can add the following to your router:\n\n```elixir\nimport MyAppWeb.AuthPlug\n\npipeline :session_users do\n  plug :load_from_session\nend\n\npipeline :bearer_users do\n  plug :load_from_bearer\nend\n\nscope \"/\", MyAppWeb do\n  pipe_through [:browser, :session_users]\n\n  live \"/\", PageLive, :home\nend\n\nscope \"/api\", MyAppWeb do\n  pipe_through [:api, :bearer_users]\n\n  get \"/\" ApiController, :index\nend\n```","ref":"AshAuthentication.Plug.html#module-using-in-phoenix"},{"type":"behaviour","title":"Using in a Plug application - AshAuthentication.Plug","doc":"```elixir\nuse Plug.Router\n\nforward \"/auth\", to: MyAppWeb.AuthPlug\n```\n\nNote that you will need to include a bunch of other plugs in the pipeline to\ndo useful things like session and query param fetching.","ref":"AshAuthentication.Plug.html#module-using-in-a-plug-application"},{"type":"type","title":"AshAuthentication.Plug.activity/0","doc":"","ref":"AshAuthentication.Plug.html#t:activity/0"},{"type":"callback","title":"AshAuthentication.Plug.handle_failure/3","doc":"When there is any failure during authentication this callback is called.\n\nNote that this includes not just authentication failures but potentially\nroute-not-found errors also.\n\nThe default implementation simply returns a 401 status with the message\n\"Access denied\".  You almost definitely want to override this.","ref":"AshAuthentication.Plug.html#c:handle_failure/3"},{"type":"callback","title":"AshAuthentication.Plug.handle_success/4","doc":"When authentication has been succesful, this callback will be called with the\nconn, the successful activity, the authenticated resource and a token.\n\nThis allows you to choose what action to take as appropriate for your\napplication.\n\nThe default implementation calls `store_in_session/2` and returns a simple\n\"Access granted\" message to the user.  You almost definitely want to override\nthis behaviour.","ref":"AshAuthentication.Plug.html#c:handle_success/4"},{"type":"type","title":"AshAuthentication.Plug.token/0","doc":"","ref":"AshAuthentication.Plug.html#t:token/0"},{"type":"module","title":"AshAuthentication.Plug.Helpers","doc":"Authentication helpers for use in your router, etc.","ref":"AshAuthentication.Plug.Helpers.html"},{"type":"function","title":"AshAuthentication.Plug.Helpers.get_authentication_result/1","doc":"","ref":"AshAuthentication.Plug.Helpers.html#get_authentication_result/1"},{"type":"function","title":"AshAuthentication.Plug.Helpers.load_subjects/3","doc":"Given a list of subjects, turn as many as possible into users.\n\nOpts are forwarded to `AshAuthentication.subject_to_user/2`","ref":"AshAuthentication.Plug.Helpers.html#load_subjects/3"},{"type":"function","title":"AshAuthentication.Plug.Helpers.retrieve_from_bearer/3","doc":"Validate authorization header(s).\n\nAssumes that your clients are sending a bearer-style authorization header with\nyour request.  If a valid bearer token is present then the subject is loaded\ninto the assigns under their subject name (with the prefix `current_`).\n\nIf the authentication token is required to be present in the database, it is\nloaded into the assigns using `current_#{subject_name}_token_record`\n\nIf there is no user present for a resource then the assign is set to `nil`.","ref":"AshAuthentication.Plug.Helpers.html#retrieve_from_bearer/3"},{"type":"function","title":"AshAuthentication.Plug.Helpers.retrieve_from_session/3","doc":"Attempt to retrieve all users from the connections' session.\n\nIterates through all configured authentication resources for `otp_app` and\nretrieves any users stored in the session, loads them and stores them in the\nassigns under their subject name (with the prefix `current_`).\n\nIf there is no user present for a resource then the assign is set to `nil`.","ref":"AshAuthentication.Plug.Helpers.html#retrieve_from_session/3"},{"type":"function","title":"AshAuthentication.Plug.Helpers.revoke_bearer_tokens/2","doc":"Revoke all authorization header(s).\n\nAny bearer-style authorization headers will have their tokens revoked.","ref":"AshAuthentication.Plug.Helpers.html#revoke_bearer_tokens/2"},{"type":"function","title":"AshAuthentication.Plug.Helpers.set_actor/2","doc":"Set a subject as the request actor.\n\nPresumes that you have already loaded your user resource(s) into the\nconnection's assigns.\n\nUses `Ash.PlugHelpers` to streamline integration with `AshGraphql` and\n`AshJsonApi`.","ref":"AshAuthentication.Plug.Helpers.html#set_actor/2"},{"type":"function","title":"Examples - AshAuthentication.Plug.Helpers.set_actor/2","doc":"Setting the actor for a AshGraphql API using `Plug.Router`.\n\n```elixir\ndefmodule MyApp.ApiRouter do\n  use Plug.Router\n  import MyApp.AuthPlug\n\n  plug :match\n\n  plug :retrieve_from_bearer\n  plug :set_actor, :user\n\n  plug :dispatch\n\n  forward \"/gql\",\n    to: Absinthe.Plug,\n    init_opts: [schema: MyApp.Schema]\nend\n```","ref":"AshAuthentication.Plug.Helpers.html#set_actor/2-examples"},{"type":"function","title":"AshAuthentication.Plug.Helpers.store_authentication_result/2","doc":"Store result in private.\n\nThis is used by authentication plug handlers to store their result for passing\nback to the dispatcher.","ref":"AshAuthentication.Plug.Helpers.html#store_authentication_result/2"},{"type":"function","title":"AshAuthentication.Plug.Helpers.store_in_session/2","doc":"Store the user in the connections' session.","ref":"AshAuthentication.Plug.Helpers.html#store_in_session/2"},{"type":"module","title":"AshAuthentication.Checks.AshAuthenticationInteraction","doc":"This check is true if the context `private.ash_authentication?` is set to true.\n\nThis context will only ever be set in code that is called internally by\n`ash_authentication`, allowing you to create a bypass in your policies on your\nuser/user_token resources.\n\n```elixir\npolicies do\n  bypass AshAuthenticationInteraction do\n    authorize_if always()\n  end\nend\n```","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html"},{"type":"function","title":"AshAuthentication.Checks.AshAuthenticationInteraction.eager_evaluate?/0","doc":"","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html#eager_evaluate?/0"},{"type":"function","title":"AshAuthentication.Checks.AshAuthenticationInteraction.prefer_expanded_description?/0","doc":"","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html#prefer_expanded_description?/0"},{"type":"function","title":"AshAuthentication.Checks.AshAuthenticationInteraction.requires_original_data?/2","doc":"","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html#requires_original_data?/2"},{"type":"function","title":"AshAuthentication.Checks.AshAuthenticationInteraction.strict_check/3","doc":"","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html#strict_check/3"},{"type":"function","title":"AshAuthentication.Checks.AshAuthenticationInteraction.type/0","doc":"","ref":"AshAuthentication.Checks.AshAuthenticationInteraction.html#type/0"},{"type":"module","title":"AshAuthentication.GenerateTokenChange","doc":"Given a successful registration or sign-in, generate a token.","ref":"AshAuthentication.GenerateTokenChange.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.HashPasswordChange","doc":"Set the hash based on the password input.\n\nUses the configured `AshAuthentication.HashProvider` to generate a hash of the\nuser's password input and store it in the changeset.\n\nYou can use this change in your actions where you want to change the user's\npassword.  If you're not using one of the actions generated by the password\nstrategy then you'll need to manually pass the strategy name in the changeset\ncontext.  Eg:\n\n```elixir\nChangeset.new(user, %{})\n|> Changeset.set_context(%{strategy_name: :password})\n|> Changeset.for_update(:update, params)\n|> Accounts.update()\n```\n\nor by adding it statically to your action definition:\n\n```elixir\nupdate :change_password do\n  change set_context(%{strategy_name: :password})\n  change AshAuthentication.Strategy.Password.HashPasswordChange\nend\n```\n\nor by adding it as an option to the change definition:\n\n```elixir\nupdate :change_password do\n  change {AshAuthentication.Strategy.Password.HashPasswordChange, strategy_name: :password}\nend\n```","ref":"AshAuthentication.Strategy.Password.HashPasswordChange.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.PasswordConfirmationValidation","doc":"Validate that the password and password confirmation match.\n\nThis check is only performed when the `confirmation_required?` DSL option is\nset to `true`.\n\nYou can use this validation in your own actions where you want to validate\nthat the password and the password confirmation arguments match.  If you're\nnot using one of the actions generated by the password strategy then you'll\nneed to manually pass the strategy name in the changeset context.  Eg:\n\n```elixir\nChangeset.new(user, %{})\n|> Changeset.set_context(%{strategy_name: :password})\n|> Changeset.for_update(:change_password, params)\n|> Accounts.update()\n```\n\nor by adding it statically in your action definition:\n\n```elixir\nupdate :change_password do\n  change set_context(%{strategy_name: :password})\n  change AshAuthentication.Strategy.Password.HashPasswordChange\nend\n```","ref":"AshAuthentication.Strategy.Password.PasswordConfirmationValidation.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.PasswordConfirmationValidation.validate/3","doc":"Validates that the password and password confirmation fields contain\nequivalent values - if confirmation is required.","ref":"AshAuthentication.Strategy.Password.PasswordConfirmationValidation.html#validate/3"},{"type":"module","title":"AshAuthentication.Strategy.Password.PasswordValidation","doc":"A convenience validation that checks that the password argument against the\nhashed password stored in the record.\n\nYou can use this validation in your changes where you want the user to enter\ntheir current password before being allowed to make a change (eg in a password\nchange flow).","ref":"AshAuthentication.Strategy.Password.PasswordValidation.html"},{"type":"module","title":"Options: - AshAuthentication.Strategy.Password.PasswordValidation","doc":"You can provide these options either in the DSL options, or in the changeset\ncontext.\n\n  - `strategy_name` - the name of the authentication strategy to use.  Required.\n  - `password_argument` - the name of the argument to check for the current\n    password.  If missing this will default to the `password_field` value\n    configured on the strategy.","ref":"AshAuthentication.Strategy.Password.PasswordValidation.html#module-options"},{"type":"module","title":"Examples - AshAuthentication.Strategy.Password.PasswordValidation","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n\n  actions do\n    update :change_password do\n      accept []\n      argument :current_password, :string, sensitive?: true, allow_nil?: false\n      argument :password, :string, sensitive?: true, allow_nil?: false\n      argument :password_confirmation, :string, sensitive?: true, allow_nil?: false\n\n      validate confirm(:password, :password_confirmation)\n      validate {AshAuthentication.Strategy.Password.PasswordValidation, strategy_name: :password, password_argument: :current_password}\n\n      change {AshAuthentication.Strategy.Password.HashPasswordChange, strategy_name: :password}\n    end\n  end\n\n  # ...\nend\n```","ref":"AshAuthentication.Strategy.Password.PasswordValidation.html#module-examples"},{"type":"module","title":"AshAuthentication.Validations","doc":"Common validations shared by several transformers.","ref":"AshAuthentication.Validations.html"},{"type":"function","title":"AshAuthentication.Validations.find_attribute/2","doc":"Find and return a named attribute in the DSL state.","ref":"AshAuthentication.Validations.html#find_attribute/2"},{"type":"function","title":"AshAuthentication.Validations.maybe_build_attribute/3","doc":"Build an attribute if not present.","ref":"AshAuthentication.Validations.html#maybe_build_attribute/3"},{"type":"function","title":"AshAuthentication.Validations.persisted_option/2","doc":"Find and return a persisted option in the DSL state.","ref":"AshAuthentication.Validations.html#persisted_option/2"},{"type":"function","title":"AshAuthentication.Validations.validate_behaviour/2","doc":"Ensure that the named module implements a specific behaviour.","ref":"AshAuthentication.Validations.html#validate_behaviour/2"},{"type":"function","title":"AshAuthentication.Validations.validate_extension/2","doc":"Validates that `extension` is present on the resource.","ref":"AshAuthentication.Validations.html#validate_extension/2"},{"type":"function","title":"AshAuthentication.Validations.validate_field_in_values/3","doc":"Given a map validate that the provided field is one of the values provided.","ref":"AshAuthentication.Validations.html#validate_field_in_values/3"},{"type":"function","title":"AshAuthentication.Validations.validate_field_with/4","doc":"Given a map, validate that the provided field predicate returns true for the value.","ref":"AshAuthentication.Validations.html#validate_field_with/4"},{"type":"function","title":"AshAuthentication.Validations.validate_secret/3","doc":"Validate that a \"secret\" field is configured correctly.","ref":"AshAuthentication.Validations.html#validate_secret/3"},{"type":"function","title":"AshAuthentication.Validations.validate_token_generation_enabled/2","doc":"Ensure that token generation is enabled for the resource.","ref":"AshAuthentication.Validations.html#validate_token_generation_enabled/2"},{"type":"module","title":"AshAuthentication.Validations.Action","doc":"Validation helpers for Resource actions.","ref":"AshAuthentication.Validations.Action.html"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_argument_option/4","doc":"Validate an action's argument has an option set to one of the provided values.","ref":"AshAuthentication.Validations.Action.html#validate_action_argument_option/4"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_exists/2","doc":"Validate that a named action actually exists.","ref":"AshAuthentication.Validations.Action.html#validate_action_exists/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_has_argument/2","doc":"Validate the presence of an argument on an action.","ref":"AshAuthentication.Validations.Action.html#validate_action_has_argument/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_has_change/2","doc":"Validate the presence of the named change module on an action.","ref":"AshAuthentication.Validations.Action.html#validate_action_has_change/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_has_manual/2","doc":"Validate the presence of the named manual module on an action.","ref":"AshAuthentication.Validations.Action.html#validate_action_has_manual/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_has_preparation/2","doc":"Validate the presence of the named preparation module on an action.","ref":"AshAuthentication.Validations.Action.html#validate_action_has_preparation/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_has_validation/2","doc":"Validate the presence of the named validation module on an action.","ref":"AshAuthentication.Validations.Action.html#validate_action_has_validation/2"},{"type":"function","title":"AshAuthentication.Validations.Action.validate_action_option/3","doc":"Validate the action has the provided option.","ref":"AshAuthentication.Validations.Action.html#validate_action_option/3"},{"type":"module","title":"AshAuthentication.Validations.Attribute","doc":"Validation helpers for Resource attributes.","ref":"AshAuthentication.Validations.Attribute.html"},{"type":"function","title":"AshAuthentication.Validations.Attribute.validate_attribute_option/4","doc":"Validate that an option is set correctly on an attribute","ref":"AshAuthentication.Validations.Attribute.html#validate_attribute_option/4"},{"type":"function","title":"AshAuthentication.Validations.Attribute.validate_attribute_unique_constraint/3","doc":"Validate than an attribute has a unique identity applied.","ref":"AshAuthentication.Validations.Attribute.html#validate_attribute_unique_constraint/3"},{"type":"exception","title":"AshAuthentication.Errors.AuthenticationFailed","doc":"A generic, authentication failed error.","ref":"AshAuthentication.Errors.AuthenticationFailed.html"},{"type":"function","title":"AshAuthentication.Errors.AuthenticationFailed.exception/0","doc":"","ref":"AshAuthentication.Errors.AuthenticationFailed.html#exception/0"},{"type":"type","title":"AshAuthentication.Errors.AuthenticationFailed.t/0","doc":"","ref":"AshAuthentication.Errors.AuthenticationFailed.html#t:t/0"},{"type":"exception","title":"AshAuthentication.Errors.CannotConfirmUnconfirmedUser","doc":"An unconfirmed user cannot be confirmed outside of explicit actions.\n\nThis can be allowed by making an action confirm a user by placing it in the `auto_confirm_actions` list.\n\nHowever, it is a security risk to allow unconfirmed users to be confirmed except for via the `confirm` action, invoked with a token.\n\nSee the confirmation tutorial on hexdocs for more.","ref":"AshAuthentication.Errors.CannotConfirmUnconfirmedUser.html"},{"type":"function","title":"AshAuthentication.Errors.CannotConfirmUnconfirmedUser.exception/0","doc":"","ref":"AshAuthentication.Errors.CannotConfirmUnconfirmedUser.html#exception/0"},{"type":"exception","title":"AshAuthentication.Errors.InvalidToken","doc":"An invalid token was presented.","ref":"AshAuthentication.Errors.InvalidToken.html"},{"type":"function","title":"AshAuthentication.Errors.InvalidToken.exception/0","doc":"","ref":"AshAuthentication.Errors.InvalidToken.html#exception/0"},{"type":"exception","title":"AshAuthentication.Errors.MissingSecret","doc":"A secret is now missing.","ref":"AshAuthentication.Errors.MissingSecret.html"},{"type":"function","title":"AshAuthentication.Errors.MissingSecret.exception/0","doc":"","ref":"AshAuthentication.Errors.MissingSecret.html#exception/0"},{"type":"exception","title":"AshAuthentication.Errors.UnconfirmedUser","doc":"The user is unconfirmed and so the operation cannot be executed.","ref":"AshAuthentication.Errors.UnconfirmedUser.html"},{"type":"function","title":"AshAuthentication.Errors.UnconfirmedUser.exception/0","doc":"","ref":"AshAuthentication.Errors.UnconfirmedUser.html#exception/0"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.Actions","doc":"Actions for the confirmation add-on.\n\nProvides the code interface for working with resources via confirmation.","ref":"AshAuthentication.AddOn.Confirmation.Actions.html"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.Actions.confirm/3","doc":"Attempt to confirm a user.","ref":"AshAuthentication.AddOn.Confirmation.Actions.html#confirm/3"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.Actions.get_changes/3","doc":"Get changes from the tokens resource for application.","ref":"AshAuthentication.AddOn.Confirmation.Actions.html#get_changes/3"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.Actions.store_changes/4","doc":"Store changes in the tokens resource for later re-use.","ref":"AshAuthentication.AddOn.Confirmation.Actions.html#store_changes/4"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.ConfirmChange","doc":"Performs a change based on the contents of a confirmation token.","ref":"AshAuthentication.AddOn.Confirmation.ConfirmChange.html"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.ConfirmationHookChange","doc":"Triggers a confirmation flow when one of the monitored fields is changed.\n\nOptionally inhibits changes to monitored fields on update.\n\nYou can use this change in your actions where you want to send the user a\nconfirmation (or inhibit changes after confirmation).  If you're not using one\nof the actions generated by the confirmation add-on then you'll need to\nmanually pass the strategy name in the changeset context. Eg:\n\n```elixir\nChangeset.new(user, %{})\n|> Changeset.set_context(%{strategy_name: :confirm})\n|> Changeset.for_update(:update, params)\n|> Accounts.update()\n```\n\nor by adding it statically to your action definition:\n\n```elixir\nupdate :change_email do\n  change set_context(%{strategy_name: :confirm})\n  change AshAuthentication.AddOn.Confirmation.ConfirmationHookChange\nend\n```\n\nor by adding it as an option to the change definition:\n\n```elixir\nupdate :change_email do\n  change {AshAuthentication.AddOn.Confirmation.ConfirmationHookChange, strategy_name: :confirm}\nend\n```","ref":"AshAuthentication.AddOn.Confirmation.ConfirmationHookChange.html"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.Dsl","doc":"Defines the Spark DSL entity for this add on.","ref":"AshAuthentication.AddOn.Confirmation.Dsl.html"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.Plug","doc":"Handlers for incoming OAuth2 HTTP requests.","ref":"AshAuthentication.AddOn.Confirmation.Plug.html"},{"type":"function","title":"AshAuthentication.AddOn.Confirmation.Plug.confirm/2","doc":"Attempt to perform a confirmation.","ref":"AshAuthentication.AddOn.Confirmation.Plug.html#confirm/2"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.Transformer","doc":"DSL transformer for confirmation add-on.\n\nEnsures that there is only ever one present and that it is correctly\nconfigured.","ref":"AshAuthentication.AddOn.Confirmation.Transformer.html"},{"type":"module","title":"AshAuthentication.AddOn.Confirmation.Verifier","doc":"DSL verifier for confirmation add-on.","ref":"AshAuthentication.AddOn.Confirmation.Verifier.html"},{"type":"module","title":"AshAuthentication.Igniter","doc":"Codemods for working with AshAuthentication","ref":"AshAuthentication.Igniter.html"},{"type":"function","title":"AshAuthentication.Igniter.add_add_on/3","doc":"Adds an add on to the authentication.add_ons section of a resource","ref":"AshAuthentication.Igniter.html#add_add_on/3"},{"type":"function","title":"AshAuthentication.Igniter.add_new_add_on/5","doc":"Adds a new add_on to the authentication.strategies section of a resource","ref":"AshAuthentication.Igniter.html#add_new_add_on/5"},{"type":"function","title":"AshAuthentication.Igniter.add_new_strategy/5","doc":"Adds a new strategy to the authentication.strategies section of a resource","ref":"AshAuthentication.Igniter.html#add_new_strategy/5"},{"type":"function","title":"AshAuthentication.Igniter.add_secret_from_env/5","doc":"Adds a secret to a secret module that reads from application env","ref":"AshAuthentication.Igniter.html#add_secret_from_env/5"},{"type":"function","title":"AshAuthentication.Igniter.add_strategy/3","doc":"Adds a strategy to the authentication.strategies section of a resource","ref":"AshAuthentication.Igniter.html#add_strategy/3"},{"type":"function","title":"AshAuthentication.Igniter.defines_add_on/4","doc":"Returns true if the given resource defines an authentication add on with the provided name","ref":"AshAuthentication.Igniter.html#defines_add_on/4"},{"type":"function","title":"AshAuthentication.Igniter.defines_strategy/4","doc":"Returns true if the given resource defines an authentication strategy with the provided name","ref":"AshAuthentication.Igniter.html#defines_strategy/4"},{"type":"module","title":"AshAuthentication.Jwt.Config","doc":"Implementation details JWT generation and validation.\n\nProvides functions to generate token configuration at runtime, based on the\nresource being signed for and for verifying claims and checking for token\nrevocation.","ref":"AshAuthentication.Jwt.Config.html"},{"type":"function","title":"AshAuthentication.Jwt.Config.default_claims/2","doc":"Generate the default claims for a specified resource.","ref":"AshAuthentication.Jwt.Config.html#default_claims/2"},{"type":"function","title":"AshAuthentication.Jwt.Config.generate_audience/1","doc":"The generator function used to generate the \"aud\" claim.\n\nIt generates an Elixir-style `~>` version requirement against the current\nmajor and minor version numbers of AshAuthentication.","ref":"AshAuthentication.Jwt.Config.html#generate_audience/1"},{"type":"function","title":"AshAuthentication.Jwt.Config.generate_issuer/1","doc":"The generator function used to generate the \"iss\" claim.","ref":"AshAuthentication.Jwt.Config.html#generate_issuer/1"},{"type":"function","title":"AshAuthentication.Jwt.Config.token_signer/2","doc":"The signer used to sign the token on a per-resource basis.","ref":"AshAuthentication.Jwt.Config.html#token_signer/2"},{"type":"function","title":"AshAuthentication.Jwt.Config.validate_audience/4","doc":"The validation function used to validate the \"aud\" claim.\n\nUses `Version.match?/2` to validate the provided claim against the current\nversion.  The use of `~>` means that tokens generated by versions of\nAshAuthentication with the the same major version and at least the same minor\nversion should be compatible.","ref":"AshAuthentication.Jwt.Config.html#validate_audience/4"},{"type":"function","title":"AshAuthentication.Jwt.Config.validate_issuer/3","doc":"The validation function used to validate the \"iss\" claim.\n\nIt simply verifies that the claim starts with `\"AshAuthentication\"`","ref":"AshAuthentication.Jwt.Config.html#validate_issuer/3"},{"type":"function","title":"AshAuthentication.Jwt.Config.validate_jti/4","doc":"The validation function used to the validate the \"jti\" claim.\n\nThis is done by checking that the token is valid with the token revocation\nresource.  Requires that the subject's resource configuration be passed as the\nvalidation context.  This is automatically done by calling `Jwt.verify/2`.","ref":"AshAuthentication.Jwt.Config.html#validate_jti/4"},{"type":"module","title":"AshAuthentication.Plug.Defaults","doc":"Provides the default implementations of `handle_success/3` and\n`handle_failure/2` used in generated authentication plugs.","ref":"AshAuthentication.Plug.Defaults.html"},{"type":"function","title":"AshAuthentication.Plug.Defaults.handle_failure/3","doc":"The default implementation of `handle_failure/1`.\n\nSends a very basic 401 response.","ref":"AshAuthentication.Plug.Defaults.html#handle_failure/3"},{"type":"function","title":"AshAuthentication.Plug.Defaults.handle_success/4","doc":"The default implementation of `handle_success/3`.\n\nCalls `AshAuthentication.Plug.Helpers.store_in_session/2` then sends a\nbasic 200 response.","ref":"AshAuthentication.Plug.Defaults.html#handle_success/4"},{"type":"module","title":"AshAuthentication.Plug.Dispatcher","doc":"Route requests and callbacks to the correct provider plugs.","ref":"AshAuthentication.Plug.Dispatcher.html"},{"type":"function","title":"AshAuthentication.Plug.Dispatcher.call/2","doc":"Send the request to the correct strategy and then return the result.","ref":"AshAuthentication.Plug.Dispatcher.html#call/2"},{"type":"type","title":"AshAuthentication.Plug.Dispatcher.config/0","doc":"","ref":"AshAuthentication.Plug.Dispatcher.html#t:config/0"},{"type":"module","title":"AshAuthentication.Plug.Macros","doc":"Generators used within `use AshAuthentication.Plug`.","ref":"AshAuthentication.Plug.Macros.html"},{"type":"macro","title":"AshAuthentication.Plug.Macros.define_load_from_bearer/1","doc":"Generates the `load_from_bearer/2` plug with the `otp_app` prefilled.","ref":"AshAuthentication.Plug.Macros.html#define_load_from_bearer/1"},{"type":"macro","title":"AshAuthentication.Plug.Macros.define_load_from_session/1","doc":"Generates the `load_from_session/2` plug with the `otp_app` prefilled.","ref":"AshAuthentication.Plug.Macros.html#define_load_from_session/1"},{"type":"macro","title":"AshAuthentication.Plug.Macros.define_revoke_bearer_tokens/1","doc":"Generates the `revoke_bearer_tokens/2` plug with the `otp_app` prefilled.","ref":"AshAuthentication.Plug.Macros.html#define_revoke_bearer_tokens/1"},{"type":"macro","title":"AshAuthentication.Plug.Macros.validate_subject_name_uniqueness/1","doc":"Generates the subject name validation code for the auth plug.","ref":"AshAuthentication.Plug.Macros.html#validate_subject_name_uniqueness/1"},{"type":"module","title":"AshAuthentication.Plug.Router","doc":"Dynamically generates the authentication router for the authentication\nrequests and callbacks.\n\nUsed internally by `AshAuthentication.Plug`.","ref":"AshAuthentication.Plug.Router.html"},{"type":"module","title":"AshAuthentication.Preparations.FilterBySubject","doc":"Filters a user by the identifier in the subject of a JWT.","ref":"AshAuthentication.Preparations.FilterBySubject.html"},{"type":"function","title":"AshAuthentication.Preparations.FilterBySubject.init/1","doc":"","ref":"AshAuthentication.Preparations.FilterBySubject.html#init/1"},{"type":"module","title":"AshAuthentication.SecretFunction","doc":"Implements `AshAuthentication.Secret` for functions that are provided to the\nDSL instead of modules.","ref":"AshAuthentication.SecretFunction.html"},{"type":"module","title":"AshAuthentication.SenderFunction","doc":"Implements `AshAuthentication.Sender` for functions that are provided to the\nDSL instead of modules.","ref":"AshAuthentication.SenderFunction.html"},{"type":"module","title":"AshAuthentication.Strategy.Apple.Verifier","doc":"DSL verifier for Apple strategy.","ref":"AshAuthentication.Strategy.Apple.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.Custom.Helpers","doc":"Helpers for use within custom strategies.","ref":"AshAuthentication.Strategy.Custom.Helpers.html"},{"type":"function","title":"AshAuthentication.Strategy.Custom.Helpers.put_add_on/2","doc":"Update the add-on in the DSL state by name.\n\nThis helper should only be used within transformers.","ref":"AshAuthentication.Strategy.Custom.Helpers.html#put_add_on/2"},{"type":"function","title":"AshAuthentication.Strategy.Custom.Helpers.put_strategy/2","doc":"Update the strategy in the DSL state by name.\n\nThis helper should only be used within transformers.","ref":"AshAuthentication.Strategy.Custom.Helpers.html#put_strategy/2"},{"type":"function","title":"AshAuthentication.Strategy.Custom.Helpers.register_strategy_actions/3","doc":"If there's any chance that an implementor may try and use actions genrated by\nyour strategy programatically then you should register your actions with Ash\nAuthentication so that it can find the appropriate strategy when needed.\n\nThe strategy can be retrieved again by calling\n`AshAuthentication.Info.strategy_for_action/2`.\n\nThis helper should only be used within transformers.","ref":"AshAuthentication.Strategy.Custom.Helpers.html#register_strategy_actions/3"},{"type":"module","title":"AshAuthentication.Strategy.Custom.Transformer","doc":"Transformer used by custom strategies.\n\nIt delegates transformation passes to the individual strategies.","ref":"AshAuthentication.Strategy.Custom.Transformer.html"},{"type":"function","title":"AshAuthentication.Strategy.Custom.Transformer.after_compile?/0","doc":"","ref":"AshAuthentication.Strategy.Custom.Transformer.html#after_compile?/0"},{"type":"module","title":"AshAuthentication.Strategy.Custom.Verifier","doc":"Verifier used by custom strategies.\n\nIt delegates verification passes to the individual strategies.","ref":"AshAuthentication.Strategy.Custom.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.Actions","doc":"Actions for the magic link strategy.\n\nProvides the code interface for working with user resources for providing\nmagic links.","ref":"AshAuthentication.Strategy.MagicLink.Actions.html"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.Actions.request/3","doc":"Request a magic link for a user.","ref":"AshAuthentication.Strategy.MagicLink.Actions.html#request/3"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.Actions.sign_in/3","doc":"Attempt to sign a user in via magic link.","ref":"AshAuthentication.Strategy.MagicLink.Actions.html#sign_in/3"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.Plug","doc":"Plugs for the magic link strategy.\n\nHandles requests and sign-ins.","ref":"AshAuthentication.Strategy.MagicLink.Plug.html"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.Plug.request/2","doc":"Handle a request for a magic link.\n\nRetrieves form parameters from nested within the subject name, eg:\n\n```\n%{\n  \"user\" => %{\n    \"email\" => \"marty@mcfly.me\"\n  }\n}\n```","ref":"AshAuthentication.Strategy.MagicLink.Plug.html#request/2"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.Plug.sign_in/2","doc":"Sign in via magic link.","ref":"AshAuthentication.Strategy.MagicLink.Plug.html#sign_in/2"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.Request","doc":"Requests a magic link for the given identity field.","ref":"AshAuthentication.Strategy.MagicLink.Request.html"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.RequestPreparation","doc":"Prepare a query for a magic link request.\n\nThis preparation performs three jobs, one before the query executes and two\nafter:\n1. it constraints the query to match the identity field passed to the action.\n2. if there is a user returned by the query, then\n  a. generate a magic link token and\n  b. publish a notification.\n\nAlways returns an empty result.","ref":"AshAuthentication.Strategy.MagicLink.RequestPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.RequestPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.MagicLink.RequestPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.SignInChange","doc":"Set up a create action for magic link sign in.","ref":"AshAuthentication.Strategy.MagicLink.SignInChange.html"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.SignInPreparation","doc":"Prepare a query for sign in.","ref":"AshAuthentication.Strategy.MagicLink.SignInPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.MagicLink.SignInPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.MagicLink.SignInPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.Transformer","doc":"DSL transformer for magic links.","ref":"AshAuthentication.Strategy.MagicLink.Transformer.html"},{"type":"module","title":"AshAuthentication.Strategy.MagicLink.Verifier","doc":"DSL verifier for magic links.","ref":"AshAuthentication.Strategy.MagicLink.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.Actions","doc":"Actions for the oauth2 strategy.\n\nProvides the code interface for working with resources via an OAuth2 strategy.","ref":"AshAuthentication.Strategy.OAuth2.Actions.html"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.Actions.register/3","doc":"Attempt to register a new user.","ref":"AshAuthentication.Strategy.OAuth2.Actions.html#register/3"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.Actions.sign_in/3","doc":"Attempt to sign in a user.","ref":"AshAuthentication.Strategy.OAuth2.Actions.html#sign_in/3"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.Dsl","doc":"Defines the Spark DSL entity for this strategy.","ref":"AshAuthentication.Strategy.OAuth2.Dsl.html"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.IdentityChange","doc":"Updates the identity resource when a user is registered.","ref":"AshAuthentication.Strategy.OAuth2.IdentityChange.html"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.Plug","doc":"Handlers for incoming OAuth2 HTTP requests.","ref":"AshAuthentication.Strategy.OAuth2.Plug.html"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.Plug.callback/2","doc":"Perform the callback phase of OAuth2.\n\nResponds to a user being redirected back from the remote authentication\nprovider, and validates the passed options, ultimately registering or\nsigning-in a user if the authentication was successful.","ref":"AshAuthentication.Strategy.OAuth2.Plug.html#callback/2"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.Plug.request/2","doc":"Perform the request phase of OAuth2.\n\nBuilds a redirection URL based on the provider configuration and redirects the\nuser to that endpoint.","ref":"AshAuthentication.Strategy.OAuth2.Plug.html#request/2"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.SignInPreparation","doc":"Prepare a query for sign in\n\nPerforms three main tasks:\n\n  1. Ensures that there is only one matching user record returned, otherwise\n     returns an authentication failed error.\n  2. Generates an access token if token generation is enabled.\n  3. Updates the user identity resource, if one is enabled.","ref":"AshAuthentication.Strategy.OAuth2.SignInPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.OAuth2.SignInPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.OAuth2.SignInPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.Transformer","doc":"DSL transformer for oauth2 strategies.\n\nIterates through any oauth2 strategies and ensures that all the correct\nactions and settings are in place.","ref":"AshAuthentication.Strategy.OAuth2.Transformer.html"},{"type":"module","title":"AshAuthentication.Strategy.OAuth2.Verifier","doc":"DSL verifier for oauth2 strategies.","ref":"AshAuthentication.Strategy.OAuth2.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.Oidc.NonceGenerator","doc":"An implmentation of `AshAuthentication.Secret` that generates nonces for\nOpenID Connect strategies.\n\nDefaults to `16` bytes of random data.  You can change this by setting the\n`byte_size` option in your DSL:\n\n```elixir\noidc do\n  nonce {AshAuthentication.NonceGenerator, byte_size: 32}\n  # ...\nend\n```","ref":"AshAuthentication.Strategy.Oidc.NonceGenerator.html"},{"type":"module","title":"AshAuthentication.Strategy.Oidc.Transformer","doc":"DSL transformer for oidc strategies.\n\nAdds a nonce generator to the strategy if `nonce` is set to `true`.\nDelegates to the default OAuth2 transformer.","ref":"AshAuthentication.Strategy.Oidc.Transformer.html"},{"type":"module","title":"AshAuthentication.Strategy.Oidc.Verifier","doc":"DSL verifier for OpenID Connect strategy.","ref":"AshAuthentication.Strategy.Oidc.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.Actions","doc":"Actions for the password strategy\n\nProvides the code interface for working with resources via a password\nstrategy.","ref":"AshAuthentication.Strategy.Password.Actions.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.Actions.register/3","doc":"Attempt to register a new user.","ref":"AshAuthentication.Strategy.Password.Actions.html#register/3"},{"type":"function","title":"AshAuthentication.Strategy.Password.Actions.reset/3","doc":"Attempt to change a user's password using a reset token.","ref":"AshAuthentication.Strategy.Password.Actions.html#reset/3"},{"type":"function","title":"AshAuthentication.Strategy.Password.Actions.reset_request/3","doc":"Request a password reset.","ref":"AshAuthentication.Strategy.Password.Actions.html#reset_request/3"},{"type":"function","title":"AshAuthentication.Strategy.Password.Actions.sign_in/3","doc":"Attempt to sign in a user.","ref":"AshAuthentication.Strategy.Password.Actions.html#sign_in/3"},{"type":"function","title":"AshAuthentication.Strategy.Password.Actions.sign_in_with_token/3","doc":"Attempt to sign in a previously-authenticated user with a short-lived sign in token.","ref":"AshAuthentication.Strategy.Password.Actions.html#sign_in_with_token/3"},{"type":"module","title":"AshAuthentication.Strategy.Password.Dsl","doc":"Defines the Spark DSL entity for this strategy.","ref":"AshAuthentication.Strategy.Password.Dsl.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.Plug","doc":"Plugs for the password strategy.\n\nHandles registration, sign-in and password resets.","ref":"AshAuthentication.Strategy.Password.Plug.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.Plug.register/2","doc":"Handle a registration request","ref":"AshAuthentication.Strategy.Password.Plug.html#register/2"},{"type":"function","title":"AshAuthentication.Strategy.Password.Plug.reset/2","doc":"Handle a reset request","ref":"AshAuthentication.Strategy.Password.Plug.html#reset/2"},{"type":"function","title":"AshAuthentication.Strategy.Password.Plug.reset_request/2","doc":"Handle a reset request request","ref":"AshAuthentication.Strategy.Password.Plug.html#reset_request/2"},{"type":"function","title":"AshAuthentication.Strategy.Password.Plug.sign_in/2","doc":"Handle a sign-in request","ref":"AshAuthentication.Strategy.Password.Plug.html#sign_in/2"},{"type":"function","title":"AshAuthentication.Strategy.Password.Plug.sign_in_with_token/2","doc":"Handle a request to validate a sign in token","ref":"AshAuthentication.Strategy.Password.Plug.html#sign_in_with_token/2"},{"type":"module","title":"AshAuthentication.Strategy.Password.RequestPasswordReset","doc":"Requests a password reset.\n\nThis implementation performs three jobs:\n1. looks up the user with the given action and field\n2. if a matching user is found:\n  a. a reset token is generated\n  b. and the password reset sender is invoked","ref":"AshAuthentication.Strategy.Password.RequestPasswordReset.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.RequestPasswordResetPreparation","doc":"Prepare a query for a password reset request.\n\nThis preparation performs three jobs, one before the query executes and two\nafter:\n1. it constraints the query to match the identity field passed to the action.\n2. if there is a user returned by the query, then\n  a. generate a reset token and\n  b. publish a notification.\n\nAlways returns an empty result.","ref":"AshAuthentication.Strategy.Password.RequestPasswordResetPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.RequestPasswordResetPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.Password.RequestPasswordResetPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.Strategy.Password.ResetTokenValidation","doc":"Validate that the token is a valid password reset request token.","ref":"AshAuthentication.Strategy.Password.ResetTokenValidation.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.Resettable","doc":"The entity used to store password reset information.","ref":"AshAuthentication.Strategy.Password.Resettable.html"},{"type":"type","title":"AshAuthentication.Strategy.Password.Resettable.t/0","doc":"","ref":"AshAuthentication.Strategy.Password.Resettable.html#t:t/0"},{"type":"module","title":"AshAuthentication.Strategy.Password.SignInPreparation","doc":"Prepare a query for sign in\n\nThis preparation performs two jobs, one before the query executes and one\nafter.\n\nFirstly, it constrains the query to match the identity field passed to the\naction.\n\nSecondly, it validates the supplied password using the configured hash\nprovider, and if correct allows the record to be returned, otherwise returns\nan authentication failed error.","ref":"AshAuthentication.Strategy.Password.SignInPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.SignInPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.Password.SignInPreparation.html#init/1"},{"type":"function","title":"AshAuthentication.Strategy.Password.SignInPreparation.user_confirmed_if_needed/2","doc":"","ref":"AshAuthentication.Strategy.Password.SignInPreparation.html#user_confirmed_if_needed/2"},{"type":"module","title":"AshAuthentication.Strategy.Password.SignInWithTokenPreparation","doc":"Prepare a query for sign in via token.\n\nThis preparation first validates the token argument and extracts the subject\nfrom it and constrains the query to a matching user.","ref":"AshAuthentication.Strategy.Password.SignInWithTokenPreparation.html"},{"type":"function","title":"AshAuthentication.Strategy.Password.SignInWithTokenPreparation.init/1","doc":"","ref":"AshAuthentication.Strategy.Password.SignInWithTokenPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.Strategy.Password.Transformer","doc":"DSL transformer for the password strategy.\n\nIterates through any password authentication strategies and ensures that all\nthe correct actions and settings are in place.","ref":"AshAuthentication.Strategy.Password.Transformer.html"},{"type":"module","title":"AshAuthentication.Strategy.Password.Verifier","doc":"DSL verifier for the password strategy.","ref":"AshAuthentication.Strategy.Password.Verifier.html"},{"type":"module","title":"AshAuthentication.Strategy.Slack.Verifier","doc":"DSL verifier for Slack strategy.","ref":"AshAuthentication.Strategy.Slack.Verifier.html"},{"type":"module","title":"AshAuthentication.TokenResource.Actions","doc":"The code interface for interacting with the token resource.","ref":"AshAuthentication.TokenResource.Actions.html"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.expunge_expired/2","doc":"Remove all expired records.","ref":"AshAuthentication.TokenResource.Actions.html#expunge_expired/2"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.get_token/3","doc":"Retrieve a token by token or JTI optionally filtering by purpose.","ref":"AshAuthentication.TokenResource.Actions.html#get_token/3"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.jti_revoked?/3","doc":"Has the token been revoked?\n\nSimilar to `token-revoked?/2..3` except that rather than extracting the JTI\nfrom the token, assumes that it's being passed in directly.","ref":"AshAuthentication.TokenResource.Actions.html#jti_revoked?/3"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.revoke/3","doc":"Revoke a token.\n\nExtracts the JTI from the provided token and uses it to generate a revocation\nrecord.","ref":"AshAuthentication.TokenResource.Actions.html#revoke/3"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.store_token/3","doc":"Store a token.\n\nStores a token for any purpose.","ref":"AshAuthentication.TokenResource.Actions.html#store_token/3"},{"type":"function","title":"AshAuthentication.TokenResource.Actions.token_revoked?/3","doc":"Has the token been revoked?\n\nSimilar to `jti_revoked?/2..3` except that it extracts the JTI from the token,\nrather than relying on it to be passed in.","ref":"AshAuthentication.TokenResource.Actions.html#token_revoked?/3"},{"type":"module","title":"AshAuthentication.TokenResource.Expunger","doc":"A `GenServer` which periodically removes expired token revocations.\n\nScans all token revocation resources based on their configured expunge\ninterval and removes any expired records.\n\n```elixir\ndefmodule MyApp.Accounts.Token do\n  use Ash.Resource,\n    extensions: [AshAuthentication.TokenResource],\n    domain: MyApp.Accounts\n\n  token do\n    expunge_interval 12\n  end\nend\n```\n\nThis GenServer is started by the `AshAuthentication.Supervisor` which should\nbe added to your app's supervision tree.","ref":"AshAuthentication.TokenResource.Expunger.html"},{"type":"function","title":"AshAuthentication.TokenResource.Expunger.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AshAuthentication.TokenResource.Expunger.html#child_spec/1"},{"type":"module","title":"AshAuthentication.TokenResource.GetConfirmationChangesPreparation","doc":"Constrains a query to only records which are confirmations that match the jti\nargument.","ref":"AshAuthentication.TokenResource.GetConfirmationChangesPreparation.html"},{"type":"function","title":"AshAuthentication.TokenResource.GetConfirmationChangesPreparation.init/1","doc":"","ref":"AshAuthentication.TokenResource.GetConfirmationChangesPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.TokenResource.GetTokenPreparation","doc":"Constrains a query to only records which match the `jti` or `token` argument\nand optionally by the `purpose` argument.","ref":"AshAuthentication.TokenResource.GetTokenPreparation.html"},{"type":"function","title":"AshAuthentication.TokenResource.GetTokenPreparation.init/1","doc":"","ref":"AshAuthentication.TokenResource.GetTokenPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.TokenResource.IsRevoked","doc":"Checks for the existence of a revocation token for the provided tokenrevocation token for the provided token.","ref":"AshAuthentication.TokenResource.IsRevoked.html"},{"type":"module","title":"AshAuthentication.TokenResource.IsRevokedPreparation","doc":"Constrains a query to only records which are revocations that match the token\nor jti argument.","ref":"AshAuthentication.TokenResource.IsRevokedPreparation.html"},{"type":"function","title":"AshAuthentication.TokenResource.IsRevokedPreparation.init/1","doc":"","ref":"AshAuthentication.TokenResource.IsRevokedPreparation.html#init/1"},{"type":"module","title":"AshAuthentication.TokenResource.RevokeTokenChange","doc":"Generates a revocation record for a given token.","ref":"AshAuthentication.TokenResource.RevokeTokenChange.html"},{"type":"module","title":"AshAuthentication.TokenResource.StoreConfirmationChangesChange","doc":"Populates the JTI based on the token argument.","ref":"AshAuthentication.TokenResource.StoreConfirmationChangesChange.html"},{"type":"module","title":"AshAuthentication.TokenResource.StoreTokenChange","doc":"Stores an arbitrary token.","ref":"AshAuthentication.TokenResource.StoreTokenChange.html"},{"type":"module","title":"AshAuthentication.TokenResource.Transformer","doc":"The token resource transformer.\n\nSets up the default schema and actions for the token resource.","ref":"AshAuthentication.TokenResource.Transformer.html"},{"type":"function","title":"AshAuthentication.TokenResource.Transformer.after_compile?/0","doc":"","ref":"AshAuthentication.TokenResource.Transformer.html#after_compile?/0"},{"type":"module","title":"AshAuthentication.TokenResource.Verifier","doc":"The token resource verifier.","ref":"AshAuthentication.TokenResource.Verifier.html"},{"type":"module","title":"AshAuthentication.Transformer","doc":"The Authentication transformer\n\nSets up non-provider-specific configuration for authenticated resources.","ref":"AshAuthentication.Transformer.html"},{"type":"function","title":"AshAuthentication.Transformer.after_compile?/0","doc":"","ref":"AshAuthentication.Transformer.html#after_compile?/0"},{"type":"module","title":"AshAuthentication.Transformer.SetSelectForSenders","doc":"Sets the `select_for_senders` options to its default value.","ref":"AshAuthentication.Transformer.SetSelectForSenders.html"},{"type":"function","title":"AshAuthentication.Transformer.SetSelectForSenders.after_compile?/0","doc":"","ref":"AshAuthentication.Transformer.SetSelectForSenders.html#after_compile?/0"},{"type":"function","title":"AshAuthentication.Transformer.SetSelectForSenders.before?/1","doc":"","ref":"AshAuthentication.Transformer.SetSelectForSenders.html#before?/1"},{"type":"module","title":"AshAuthentication.UserIdentity.Actions","doc":"Code interface for provider identity actions.\n\nAllows you to interact with UserIdentity resources without having to mess\naround with changesets, domains, etc.  These functions are delegated to from\nwithin `AshAuthentication.UserIdentity`.","ref":"AshAuthentication.UserIdentity.Actions.html"},{"type":"function","title":"AshAuthentication.UserIdentity.Actions.upsert/2","doc":"Upsert an identity for a user.","ref":"AshAuthentication.UserIdentity.Actions.html#upsert/2"},{"type":"module","title":"AshAuthentication.UserIdentity.Transformer","doc":"The user identity transformer.\n\nSets up the default schema and actions for a user identity resource.","ref":"AshAuthentication.UserIdentity.Transformer.html"},{"type":"function","title":"AshAuthentication.UserIdentity.Transformer.after_compile?/0","doc":"","ref":"AshAuthentication.UserIdentity.Transformer.html#after_compile?/0"},{"type":"module","title":"AshAuthentication.UserIdentity.UpsertIdentityChange","doc":"A change which upserts a user's identity into the user identity resource.\n\nExpects the following arguments:\n\n  - `user_info` a map with string keys as returned from the OAuth2/OpenID\n    upstream provider.\n  - `oauth_tokens` a map with string keys containing the OAuth2 token\n    response.\n  - `user_id` the ID of the user this identity relates to.\n  - `strategy` the name of the strategy.\n\nThis is usually dynamically inserted into a generated action, however you can\nadd it to your own action if needed.","ref":"AshAuthentication.UserIdentity.UpsertIdentityChange.html"},{"type":"module","title":"AshAuthentication.UserIdentity.Verifier","doc":"The user identity verifier.","ref":"AshAuthentication.UserIdentity.Verifier.html"},{"type":"module","title":"AshAuthentication.Verifier","doc":"The Authentication verifier.\n\nChecks configuration constraints after compile.","ref":"AshAuthentication.Verifier.html"},{"type":"task","title":"mix ash_authentication.add_strategy","doc":"Adds the provided strategy or strategies to your user resource\n\nThis task will add the provided strategy or strategies to your user resource.\n\nThe following strategies are available. For all others, see the relevant documentation for setup\n\n  * `password` - Register and sign in with a username/email and a password.\n  * `magic_link` - Register and sign in with a magic link, sent via email to the user.","ref":"Mix.Tasks.AshAuthentication.AddStrategy.html"},{"type":"task","title":"Example - mix ash_authentication.add_strategy","doc":"```bash\nmix ash_authentication.add_strategy password\n```","ref":"Mix.Tasks.AshAuthentication.AddStrategy.html#module-example"},{"type":"task","title":"Options - mix ash_authentication.add_strategy","doc":"* `--user`, `-u` -  The user resource. Defaults to `YourApp.Accounts.User`\n* `--identity-field`, `-i` - The field on the user resource that will be used to identify\n  the user. Defaults to `email`","ref":"Mix.Tasks.AshAuthentication.AddStrategy.html#module-options"},{"type":"function","title":"Mix.Tasks.AshAuthentication.AddStrategy.igniter/2","doc":"","ref":"Mix.Tasks.AshAuthentication.AddStrategy.html#igniter/2"},{"type":"task","title":"mix ash_authentication.install","doc":"Installs AshAuthentication. Invoke with `mix igniter.install ash_authentication`","ref":"Mix.Tasks.AshAuthentication.Install.html"},{"type":"task","title":"Example - mix ash_authentication.install","doc":"```bash\nmix igniter.install ash_authentication\n```","ref":"Mix.Tasks.AshAuthentication.Install.html#module-example"},{"type":"task","title":"Options - mix ash_authentication.install","doc":"* `--accounts` or `-a` - The domain that contains your resources. Defaults to `YourApp.Accounts`.\n* `--user` or `-u` - The resource that represents a user. Defaults to ` .User`.\n* `--token` or `-t` - The resource that represents a token. Defaults to ` .Token`.\n* `--auth-strategy` - The strategy or strategies to use for authentication.\n  None by default, can be specified multiple times for more than one strategy.\n  To add after installation, use `mix ash_authentication.add_strategy password`","ref":"Mix.Tasks.AshAuthentication.Install.html#module-options"},{"type":"function","title":"Mix.Tasks.AshAuthentication.Install.data_layer_args/2","doc":"","ref":"Mix.Tasks.AshAuthentication.Install.html#data_layer_args/2"},{"type":"function","title":"Mix.Tasks.AshAuthentication.Install.setup_data_layer/2","doc":"","ref":"Mix.Tasks.AshAuthentication.Install.html#setup_data_layer/2"},{"type":"extras","title":"README","doc":"![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-black-text.png?raw=true#gh-light-mode-only)\n![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-white-text.png?raw=true#gh-dark-mode-only)\n\n![Elixir CI](https://github.com/team-alembic/ash_authentication/workflows/Elixir%20Library/badge.svg)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Hex version badge](https://img.shields.io/hexpm/v/ash_authentication.svg)](https://hex.pm/packages/ash_authentication)\n[![Hexdocs badge](https://img.shields.io/badge/docs-hexdocs-purple)](https://hexdocs.pm/ash_authentication)\n\n# Ash Authentication\n\nWelcome! Here you will find everything you need to know to get started with and use Ash Authentication. This documentation is best viewed on [hexdocs](https://hexdocs.pm/ash_authentication).","ref":"readme.html"},{"type":"extras","title":"About the Documentation - README","doc":"[**Tutorials**](#tutorials) walk you through a series of steps to accomplish a goal. These are **learning-oriented**, and are a great place for beginners to start.\n\n---\n\n[**Topics**](#topics) provide a high level overview of a specific concept or feature. These are **understanding-oriented**, and are perfect for discovering design patterns, features, and tools related to a given topic.\n\n---\n\n[**Reference**](#reference) documentation is produced automatically from our source code. It comes in the form of module documentation and DSL documentation. This documentation is **information-oriented**. Use the sidebar and the search bar to find relevant reference information.","ref":"readme.html#about-the-documentation"},{"type":"extras","title":"Tutorials - README","doc":"- [Get Started](documentation/tutorials/get-started.md)\n\n---","ref":"readme.html#tutorials"},{"type":"extras","title":"Topics - README","doc":"- [Custom Strategies](documentation/topics/custom-strategy.md)\n- [Policies on Authenticated Resources](documentation/topics/policies-on-authentication-resources.md)\n- [Testing](documentation/topics/testing.md)\n- [Tokens](documentation/topics/tokens.md)\n- [Upgrade guides](documentation/topics/upgrading.md)\n\n---","ref":"readme.html#topics"},{"type":"extras","title":"Tutorials - README","doc":"- [Authenticate with Auth0](documentation/tutorials/auth0.md)\n- [Authenticate with GitHub](documentation/tutorials/github.md)\n- [Authenticate with Google](documentation/tutorials/google.md)\n- [Authenticate with Magic Links](documentation/tutorials/magic-links.md)\n- [Confirmation](documentation/tutorials/confirmation.md)\n\n---","ref":"readme.html#tutorials"},{"type":"extras","title":"Reference - README","doc":"- [AshAuthentication DSL](documentation/dsls/DSL-AshAuthentication.md)\n- [AshAuthentication.AddOn.Confirmation DSL](documentation/dsls/DSL-AshAuthentication.AddOn.Confirmation.md))\n- [AshAuthentication.Strategy.Auth0](documentation/dsls/DSL-AshAuthentication.Strategy.Auth0.md)\n- [AshAuthentication.Strategy.Github DSL](documentation/dsls/DSL-AshAuthentication.Strategy.Github.md)\n- [AshAuthentication.Strategy.Google DSL](documentation/dsls/DSL-AshAuthentication.Strategy.Google.md)\n- [AshAuthentication.Strategy.MagicLink DSL](documentation/dsls/DSL-AshAuthentication.Strategy.MagicLink.md)\n- [AshAuthentication.Strategy.OAuth2 DSL](documentation/dsls/DSL-AshAuthentication.Strategy.OAuth2.md)\n- [AshAuthentication.Strategy.Oidc DSL](documentation/dsls/DSL-AshAuthentication.Strategy.Oidc.md)\n- [AshAuthentication.Strategy.Password DSL](documentation/dsls/DSL-AshAuthentication.Strategy.Password.md)\n- [AshAuthentication.TokenResource DSL](documentation/dsls/DSL-AshAuthentication.TokenResource.md)\n- [AshAuthentication.UserIdentity DSL](documentation/dsls/DSL-AshAuthentication.UserIdentity.md)\n- For other reference documentation, see the sidebar & search bar","ref":"readme.html#reference"},{"type":"extras","title":"Related packages - README","doc":"- [Ash Framework](https://hexdocs.pm/ash)\n- [Ash Authentication Phoenix](https://hexdocs.pm/ash_authentication_phoenix) | Integrates Ash Authentication into your Phoenix application\n\n---\n\n[![Alembic](logos/alembic.svg)](https://alembic.com.au)\n\nProudly written and maintained by the team at [Alembic](https://alembic.com.au) for the Ash community.","ref":"readme.html#related-packages"},{"type":"extras","title":"Change Log","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [v4.4.8](https://github.com/team-alembic/ash_authentication/compare/v4.4.7...v4.4.8) (2025-02-04)","ref":"changelog.html"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* fix marking hashed_password as `allow_nil?` in magic link installer","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* Allow authorization params to be defined using secret module (#900)\n\n## [v4.4.7](https://github.com/team-alembic/ash_authentication/compare/v4.4.6...v4.4.7) (2025-02-02)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* downgrade assent\n\n* OIDC: Not retrieving remote OIDC configuration.\n\n## [v4.4.6](https://github.com/team-alembic/ash_authentication/compare/v4.4.5...v4.4.6) (2025-02-01)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* OIDC: Not retrieving remote OIDC configuration.\n\n## [v4.4.5](https://github.com/team-alembic/ash_authentication/compare/v4.4.4...v4.4.5) (2025-01-27)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* Add support for OAuth2 Code Verifier (#896)\n\n## [v4.4.4](https://github.com/team-alembic/ash_authentication/compare/v4.4.3...v4.4.4) (2025-01-23)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"* make `hashed_password` optional if magic_link is also used\n\n## [v4.4.3](https://github.com/team-alembic/ash_authentication/compare/v4.4.2...v4.4.3) (2025-01-23)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* downgrade assent and upgrade markdown files\n\n## [v4.4.2](https://github.com/team-alembic/ash_authentication/compare/v4.4.1...v4.4.2) (2025-01-22)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* Format code and update cheat sheets (both part of currently failing build)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* support sqlite in the installer\n\n* Make the links in generated emails, clickable links (#890)\n\n## [v4.4.1](https://github.com/team-alembic/ash_authentication/compare/v4.4.0...v4.4.1) (2025-01-16)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* without phoenix, don't use verified routes\n\n## [v4.4.0](https://github.com/team-alembic/ash_authentication/compare/v4.3.12...v4.4.0) (2025-01-16)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"* add email sender igniters for swoosh (#835)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* properly parse multiple authentication strategies\n\n## [v4.3.12](https://github.com/team-alembic/ash_authentication/compare/v4.3.11...v4.3.12) (2025-01-14)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* Fixed a link in the docs and pinned `Assent` to `0.2` (#884)\n\n## [v4.3.11](https://github.com/team-alembic/ash_authentication/compare/v4.3.10...v4.3.11) (2025-01-13)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* fix google strategy dependency was requiring options it should not\n\n* fixed `:sign_in_with_token` that was logging in user automatically even if confirmation is required and account is not confirmed (#875)\n\n* don't pass argv through to resource generator\n\n* convert UID to string when setting (#870)\n\n* Fix converting tenant to string (#868)\n\n* wrong Enum.concat in validate_attribute_unique_constraint (#869)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* Removed use of `Assent.Config` (#877)\n\n## [v4.3.10](https://github.com/team-alembic/ash_authentication/compare/v4.3.9...v4.3.10) (2025-01-02)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* generate change_password action with `require_atomic? false`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* make unconfirmed user error like other errors\n\n## [v4.3.9](https://github.com/team-alembic/ash_authentication/compare/v4.3.8...v4.3.9) (2024-12-31)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* move change_password action to password strategy setup\n\n## [v4.3.8](https://github.com/team-alembic/ash_authentication/compare/v4.3.7...v4.3.8) (2024-12-31)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* add `require_confirmed_with` option to password strategy  (#861)\n\n## [v4.3.7](https://github.com/team-alembic/ash_authentication/compare/v4.3.6...v4.3.7) (2024-12-26)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* return an `AuthenticationFailed` error properly\n\n* go back to generating the repo if its unavailable\n\n* don't generate repo if its not present","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* use ets if postgres isn't available\n\n* add `change_password` action to generated code\n\n* use better action name for password reset\n\n## [v4.3.6](https://github.com/team-alembic/ash_authentication/compare/v4.3.5...v4.3.6) (2024-12-20)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"* make igniter optional\n\n* store all tokens by default in generators\n\n## [v4.3.5](https://github.com/team-alembic/ash_authentication/compare/v4.3.4...v4.3.5) (2024-12-12)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* ensure that `auto_confirm_actions` does not override\n\n* include tenant when checking identity conflicts\n\n* handle tokens without a primary key encoded in sub, using `identity`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* allow setting identity field to allow nil with password\n\n## [v4.3.4](https://github.com/team-alembic/ash_authentication/compare/v4.3.3...v4.3.4) (2024-12-02)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* ensure tenant is passed through on password reset request\n\n* invalidate magic link sign in on invalid token\n\n* pass opts to confirm action invocations","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* add primary read action to users resource for atomic upgrade\n\n* auto confirm on sign in with magic link in generators\n\n* Add opts to retrieve funs of AshAuthentication.Plug.Helpers (#847)\n\n## [v4.3.3](https://github.com/team-alembic/ash_authentication/compare/v4.3.2...v4.3.3) (2024-11-14)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* Use correct typespec for `AshAuthentication.Sender.send/3` callback (#836)\n\n## [v4.3.2](https://github.com/team-alembic/ash_authentication/compare/v4.3.1...v4.3.2) (2024-11-13)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* The documentation says that we ignore sender returns, so we need to ignore them. (#838)\n\n## [v4.3.1](https://github.com/team-alembic/ash_authentication/compare/v4.3.0...v4.3.1) (2024-11-12)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* RequestPasswordReset: fails when action called directly. (#833)\n\n* ash_authentication.add_strategy: Generated password reset action names did not match the defaults. (#834)\n\n* confirmation warning 'changeset has already been validated for action'\n\n## [v4.3.0](https://github.com/team-alembic/ash_authentication/compare/v4.2.7...v4.3.0) (2024-11-05)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"* Strategy.Slack: Add direct support for Slack strategy. (#825)\n\n* Strategy.Slack: Add direct support for Slack strategy.","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* handle igniter/rewrite upgrades\n\n* set sign_in_with_token action name properly\n\n## [v4.2.7](https://github.com/team-alembic/ash_authentication/compare/v4.2.6...v4.2.7) (2024-11-01)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* change_attribute -> force_* to eliminate waring\n\n## [v4.2.6](https://github.com/team-alembic/ash_authentication/compare/v4.2.5...v4.2.6) (2024-10-31)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* run codegen after adding an auth strategy\n\n## [v4.2.5](https://github.com/team-alembic/ash_authentication/compare/v4.2.4...v4.2.5) (2024-10-23)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* proper error instead of match error on not found user\n\n## [v4.2.4](https://github.com/team-alembic/ash_authentication/compare/v4.2.3...v4.2.4) (2024-10-23)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* generate link using `confirm` instead of `token` in the generators\n\n## [v4.2.3](https://github.com/team-alembic/ash_authentication/compare/v4.2.2...v4.2.3) (2024-10-19)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* respond to `--auth-strategy` option in installer\n\n* issues with OIDC strategy (#800)\n\n## [v4.2.2](https://github.com/team-alembic/ash_authentication/compare/v4.2.1...v4.2.2) (2024-10-15)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* support registration via magic link (#796)\n\n* support registration via magic link\n\n* prevent account takeover hijacking by protecting against upserts against unconfirmed records\n\n* add confirmation add on when identity_field is email\n\n* implement our own identity checking instead of relying on eager_check\n\n## [v4.2.1](https://github.com/team-alembic/ash_authentication/compare/v4.2.0...v4.2.1) (2024-10-14)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"* update igniter\n\n## [v4.2.0](https://github.com/team-alembic/ash_authentication/compare/v4.1.0...v4.2.0) (2024-10-07)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"* add_strategy task (#794)","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"* add `ash_authentication.add_strategy` task\n\n* add atomic implementations for various changes/validations\n\n* support `--auth-strategy` option when installing\n\n## [v4.1.0](https://github.com/team-alembic/ash_authentication/compare/v4.0.4...v4.1.0) (2024-10-06)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"* Add AshAuthentication igniter installer (#782)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* handle options properly for subect to user (#786)\n\n* setup options properly for ash 3.0 (#785)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* igniter installer for user & user token resources\n\n## [v4.0.4](https://github.com/team-alembic/ash_authentication/compare/v4.0.3...v4.0.4) (2024-09-01)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* update types and formatter\n\n* add secret values to config\n\n* sort new fields\n\n* sort new types\n\n* properly set allow_nil for apple secrets\n\n* credo and sobelow warnings","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* add apple strategy (#750)\n\n* add apple strategy\n\n## [v4.0.3](https://github.com/team-alembic/ash_authentication/compare/v4.0.2...v4.0.3) (2024-08-22)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* allow overriding strategy defaults (#766)\n\n* bug where `nil` is not allowed but is returned from secret functions.\n\n* add back in accidentally removed debug errors code (#768)\n\n* set options earlier in magic link/oauth2","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* avoid warning about comparison with `nil`\n\n* set context in addition to tenant\n\n* use `no_depend_modules` for better compile dependencies\n\n* enable custom `http_adapters` (#760)\n\n## [v4.0.2](https://github.com/team-alembic/ash_authentication/compare/v4.0.1...v4.0.2) (2024-08-05)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* only pass the \"token\" parameter to reset with token action (#748)\n\n* handle case where `action.accept` is `nil`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* validate that tokens are enabled when password resets are enabled. (#758)\n\n* compile-time check to make sure that the configured `token_resource` is an Ash.Resource (#749)\n\n* Tokens: improved compile-time validation of the token_resource option of the tokens DSL by checking that the passed value is an Ash.Resource.\n\n* Tokens: removed unnecessary stuff from the test file.\n\n* Tokens: fixed credo warning and changed some things after PR feedback\n\n## [v4.0.1](https://github.com/team-alembic/ash_authentication/compare/v4.0.0...v4.0.1) (2024-06-11)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* no need to `allow_nil_input` for an unaccepted field\n\n* correctly generate sign-in tokens when requested.\n\n* ensure tenant is set when revoking tokens and on changeset for updating\n\n* broken links in readme (#692)\n\n* broken links\n\n* bug in tokens required verifier.\n\n## [4.0.0](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.7...4.0.0) (2024-05-10)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"* Sign in tokens are enabled by default for the password strategy.\n\n* Tokens are now enabled by default.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* Jwt: Include authentication interaction context when storing tokens.\n\n* Strategy.Password: Reset tokens are single use. (#625)\n\n* Confirmation: Only allow the confirmation token to be used once. (#623)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* Only require tokens to be enabled when using a strategy which needs them.\n\n* OIDC: Adjust dsl of OIDC reflect assent requirements (#538)\n\n* Use `Ash` functions instead of generated domain functions.\n\n## [v4.0.0-rc.7](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.6...v4.0.0-rc.7) (2024-05-10)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* Jwt: Include authentication interaction context when storing tokens.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"* Only require tokens to be enabled when using a strategy which needs them.\n\n## [v4.0.0-rc.6](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.5...v4.0.0-rc.6) (2024-04-11)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"* OIDC: Adjust dsl of OIDC reflect assent requirements (#538)\n\n## [v4.0.0-rc.5](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.4...v4.0.0-rc.5) (2024-04-10)","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Sign in tokens are enabled by default for the password strategy.\n\n- Tokens are now enabled by default.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Strategy.Password: Reset tokens are single use. (#625)\n\n## [v4.0.0-rc.4](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.3...v4.0.0-rc.4) (2024-04-09)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Use `Ash` functions instead of generated domain functions.\n\n## [v4.0.0-rc.3](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.2...v4.0.0-rc.3) (2024-04-08)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Confirmation: Only allow the confirmation token to be used once. (#623)\n\n## [v4.0.0-rc.2](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.1...v4.0.0-rc.2) (2024-04-02)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Update to support Ash 3.0. (#599)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- allow future versions of ash rc\n\n- Jwt: Ignore pre-release versions verifying token versions.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- re-integrate ash_graphql and ash_json_api RCs.\n\n## [v4.0.0-rc.1](https://github.com/team-alembic/ash_authentication/compare/v4.0.0-rc.0...v4.0.0-rc.1) (2024-04-01)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- re-integrate ash_graphql and ash_json_api RCs.\n\n## [v4.0.0-rc.0](https://github.com/team-alembic/ash_authentication/compare/v3.12.4...v4.0.0-rc.0) (2024-03-28)","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Update to support Ash 3.0. (#599)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Jwt: Ignore pre-release versions verifying token versions.\n\n## [v3.12.4](https://github.com/team-alembic/ash_authentication/compare/v3.12.3...v3.12.4) (2024-03-11)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- infer `api` from a resource\n\n## [v3.12.3](https://github.com/team-alembic/ash_authentication/compare/v3.12.2...v3.12.3) (2024-02-20)\n\n## [v3.12.2](https://github.com/team-alembic/ash_authentication/compare/v3.12.1...v3.12.2) (2024-01-30)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- deps: mark ash_postgres as optional","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- support atom keys for uid in addition to strings (#556)\n\n## [v3.12.1](https://github.com/team-alembic/ash_authentication/compare/v3.12.0...v3.12.1) (2024-01-25)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- support atom keys for uid in addition to strings (#556)\n\n## [v3.12.0](https://github.com/team-alembic/ash_authentication/compare/v3.11.16...v3.12.0) (2023-11-21)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- Add Google strategy (#474)\n\n- Add Google strategy","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- include Google strategy cheat sheet\n\n- Add documentation grouping for Google strategy","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Change redirect_uri secret to be more flexible (#473)\n\n## [v3.11.16](https://github.com/team-alembic/ash_authentication/compare/v3.11.15...v3.11.16) (2023-10-25)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Change overwriting of refresh_token to not overwrite them with nil (#483)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Add id as an option for sourcing uid for UserIdentity (#481)\n\n## [v3.11.15](https://github.com/team-alembic/ash_authentication/compare/v3.11.14...v3.11.15) (2023-09-22)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- ensure we aren't calling `Map.take` on `nil`\n\n## [v3.11.14](https://github.com/team-alembic/ash_authentication/compare/v3.11.13...v3.11.14) (2023-09-22)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- TokenResource: don't silently drop notifications about token removal. (#432)\n\n## [v3.11.13](https://github.com/team-alembic/ash_authentication/compare/v3.11.12...v3.11.13) (2023-09-22)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Allow all token lifetimes to be specified with a time unit.\n\n## [v3.11.12](https://github.com/team-alembic/ash_authentication/compare/v3.11.11...v3.11.12) (2023-09-21)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- include `finch` in the dependencies.\n\n- deprecated mint httpadapter (#425)\n\n## [v3.11.11](https://github.com/team-alembic/ash_authentication/compare/v3.11.10...v3.11.11) (2023-09-21)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- include `finch` in the dependencies.\n\n- deprecated mint httpadapter (#425)\n\n## [v3.11.10](https://github.com/team-alembic/ash_authentication/compare/v3.11.9...v3.11.10) (2023-09-18)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- only use sign in token expiration for sign in tokens (#424)\n\n## [v3.11.9](https://github.com/team-alembic/ash_authentication/compare/v3.11.8...v3.11.9) (2023-09-17)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- support generating tokens for other strategies.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- support generating sign in tokens on register (#421)\n\n- support generating sign in tokens on register\n\n## [v3.11.8](https://github.com/team-alembic/ash_authentication/compare/v3.11.7...v3.11.8) (2023-08-16)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- correct spec for `Jwt.token_for_user` (#389)\n\n## [v3.11.7](https://github.com/team-alembic/ash_authentication/compare/v3.11.6...v3.11.7) (2023-07-14)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- ensure that the `current_` atom exists at compile time. (#359)\n\n## [v3.11.6](https://github.com/team-alembic/ash_authentication/compare/v3.11.5...v3.11.6) (2023-06-23)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- fix Logger deprecations for elixir 1.15 (#343)\n\n## [v3.11.5](https://github.com/team-alembic/ash_authentication/compare/v3.11.4...v3.11.5) (2023-06-18)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- ConfirmationHookChange: use `Info.find_strategy/2..3` rather than a hard coded strategy name. (#336)\n\n## [v3.11.4](https://github.com/team-alembic/ash_authentication/compare/v3.11.3...v3.11.4) (2023-06-15)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- primary keys are implicitly uniquely constrained. (#333)\n\n## [v3.11.3](https://github.com/team-alembic/ash_authentication/compare/v3.11.2...v3.11.3) (2023-05-31)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- duplicate mime type for \"json\".\n\n## [v3.11.2](https://github.com/team-alembic/ash_authentication/compare/v3.11.1...v3.11.2) (2023-05-28)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Strategy.Password: Preparations should allow strategy to be passed in. (#314)\n\n## [v3.11.1](https://github.com/team-alembic/ash_authentication/compare/v3.11.0...v3.11.1) (2023-05-04)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- correct oauth2 and getting started typos (#267)\n\n## [v3.11.0](https://github.com/team-alembic/ash_authentication/compare/v3.10.8...v3.11.0) (2023-05-04)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"- OpenID Connect Strategy (#197)\n\n- AshAuthentication.Strategy.Oidc: Add OpenID Connect strategy.\n\n## [v3.10.8](https://github.com/team-alembic/ash_authentication/compare/v3.10.7...v3.10.8) (2023-04-28)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- PasswordValidation should associate errors with the field being â¦ (#279)\n\n## [v3.10.7](https://github.com/team-alembic/ash_authentication/compare/v3.10.6...v3.10.7) (2023-04-28)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- run CI on pull requests\n\n## [v3.10.6](https://github.com/team-alembic/ash_authentication/compare/v3.10.5...v3.10.6) (2023-04-09)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- require spark ~> 1.0 (#261)\n\n## [v3.10.5](https://github.com/team-alembic/ash_authentication/compare/v3.10.4...v3.10.5) (2023-04-06)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add sign in tokens to password strategy (#252)\n\n- add sign in tokens to password strategy\n\n- convert `sign_in_with_token` into an action.\n\n## [v3.10.4](https://github.com/team-alembic/ash_authentication/compare/v3.10.3...v3.10.4) (2023-04-03)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- update spark (#254)\n\n- update spark\n\n## [v3.10.3](https://github.com/team-alembic/ash_authentication/compare/v3.10.2...v3.10.3) (2023-04-03)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- update spark (#254)\n\n- update spark\n\n## [v3.10.2](https://github.com/team-alembic/ash_authentication/compare/v3.10.1...v3.10.2) (2023-03-06)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- respect `identity_relationship_user_id_attribute` on `Strategy.OAuth2.IdentityChange` (#213)\n\n## [v3.10.1](https://github.com/team-alembic/ash_authentication/compare/v3.10.0...v3.10.1) (2023-03-06)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- fix failing JWT tests because of bad version regex.\n\n## [v3.10.0](https://github.com/team-alembic/ash_authentication/compare/v3.9.6...v3.10.0) (2023-03-04)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Configure accepted fields on register (#219)\n\n## [v3.9.6](https://github.com/team-alembic/ash_authentication/compare/v3.9.5...v3.9.6) (2023-03-01)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- allow registration and sign in to be disabled on password strategies. (#218)\n\n## [v3.9.5](https://github.com/team-alembic/ash_authentication/compare/v3.9.4...v3.9.5) (2023-02-23)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- support multiple otp apps w/resources (#209)\n\n## [v3.9.4](https://github.com/team-alembic/ash_authentication/compare/v3.9.3...v3.9.4) (2023-02-22)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- PasswordConfirmationValidation: allow `strategy_name` to be passed as an option. (#208)\n\n## [v3.9.3](https://github.com/team-alembic/ash_authentication/compare/v3.9.2...v3.9.3) (2023-02-19)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- sign in preparation without identity resource (#198)\n\n## [v3.9.2](https://github.com/team-alembic/ash_authentication/compare/v3.9.1...v3.9.2) (2023-02-12)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Password.Transformer: don't force users to define a `hashed_password` argument to the register action. (#192)\n\n## [v3.9.1](https://github.com/team-alembic/ash_authentication/compare/v3.9.0...v3.9.1) (2023-02-12)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- select `hashed_password` on sign in preparation\n\n- don't allow special purpose tokens to be used for sign in. (#191)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add select_for_senders (#189)\n\n- add select_for_senders\n\n- include metadata declaration on register action\n\n## [v3.9.0](https://github.com/team-alembic/ash_authentication/compare/v3.8.0...v3.9.0) (2023-02-09)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- Add new \"magic link\" authentication strategy. (#184)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- validate uniqueness of strategy names. (#185)\n\n- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Strategy.Custom: handle custom strategies as extensions. (#183)\n\n- improve error message for badly formed token secrets (#181)\n\n- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.8.0](https://github.com/team-alembic/ash_authentication/compare/v3.7.9...v3.8.0) (2023-02-09)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- Add new \"magic link\" authentication strategy. (#184)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- validate uniqueness of strategy names. (#185)\n\n- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Strategy.Custom: handle custom strategies as extensions. (#183)\n\n- improve error message for badly formed token secrets (#181)\n\n- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.9](https://github.com/team-alembic/ash_authentication/compare/v3.7.8...v3.7.9) (2023-02-09)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- validate uniqueness of strategy names. (#185)\n\n- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Strategy.Custom: handle custom strategies as extensions. (#183)\n\n- improve error message for badly formed token secrets (#181)\n\n- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.8](https://github.com/team-alembic/ash_authentication/compare/v3.7.7...v3.7.8) (2023-02-08)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Strategy.Custom: handle custom strategies as extensions. (#183)\n\n- improve error message for badly formed token secrets (#181)\n\n- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.7](https://github.com/team-alembic/ash_authentication/compare/v3.7.6...v3.7.7) (2023-02-06)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- improve error message for badly formed token secrets (#181)\n\n- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.6](https://github.com/team-alembic/ash_authentication/compare/v3.7.5...v3.7.6) (2023-01-30)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- resources can appear in multiple apis, so we need to uniq them here (#169)\n\n- put_add_on/2 was putting into strategies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.5](https://github.com/team-alembic/ash_authentication/compare/v3.7.4...v3.7.5) (2023-01-30)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add metadata declarations to actions that have a `token` (#164)\n\n- validate signing secret is a string (#163)\n\n## [v3.7.4](https://github.com/team-alembic/ash_authentication/compare/v3.7.3...v3.7.4) (2023-01-30)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- validate signing secret is a string (#163)\n\n## [v3.7.3](https://github.com/team-alembic/ash_authentication/compare/v3.7.2...v3.7.3) (2023-01-18)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Password: validate fields using both methods of allowing nil input. (#151)\n\n## [v3.7.2](https://github.com/team-alembic/ash_authentication/compare/v3.7.1...v3.7.2) (2023-01-18)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- AuthenticationFailed: store a `caused_by` value in authentication failures. (#145)\n\n## [v3.7.1](https://github.com/team-alembic/ash_authentication/compare/v3.7.0...v3.7.1) (2023-01-18)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- update ash & switch to new docs patterns (#146)\n\n## [v3.7.0](https://github.com/team-alembic/ash_authentication/compare/v3.6.1...v3.7.0) (2023-01-18)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- PasswordValidation: Add a validation which can check a password. (#144)\n\n## [v3.6.1](https://github.com/team-alembic/ash_authentication/compare/v3.6.0...v3.6.1) (2023-01-15)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- don't call `hash_provider.valid?` on `nil` values (#135)\n\n- use configured hashed_password_field","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- set confirmed field to `nil`, for reconfirmation (#136)\n\n- set confirmed field to `nil`, for reconfirmation\n\n- only change `confirmed_at_field` if its not changing, and only on updates\n\n## [v3.6.0](https://github.com/team-alembic/ash_authentication/compare/v3.5.3...v3.6.0) (2023-01-13)","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- TokenResource: Store the token subject in the token resource. (#133)\n\n- TokenResource: Store the token subject in the token resource.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- don't call `hash_provider.valid?` on `nil` values (#135)\n\n- use configured hashed_password_field\n\n## [v3.5.3](https://github.com/team-alembic/ash_authentication/compare/v3.5.2...v3.5.3) (2023-01-13)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Confirmation: send the original changeset to confirmation senders. (#132)\n\n## [v3.5.2](https://github.com/team-alembic/ash_authentication/compare/v3.5.1...v3.5.2) (2023-01-12)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add user context when creating tokens (#129)\n\n## [v3.5.1](https://github.com/team-alembic/ash_authentication/compare/v3.5.0...v3.5.1) (2023-01-12)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- missing icons in OAuth2 strategies. (#126)\n\n## [v3.5.0](https://github.com/team-alembic/ash_authentication/compare/v3.4.2...v3.5.0) (2023-01-12)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- GitHub: Add GitHub authentication strategy. (#125)\n\n## [v3.4.2](https://github.com/team-alembic/ash_authentication/compare/v3.4.1...v3.4.2) (2023-01-12)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- improve some error message/validation logic","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add policy utilities and accompanying guide (#119)\n\n- add policy utilities and accompanying guide\n\n- fix build/warnings/dialyzer/format\n\n## [v3.4.1](https://github.com/team-alembic/ash_authentication/compare/v3.4.0...v3.4.1) (2023-01-12)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- Confirmation: correctly generate confirmation token subjects. (#124)\n\n## [v3.4.0](https://github.com/team-alembic/ash_authentication/compare/v3.3.1...v3.4.0) (2023-01-11)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"- Add token-required-for-authentication feature. (#116)\n\n## [v3.3.1](https://github.com/team-alembic/ash_authentication/compare/v3.3.0...v3.3.1) (2023-01-09)","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Set Ash actor and tenant when executing internal plugs. (#115)\n\n## [v3.3.0](https://github.com/team-alembic/ash_authentication/compare/v3.2.2...v3.3.0) (2023-01-09)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- Make strategy names optional where possible. (#113)\n\n## [v3.2.2](https://github.com/team-alembic/ash_authentication/compare/v3.2.1...v3.2.2) (2023-01-08)","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Allow the strategy name to be passed for password validations and changes. (#102)\n\n## [v3.2.1](https://github.com/team-alembic/ash_authentication/compare/v3.2.0...v3.2.1) (2022-12-16)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- add `icon` field to OAuth2 strategy. (#100)\n\n## [v3.2.0](https://github.com/team-alembic/ash_authentication/compare/v3.1.0...v3.2.0) (2022-12-16)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- Auth0: Add a pre-configured Auth0 strategy. (#99)\n\n## [v3.1.0](https://github.com/team-alembic/ash_authentication/compare/v3.0.4...v3.1.0) (2022-12-14)","ref":"changelog.html#features"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Jwt: Use token signing secret into the DSL.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features: - Change Log","doc":"- Add option to store all tokens when they're created. (#91)","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"- remove the need for a strategy in changeset/query contexts. (#89)\n\n- add transaction reason\n\n- try a simpler way of ensuring module is compiled\n\n## [v3.0.4](https://github.com/team-alembic/ash_authentication/compare/v3.0.3...v3.0.4) (2022-12-08)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- update to latest ash version\n\n## [v3.0.3](https://github.com/team-alembic/ash_authentication/compare/v3.0.2...v3.0.3) (2022-12-07)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- break potential compiler dependency loops. (#64)\n\n## [v3.0.2](https://github.com/team-alembic/ash_authentication/compare/v3.0.1...v3.0.2) (2022-12-05)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- supervisor: require that the user adds the supervisor to their OTP app. (#62)\n\n## [v3.0.1](https://github.com/team-alembic/ash_authentication/compare/v3.0.0...v3.0.1) (2022-12-05)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- actions: All actions now take optional arguments for the underlying API call. (#61)\n\n## [v3.0.0](https://github.com/team-alembic/ash_authentication/compare/v2.0.1...v3.0.0) (2022-12-04)","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- TokenResource: Move `TokenRevocation` -> `TokenResource`.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Confirmation: Store confirmation changes in the token resource.\n\n## [v2.0.1](https://github.com/team-alembic/ash_authentication/compare/v2.0.0...v2.0.1) (2022-11-24)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- Confirmation: Confirmation is not a strategy. (#46)\n\n- Confirmation: Confirmation is not a strategy.\n\n- Confirmation: Support more than one confirmation entity.\n\n## [v2.0.0](https://github.com/team-alembic/ash_authentication/compare/v1.0.0...v2.0.0) (2022-11-22)","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- Major redesign of DSL and code structure. (#35)\n\n## [v1.0.0](https://github.com/team-alembic/ash_authentication/compare/v0.6.1...v1.0.0) (2022-11-15)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"- OAuth2Authentication: Make the `site` option runtime configurable. (#31)\n\n## [v0.6.1](https://github.com/team-alembic/ash_authentication/compare/v0.6.0...v0.6.1) (2022-11-15)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- OAuth2Authentication: Return the failure reason even if it's not a changeset. (#29)\n\n## [v0.6.0](https://github.com/team-alembic/ash_authentication/compare/v0.5.0...v0.6.0) (2022-11-10)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"- OAuth2Authentication: Add support for generic OAuth2 endpoints. (#28)\n\n## [v0.5.0](https://github.com/team-alembic/ash_authentication/compare/v0.4.3...v0.5.0) (2022-11-04)","ref":"changelog.html#features"},{"type":"extras","title":"Features: - Change Log","doc":"- Confirmation: Add extension that allows a user to be confirmed when created or updated. (#27)\n\n## [v0.4.3](https://github.com/team-alembic/ash_authentication/compare/v0.4.2...v0.4.3) (2022-11-03)","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"- docs: Improve endpoint docs for PasswordAuthentication and PasswordReset.\n\n## [v0.4.2](https://github.com/team-alembic/ash_authentication/compare/v0.4.1...v0.4.2) (2022-11-03)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- PasswordReset: Generate the reset token using the target action, not the source action. (#25)\n\n- PasswordReset: Generate the reset token using the target action, not the source action.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- PasswordReset: rework PasswordReset to be a provider in it's own right - this means it has it's own routes, etc.\n\n## [v0.4.1](https://github.com/team-alembic/ash_authentication/compare/v0.4.0...v0.4.1) (2022-11-03)","ref":"changelog.html#improvements"},{"type":"extras","title":"Improvements: - Change Log","doc":"- PasswordReset: A reset request is actually a query, not an update. (#23)\n\n## [v0.4.0](https://github.com/team-alembic/ash_authentication/compare/v0.3.0...v0.4.0) (2022-11-02)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- PasswordReset: allow users to request and reset their password. (#22)\n\n## [v0.3.0](https://github.com/team-alembic/ash_authentication/compare/v0.2.1...v0.3.0) (2022-10-31)","ref":"changelog.html#features"},{"type":"extras","title":"Features: - Change Log","doc":"- Ash.PlugHelpers: Support standard actor configuration. (#16)\n\n- Ash.PlugHelpers: Support standard actor configuration.","ref":"changelog.html#features"},{"type":"extras","title":"Improvements: - Change Log","doc":"- docs: change all references to `actor` to `user`.\n\n## [v0.2.1](https://github.com/team-alembic/ash_authentication/compare/v0.2.0...v0.2.1) (2022-10-26)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"- deprecation warnings caused by use of `Macro.expand_literal/2`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Change Log","doc":"- move subject_name uniqueness validation to compile time.\n\n- remove `generated: true` from macros.\n\n## [v0.2.0](https://github.com/team-alembic/ash_authentication/compare/v0.1.0...v0.2.0) (2022-10-24)","ref":"changelog.html#improvements"},{"type":"extras","title":"Features: - Change Log","doc":"- PasswordAuthentication: Registration and authentication with local credentials (#4)\n\n## [v0.1.0](https://github.com/team-alembic/ash_authentication/compare/v0.1.0...v0.1.0) (2022-09-27)","ref":"changelog.html#features"},{"type":"extras","title":"Get started with Ash Authentication","doc":"# Get started with Ash Authentication\n\nIf you haven't already, read [the getting started guide for\nAsh](https://hexdocs.pm/ash/get-started.html). This\nassumes that you already have resources set up, and only gives you the steps to\nadd authentication to your resources and APIs.\n\n<!-- tabs-open -->","ref":"get-started.html"},{"type":"extras","title":"Using Igniter (recommended) - Get started with Ash Authentication","doc":"#### Install the extension\n\n```sh\nmix igniter.install ash_authentication --auth-strategy magic_link,password\n```\n\n##### Using Phoenix?\n\nUse the following. If you have not yet run the above command, this will prompt you to do so,\nso you can run both or only this one.\n\n```sh\nmix igniter.install ash_authentication_phoenix --auth-strategy magic_link,password\n```","ref":"get-started.html#using-igniter-recommended"},{"type":"extras","title":"Manual - Get started with Ash Authentication","doc":"#### Add to your application's dependencies\n\nBring in the `ash_authentication` dependency:\n\n```elixir\n# mix.exs\n\ndefp deps()\n  [\n    # ...\n    {:ash_authentication, \"~> 4.0\"}\n  ]\nend\n```\n\nAnd add `ash_authentication` to your `.formatter.exs`:\n\n```elixir\n# .formatter.exs\n[\n  import_deps: [..., :ash_authentication]\n]\n```\n\n#### Create authentication domain and resources\n\nLet's create an `Accounts` domain in our application which provides a `User`\nresource and a `Token` resource. This tutorial is assuming that you are using `AshPostgres`.\n\nFirst, let's define our domain:\n\n```elixir\n# lib/my_app/accounts.ex\n\ndefmodule MyApp.Accounts do\n  use Ash.Domain\n\n  resources do\n    resource MyApp.Accounts.User\n    resource MyApp.Accounts.Token\n  end\nend\n```\n\nBe sure to add it to the `ash_domains` config in your `config.exs`\n\n```elixir\n# in config/config.exs\nconfig :my_app, ash_domains: [..., MyApp.Accounts]\n```\n\nNext, let's define our `Token` resource. This resource is needed\nif token generation is enabled for any resources in your application. Most of\nthe contents are auto-generated, so we just need to provide the data layer\nconfiguration and the API to use.\n\nBut before we do, we need to install a postgres extension.\n\n```elixir\n# lib/my_app/repo.ex\n\ndefmodule MyApp.Repo do\n  use AshPostgres.Repo, otp_app: :my_app\n\n  def installed_extensions do\n    [\"ash-functions\", \"citext\"]\n  end\nend\n```\n\n#### Setup Token Resource\n\n```elixir\n# lib/my_app/accounts/token.ex\ndefmodule MyApp.Accounts.Token do\n  use Ash.Resource,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshAuthentication.TokenResource],\n    # If using policies, enable the policy authorizer:\n    authorizers: [Ash.Policy.Authorizer],\n    domain: MyApp.Accounts\n\n  postgres do\n    table \"tokens\"\n    repo MyApp.Repo\n  end\n\n  policies do\n    bypass AshAuthentication.Checks.AshAuthenticationInteraction do\n      authorize_if always()\n    end\n  end\nend\n```\n\n#### Supervisor\n\nAshAuthentication includes a supervisor which you should add to your\napplication's supervisor tree. This is used to run any periodic jobs related to\nyour authenticated resources (removing expired tokens, for example).\n\n##### Example\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # ...\n      # add this line -->\n      {AshAuthentication.Supervisor, otp_app: :my_app}\n      # <-- add this line\n    ]\n    # ...\n  end\nend\n```\n\nLastly let's define our `User` resource. Note that we aren't defining any authentication strategies here.\nThis setup is used for all strategies. Once you have done this, you can follow one of the strategy specific\nguides at the bottom of this page.\n\n```elixir\n# lib/my_app/accounts/user.ex\n\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshAuthentication],\n    authorizers: [Ash.Policy.Authorizer],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n  end\n\n  authentication do\n    tokens do\n      enabled? true\n      token_resource MyApp.Accounts.Token\n      signing_secret fn _, _ ->\n        # This is a secret key used to sign tokens. See the note below on secrets management\n        Application.fetch_env(:my_app, :token_signing_secret)\n      end\n    end\n  end\n\n  postgres do\n    table \"users\"\n    repo MyApp.Repo\n  end\n\n  # You can customize this if you wish, but this is a safe default that\n  # only allows user data to be interacted with via AshAuthentication.\n  policies do\n    bypass AshAuthentication.Checks.AshAuthenticationInteraction do\n      authorize_if always()\n    end\n\n    policy always() do\n      forbid_if always()\n    end\n  end\nend\n```\n\n>","ref":"get-started.html#manual"},{"type":"extras","title":"The signing secret must not be committed to source control {: .warning} - Get started with Ash Authentication","doc":">\n> Proper management of secrets is outside the scope of this tutorial, but is\n> absolutely crucial to the security of your application.\n\n<!-- tabs-close -->","ref":"get-started.html#the-signing-secret-must-not-be-committed-to-source-control-warning"},{"type":"extras","title":"Choose your strategies and add-ons - Get started with Ash Authentication","doc":"### `mix ash_authentication.add_strategy`\n\nA mix task is provided to add strategies and add-ons to your application.\nFor now, this only supports the `password` strategy, but more will be added in the future.\n\n```sh\nmix ash_authentication.add_strategy password\n```\n\n#### Strategies\n\n- [Password](/documentation/tutorials/password.md)\n- [Github](/documentation/tutorials/github.md)\n- [Google](/documentation/tutorials/google.md)\n- [Magic Links](/documentation/tutorials/magic-links.md)\n- [Auth0](/documentation/tutorials/auth0.md)\n- Open ID: `AshAuthentication.Strategy.Oidc`\n- OAuth2: `AshAuthentication.Strategy.OAuth2`\n\n#### Add-Ons\n\n- [Confirmation](/documentation/tutorials/confirmation.md): confirming changes to user accounts (i.e via email)\n- UserIdentity: `AshAuthentication.UserIdentity`: supporting multiple social sign on identities & refreshing tokens","ref":"get-started.html#choose-your-strategies-and-add-ons"},{"type":"extras","title":"Set up your Phoenix or Plug application - Get started with Ash Authentication","doc":"If you're using Phoenix, skip this section and go to\n[Integrating Ash Authentication and Phoenix](https://hexdocs.pm/ash_authentication_phoenix/get-started.html)\n\nIn order for your users to be able to sign in, you will likely need to provide\nan HTTP endpoint to submit credentials or OAuth requests to. Ash Authentication\nprovides `AshAuthentication.Plug` for this purposes. It provides a `use` macro\nwhich handles routing of requests to the correct providers, and defines\ncallbacks for successful and unsuccessful outcomes.\n\nLet's generate our plug:\n\n```elixir\n# lib/my_app/auth_plug.ex\n\ndefmodule MyApp.AuthPlug do\n  use AshAuthentication.Plug, otp_app: :my_app\n\n  def handle_success(conn, _activity, user, token) do\n    if is_api_request?(conn) do\n      conn\n      |> send_resp(200, Jason.encode!(%{\n        authentication: %{\n          success: true,\n          token: token\n        }\n      }))\n    else\n      conn\n      |> store_in_session(user)\n      |> send_resp(200, EEx.eval_string(\"\"\"\n       Welcome back <%= @user.email %> \n      \"\"\", user: user))\n    end\n  end\n\n  def handle_failure(conn, _activity, _reason) do\n    if is_api_request?(conn) do\n      conn\n      |> send_resp(401, Jason.encode!(%{\n        authentication: %{\n          success: false\n        }\n      }))\n    else\n      conn\n      |> send_resp(401, \" Incorrect email or password \")\n    end\n  end\n\n  defp is_api_request?(conn), do: \"application/json\" in get_req_header(conn, \"accept\")\nend\n```\n\nNow that this is done, you can forward HTTP requests to it from your app's main\nrouter using `forward \"/auth\", to: MyApp.AuthPlug` or similar.\n\nYour generated auth plug module will also contain `load_from_session` and\n`load_from_bearer` function plugs, which can be used to load users into assigns\nbased on the contents of the session store or `Authorization` header.","ref":"get-started.html#set-up-your-phoenix-or-plug-application"},{"type":"extras","title":"Summary - Get started with Ash Authentication","doc":"In this guide we've learned how to install Ash Authentication, configure\nresources and handle authentication HTTP requests.\n\nYou should now have an Ash application with working user authentication.\n\nUp next, [Using with Phoenix](https://hexdocs.pm/ash_authentication_phoenix/get-started.html)","ref":"get-started.html#summary"},{"type":"extras","title":"Auth0 Tutorial","doc":"# Auth0 Tutorial\n\nThis is a quick tutorial on how to configure your application to use Auth0 for authentication.\n\nFirst, you need to configure an application in [the Auth0 dashboard](https://manage.auth0.com/) using the following steps:\n\n1. Click \"Create Application\".\n2. Set your application name to something that identifies it. You will likely\n   need separate applications for development and production environments, so\n   keep that in mind.\n3. Select \"Regular Web Application\" and click \"Create\".\n4. Switch to the \"Settings\" tab.\n5. Copy the \"Domain\", \"Client ID\" and \"Client Secret\" somewhere safe - we'll need them soon.\n6. In the \"Allowed Callback URLs\" section, add your callback URL. The callback URL is generated from the following information:\n\n   - The base URL of the application - in development that would be\n     `http://localhost:4000/` but in production will be your application's\n     URL.\n   - The mount point of the auth routes in your router - we'll assume\n     `/auth`.\n   - The \"subject name\" of the resource being authenticated - we'll assume `user`.\n   - The name of the strategy in your configuration. By default this is\n     `auth0`.\n\n   This means that the callback URL should look something like\n   `http://localhost:4000/auth/user/auth0/callback`.\n\n7. Set \"Allowed Web Origins\" to your application's base URL.\n8. Click \"Save Changes\".\n\nNext we can configure our resource:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      auth0 do\n        client_id MyApp.Secrets\n        redirect_uri MyApp.Secrets\n        client_secret MyApp.Secrets\n        base_url MyApp.Secrets\n      end\n    end\n  end\nend\n```\n\nBecause all the configuration values should be kept secret (ie the `client_secret`) or are likely to be different for each environment we use the `AshAuthentication.Secret` behaviour to provide them. In this case we're delegating to the OTP application environment, however you may want to use a system environment variable or some other secret store (eg Vault).\n\n```elixir\ndefmodule MyApp.Secrets do\n  use AshAuthentication.Secret\n\n  def secret_for([:authentication, :strategies, :auth0, :client_id], MyApp.Accounts.User, _) do\n    get_config(:client_id)\n  end\n\n  def secret_for([:authentication, :strategies, :auth0, :redirect_uri], MyApp.Accounts.User, _) do\n    get_config(:redirect_uri)\n  end\n\n  def secret_for([:authentication, :strategies, :auth0, :client_secret], MyApp.Accounts.User, _) do\n    get_config(:client_secret)\n  end\n\n  def secret_for([:authentication, :strategies, :auth0, :base_url], MyApp.Accounts.User, _) do\n    get_config(:base_url)\n  end\n\n  defp get_config(key) do\n    :my_app\n    |> Application.fetch_env!(:auth0)\n    |> Keyword.fetch!(key)\n    |> then(&{:ok, &1})\n  end\nend\n```\n\nThe values for this configuration should be:\n\n- `client_id` - the client ID copied from the Auth0 settings page.\n- `redirect_uri` - the URL to the generated auth routes in your application (eg `http://localhost:4000/auth`).\n- `client_secret` the client secret copied from the Auth0 settings page.\n- `base_url` - the \"domain\" value copied from the Auth0 settings page prefixed with `https://` (eg `https://dev-yu30yo5y4tg2hg0y.us.auth0.com`).\n\nLastly, we need to add a register action to your user resource. This is defined as an upsert so that it can register new users, or update information for returning users. The default name of the action is `register_with_` followed by the strategy name. In our case that is `register_with_auth0`.\n\nThe register action takes two arguments, `user_info` and the `oauth_tokens`.\n\n- `user_info` contains the [`GET /userinfo` response from Auth0](https://auth0.com/docs/api/authentication#get-user-info) which you can use to populate your user attributes as needed.\n- `oauth_tokens` contains the [`POST /oauth/token` response from Auth0](https://auth0.com/docs/api/authentication#get-token) - you may want to store these if you intend to call the Auth0 API on behalf of the user.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  # ...\n\n  actions do\n    create :register_with_auth0 do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      upsert? true\n      upsert_identity :unique_email\n\n      # Required if you have token generation enabled.\n      change AshAuthentication.GenerateTokenChange\n\n      # Required if you have the `identity_resource` configuration enabled.\n      change AshAuthentication.Strategy.OAuth2.IdentityChange\n\n      change fn changeset, _ ->\n        user_info = Ash.Changeset.get_argument(changeset, :user_info)\n\n        Ash.Changeset.change_attributes(changeset, Map.take(user_info, [\"email\"]))\n      end\n    end\n  end\n\n  # ...\n\nend\n```","ref":"auth0.html"},{"type":"extras","title":"Confirmation Tutorial","doc":"# Confirmation Tutorial\n\nThis add-on allows you to confirm changes to a user record by generating and\nsending them a confirmation token which they must submit before allowing the\nchange to take place.\n\nIn this tutorial we'll assume that you have a `User` resource which uses `email` as it's user identifier.\nWe'll show you how to confirm a new user on sign-up and also require them to confirm if they wish to change their email address.","ref":"confirmation.html"},{"type":"extras","title":"Important security notes - Confirmation Tutorial","doc":"If you are using multiple strategies that use emails, where one of the strategy has an upsert registration (like social sign-up, magic link registration),\nthen you _must_ use the confirmation add-on to prevent account hijacking, as described below.\n\nExample scenario:\n\n- Attacker signs up with email of their target and a password, but does not confirm their email.\n- Their target signs up with google or magic link, etc, which upserts the user, and sets `confirmed_at` to `true`.\n- Now, the user has created an account but the attacker has access via the password they originally set.","ref":"confirmation.html#important-security-notes"},{"type":"extras","title":"How to handle this? - Confirmation Tutorial","doc":"#### Automatic Handling\n\nThe confirmation add-on prevents this by default by not allowing an upsert action to set `confirmed_at`, if there is\na matching record that has `confirmed_at` that is currently `nil`. This allows you to show a message to the user like\n\"You signed up with a different method. Please sign in with the method you used to sign up.\"\n\n#### auto_confirming and clearing the password on upsert\n\nAn alternative is to clear the user's password on upsert. To do this, you would want to ensure the following things are true:\n\n- The upsert registration action(s) are in the `auto_confirm_actions` (which you want anyway)\n- The upsert registration action(s) set `hashed_password` to `nil`, removing any access an attacker may have had\n- The `prevent_hijacking?` option is set to `false` on the confirmation add on and the auth strategies you are using.\n- A user cannot access your application or take any action without a confirmed account. For example, redirecting to a \"please confirm your account\" page.\n\nWhy do you have to ensure that no actions can be taken without a confirmed account?\n\nThis does technically remove any access that the attacker may have had from the account, but we don't suggest taking this approach\nunless you are absolutely sure that you know what you are doing. For example, lets say you have an app that shows where the user is\nin the world, or where their friends are in the world. Lets say you also allow configuring a phone number to receive text notifications\nwhen they are near one of their friends. An attacker could sign up with a password, and configure their phone number. Then, their target\nsigns up with Oauth or magic link, adds some friends, but doesn't notice that a phone number is configured.\n\nNow the attacker is getting text messages about where the user and/or their friends are.\n\n#### Opt-out\n\nYou can set `prevent_hijacking? false` on either the confirmation add-on, or your strategy to disable the automatic handling\ndescribed above, and not follow the steps recommended in the section section above. This is not recommended.","ref":"confirmation.html#how-to-handle-this"},{"type":"extras","title":"Tutorial - Confirmation Tutorial","doc":"Here's the user resource we'll be starting with:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :ci_string, allow_nil?: false, public?: true, sensitive?: true\n    attribute :hashed_password, :string, allow_nil?: false, public?: false, sensitive?: true\n  end\n\n  authentication do\n    strategies do\n      password :password do\n        identity_field :email\n        hashed_password_field :hashed_password\n      end\n    end\n  end\n\n  identities do\n    identity :unique_email, [:email]\n  end\nend\n```","ref":"confirmation.html#tutorial"},{"type":"extras","title":"Confirming newly registered users - Confirmation Tutorial","doc":"First we start by adding the confirmation add-on to your existing authentication DSL:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n\n  authentication do\n    # ...\n\n    add_ons do\n      confirmation :confirm_new_user do\n        monitor_fields [:email]\n        confirm_on_create? true\n        confirm_on_update? false\n        sender MyApp.Accounts.User.Senders.SendNewUserConfirmationEmail\n      end\n    end\n  end\nend\n```\n\nNext we will have to generate and run migrations to add confirmed_at column to user resource\n\n```bash\nmix ash.codegen account_confirmation\n```\n\nTo make this work we need to create a new module `MyApp.Accounts.User.Senders.SendNewUserConfirmationEmail`:\n\n```elixir\ndefmodule MyApp.Accounts.User.Senders.SendNewUserConfirmationEmail do\n  @moduledoc \"\"\"\n  Sends an email confirmation email\n  \"\"\"\n  use AshAuthentication.Sender\n  use MyAppWeb, :verified_routes\n\n  @impl AshAuthentication.Sender\n  def send(user, token, _opts) do\n    MyApp.Accounts.Emails.deliver_email_confirmation_instructions(\n      user,\n      url(~p\"/auth/user/confirm_new_user?#{[confirm: token]}\")\n    )\n  end\nend\n```\n\nWe also need to create a new email template:\n\n```elixir\ndefmodule Example.Accounts.Emails do\n  @moduledoc \"\"\"\n  Delivers emails.\n  \"\"\"\n\n  import Swoosh.Email\n\n  def deliver_email_confirmation_instructions(user, url) do\n    if !url do\n      raise \"Cannot deliver confirmation instructions without a url\"\n    end\n\n    deliver(user.email, \"Confirm your email address\", \"\"\"\n       \n        Hi #{user.email},\n       \n\n       \n        Someone has tried to register a new account using this email address.\n        If it was you, then please click the link below to confirm your identity. If you did not initiate this request then please ignore this email.\n       \n\n       \n         Click here to confirm your account \n       \n    \"\"\")\n  end\n\n  # For simplicity, this module simply logs messages to the terminal.\n  # You should replace it by a proper email or notification tool, such as:\n  #\n  #   * Swoosh - https://hexdocs.pm/swoosh\n  #   * Bamboo - https://hexdocs.pm/bamboo\n  #\n  defp deliver(to, subject, body) do\n    IO.puts(\"Sending email to #{to} with subject #{subject} and body #{body}\")\n\n    new()\n    |> from({\"Zach\", \"zach@ash-hq.org\"}) # TODO: Replace with your email\n    |> to(to_string(to))\n    |> subject(subject)\n    |> put_provider_option(:track_links, \"None\")\n    |> html_body(body)\n    |> MyApp.Mailer.deliver!()\n  end\nend\n```\n\nProvided you have your authentication routes hooked up either via `AshAuthentication.Plug` or [`AshAuthentication.Phoenix.Router`](https://hexdocs.pm/ash_authentication_phoenix/AshAuthentication.Phoenix.Router.html) then the user will be confirmed when the token is submitted.","ref":"confirmation.html#confirming-newly-registered-users"},{"type":"extras","title":"Blocking unconfirmed users from logging in - Confirmation Tutorial","doc":"The above section explains how to confirm an user account. There's a new directive in the [dsl](https://hexdocs.pm/ash_authentication/dsl-ashauthentication-strategy-password.html#authentication-strategies-password-require_confirmed_with) which can require the user to be confirmed in order to log in.\n\nSo:\n\n```\nstrategies do\n  strategy :password do\n    ...\n    require_confirmed_with :confirmed_at\n  end\nend\n```\n\nthis will make impossible for unconfirmed users to log in. Note that at the moment it is developer responsibility to handle the scenario, for example redirecting the user to a page that gives some context and maybe offers the chance to require a new confirmation email in case the previous one is lost.\n\nIf the field value is `nil` or if the field itself is not present, no confirmation check will be enforced.","ref":"confirmation.html#blocking-unconfirmed-users-from-logging-in"},{"type":"extras","title":"Confirming changes to monitored fields - Confirmation Tutorial","doc":"You may want to require a user to perform a confirmation when a certain field changes. For example if a user changes their email address we can send them a new confirmation request.\n\nFirst, let's start by defining a new confirmation add-on in our resource:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n\n  authentication do\n    # ...\n\n    add_ons do\n      confirmation :confirm_change do\n        monitor_fields [:email]\n        confirm_on_create? false\n        confirm_on_update? true\n        confirm_action_name :confirm_change\n        sender MyApp.Accounts.User.Senders.SendEmailChangeConfirmationEmail\n      end\n    end\n  end\nend\n```\n\n> #### Why two confirmation configurations? {: .info}\n>\n> While you can perform both of these confirmations with a single confirmation add-on, in general the Ash philosophy is to be more explicit. Each confirmation will have it's own URL (based on the name) and tokens for one will not be able to be used for the other.\n\nNext, let's define our new sender:\n\n```elixir\ndefmodule MyApp.Accounts.User.Senders.SendEmailChangeConfirmationEmail do\n  @moduledoc \"\"\"\n  Sends an email change confirmation email\n  \"\"\"\n  use AshAuthentication.Sender\n  use MyAppWeb, :verified_routes\n\n  @impl AshAuthentication.Sender\n  def send(user, token, _opts) do\n    MyApp.Accounts.Emails.deliver_email_change_confirmation_instructions(\n      user,\n      url(~p\"/auth/user/confirm_change?#{[confirm: token]}\")\n    )\n  end\nend\n```\n\nAnd our new email template:\n\n```elixir\ndefmodule MyApp.Accounts.Emails do\n  # ...\n\n  def deliver_email_change_confirmation_instructions(user, url) do\n    if !url do\n      raise \"Cannot deliver confirmation instructions without a url\"\n    end\n\n    deliver(user.email, \"Confirm your new email address\", \"\"\"\n       \n        Hi #{user.email},\n       \n\n       \n        You recently changed your email address. Please confirm it.\n       \n\n       \n         Click here to confirm your new email address \n       \n    \"\"\")\n  end\n\n  # ...\nend\n```\n\n> #### Inhibiting changes {: .tip}\n>\n> Depending on whether you want the user's changes to be applied _before_ or _after_ confirmation, you can enable the [`inhibit_updates?` DSL option](documentation/dsls/DSL-AshAuthentication.AddOn.Confirmation.md#authentication-add_ons-confirmation-inhibit_updates?).\n>\n> When this option is enabled, then any potential changes to monitored fields are instead temporarily stored in the [token resource](documentation/dsls/DSL-AshAuthentication.TokenResource.md) and applied when the confirmation action is run.","ref":"confirmation.html#confirming-changes-to-monitored-fields"},{"type":"extras","title":"Customising the confirmation action - Confirmation Tutorial","doc":"By default Ash Authentication will generate an update action for confirmation automatically (named `:confirm` unless you change it). You can manually implement this action in order to change it's behaviour and AshAuthentication will validate that the required changes are also present.\n\nFor example, here's an implementation of the `:confirm_change` action mentioned above, which adds a custom change that updates a remote CRM system with the user's new address.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n\n  actions do\n    # ...\n\n    update :confirm_change do\n      argument :confirm, :string, allow_nil?: false, public?: true\n      accept [:email]\n      require_atomic? false\n      change AshAuthentication.AddOn.Confirmation.ConfirmChange\n      change AshAuthentication.GenerateTokenChange\n      change MyApp.UpdateCrmSystem, only_when_valid?: true\n    end\n  end\nend\n```","ref":"confirmation.html#customising-the-confirmation-action"},{"type":"extras","title":"GitHub Tutorial","doc":"# GitHub Tutorial\n\nThis is a quick tutorial on how to configure your application to use GitHub for authentication.\n\nFirst you need to configure an application in your [GitHub developer settings](https://github.com/settings/developers):\n\n1. Click the \"New OAuth App\" button.\n2. Set your application name to something that identifies it. You will likely\n   need separate applications for development and production environments, so\n   keep that in mind.\n3. Set \"Homepage URL\" appropriately for your application and environment.\n4. In the \"Authorization callback URL\" section, add your callback URL. The\n   callback URL is generated from the following information:\n\n   - The base URL of the application - in development that would be\n     `http://localhost:4000/` but in production will be your application's\n     URL.\n   - The mount point of the auth routes in your router - we'll assume\n     `/auth`.\n   - The \"subject name\" of the resource being authenticated - we'll assume `user`.\n   - The name of the strategy in your configuration. By default this is\n     `github`.\n\n   This means that the callback URL should look something like\n   `http://localhost:4000/auth/user/github/callback`.\n\n5. Do not set \"Enable Device Flow\" unless you know why you want this.\n6. Click \"Register application\".\n7. Click \"Generate a new client secret\".\n8. Copy the \"Client ID\" and \"Client secret\" somewhere safe, we'll need them\n   soon.\n9. Click \"Update application\".\n\nNext we can configure our resource (assuming you already have everything else\nset up):\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      github do\n        client_id MyApp.Secrets\n        redirect_uri MyApp.Secrets\n        client_secret MyApp.Secrets\n      end\n    end\n  end\nend\n```\n\nBecause all the configuration values should be kept secret (ie the\n`client_secret`) or are likely to be different for each environment we use the\n`AshAuthentication.Secret` behaviour to provide them. In this case we're\ndelegating to the OTP application environment, however you may want to use a\nsystem environment variable or some other secret store (eg Vault).\n\n```elixir\ndefmodule MyApp.Secrets do\n  use AshAuthentication.Secret\n\n  def secret_for([:authentication, :strategies, :github, :client_id], MyApp.Accounts.User, _) do\n    get_config(:client_id)\n  end\n\n  def secret_for([:authentication, :strategies, :github, :redirect_uri], MyApp.Accounts.User, _) do\n    get_config(:redirect_uri)\n  end\n\n  def secret_for([:authentication, :strategies, :github, :client_secret], MyApp.Accounts.User, _) do\n    get_config(:client_secret)\n  end\n\n  defp get_config(key) do\n    :my_app\n    |> Application.get_env(:github, [])\n    |> Keyword.fetch(key)\n  end\nend\n```\n\nThe values for this configuration should be:\n\n- `client_id` - the client ID copied from the GitHub settings page.\n- `redirect_uri` - the URL to the generated auth routes in your application\n  (eg `http://localhost:4000/auth`).\n- `client_secret` the client secret copied from the GitHub settings page.\n\nLastly, we need to add a register action to your user resource. This is defined\nas an upsert so that it can register new users, or update information for\nreturning users. The default name of the action is `register_with_` followed by\nthe strategy name. In our case that is `register_with_github`.\n\nThe register action takes two arguments, `user_info` and the `oauth_tokens`.\n\n- `user_info` contains the [`GET /user` response from\n  GitHub](https://docs.github.com/en/rest/users/users?apiVersion=2022-11-28#get-the-authenticated-user)\n  which you can use to populate your user attributes as needed.\n- `oauth_tokens` contains the [`POST /login/oauth/access_token` response from\n  GitHub](https://docs.github.com/en/developers/apps/building-oauth-apps/authorizing-oauth-apps#response)\n  - you may want to store these if you intend to call the GitHub API on behalf\n    of the user.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  require Ash.Resource.Change.Builtins\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  # ...\n\n  actions do\n    create :register_with_github do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      upsert? true\n      upsert_identity :unique_email\n\n      # Required if you have token generation enabled.\n      change AshAuthentication.GenerateTokenChange\n\n      # Required if you have the `identity_resource` configuration enabled.\n      change AshAuthentication.Strategy.OAuth2.IdentityChange\n\n      change fn changeset, _ ->\n        user_info = Ash.Changeset.get_argument(changeset, :user_info)\n\n        Ash.Changeset.change_attributes(changeset, Map.take(user_info, [\"email\"]))\n      end\n\n      # Required if you're using the password & confirmation strategies\n      upsert_fields []\n      change set_attribute(:confirmed_at, &DateTime.utc_now/0)\n      change after_action(fn _changeset, user, _context ->\n        case user.confirmed_at do\n          nil -> {:error, \"Unconfirmed user exists already\"}\n          _ -> {:ok, user}\n        end\n      end)\n    end\n  end\n\n  # ...\n\nend\n```\n\nEnsure you set the `hashed_password` to `allow_nil?` if you are also using the password strategy.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n  attributes do\n    # ...\n    attribute :hashed_password, :string, allow_nil?: true, sensitive?: true\n  end\n  # ...\nend\n```\n\nAnd generate and run migrations in that case.\n\n```bash\nmix ash.codegen make_hashed_password_nullable\nmix ash.migrate\n```","ref":"github.html"},{"type":"extras","title":"Google Tutorial","doc":"# Google Tutorial\n\nThis is a quick tutorial on how to configure Google authentication.\n\nFirst you'll need a registered application in [Google Cloud](https://console.cloud.google.com/welcome), in order to get your OAuth 2.0 Client credentials.\n\n1. On the Cloud's console **Quick access** section select **APIs & Services**, then **Credentials**\n2. Click on **+ CREATE CREDENTIALS** and from the dropdown select **OAuth client ID**\n3. From the google developers console, we will need: `client_id` & `client_secret`\n4. Enter your callback uri under **Authorized redirect URIs**. E.g. `http://localhost:4000/auth/user/google/callback`.\n\nNext we configure our resource to use google credentials:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  attributes do\n    ...\n  end\n\n  authentication do\n    strategies do\n      google do\n        client_id MyApp.Secrets\n        redirect_uri MyApp.Secrets\n        client_secret MyApp.Secrets\n      end\n    end\n  end\nend\n```\n\nPlease check the guide on how to properly configure your Secrets\nThen we need to define an action that will handle the oauth2 flow, for the google case it is `:register_with_google` it will handle both cases for our resource, user registration & login.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  require Ash.Resource.Change.Builtins\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  # ...\n  actions do\n    create :register_with_google do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      upsert? true\n      upsert_identity :unique_email\n\n      change AshAuthentication.GenerateTokenChange\n\n      # Required if you have the `identity_resource` configuration enabled.\n      change AshAuthentication.Strategy.OAuth2.IdentityChange\n\n      change fn changeset, _ ->\n        user_info = Ash.Changeset.get_argument(changeset, :user_info)\n\n        Ash.Changeset.change_attributes(changeset, Map.take(user_info, [\"email\"]))\n      end\n\n      # Required if you're using the password & confirmation strategies\n      upsert_fields []\n      change set_attribute(:confirmed_at, &DateTime.utc_now/0)\n      change after_action(fn _changeset, user, _context ->\n        case user.confirmed_at do\n          nil -> {:error, \"Unconfirmed user exists already\"}\n          _ -> {:ok, user}\n        end\n      end)\n    end\n  end\n\n  # ...\n\nend\n```\n\nEnsure you set the `hashed_password` to `allow_nil?` if you are also using the password strategy.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n  attributes do\n    # ...\n    attribute :hashed_password, :string, allow_nil?: true, sensitive?: true\n  end\n  # ...\nend\n```\n\nAnd generate and run migrations in that case.\n\n```bash\nmix ash.codegen make_hashed_password_nullable\nmix ash.migrate\n```","ref":"google.html"},{"type":"extras","title":"Magic Links Tutorial","doc":"# Magic Links Tutorial","ref":"magic-links.html"},{"type":"extras","title":"With a mix task - Magic Links Tutorial","doc":"You can use `mix ash_authentication.add_strategy magic_link` to install this strategy.\nThe rest of the guide is in the case that you wish to proceed manually.","ref":"magic-links.html#with-a-mix-task"},{"type":"extras","title":"Add the Magic Link Strategy to the User resource - Magic Links Tutorial","doc":"```elixir\n# ...\n\nstrategies do\n  # add these lines -->\n  magic_link do\n    identity_field :email\n    registration_enabled? true\n\n    sender(Example.Accounts.User.Senders.SendMagicLink)\n  end\n  # <-- add these lines\nend\n\n# ...\n```","ref":"magic-links.html#add-the-magic-link-strategy-to-the-user-resource"},{"type":"extras","title":"Registration Enabled - Magic Links Tutorial","doc":"When registration is enabled, signing in with magic is a _create_ action that upserts the user by email.\nThis allows a user who does not exist to request a magic link and sign up with one action.","ref":"magic-links.html#registration-enabled"},{"type":"extras","title":"Registration Disabled (default) - Magic Links Tutorial","doc":"When registration is disabled, signing in with magic link is a _read_ action.","ref":"magic-links.html#registration-disabled-default"},{"type":"extras","title":"Create an email sender and email template - Magic Links Tutorial","doc":"Inside `/lib/example/accounts/user/senders/send_magic_link.ex`\n\n```elixir\ndefmodule Example.Accounts.User.Senders.SendMagicLink do\n  @moduledoc \"\"\"\n  Sends a magic link\n  \"\"\"\n  use AshAuthentication.Sender\n  use ExampleWeb, :verified_routes\n\n  @impl AshAuthentication.Sender\n  def send(user_or_email, token, _) do\n    # will be a user if the token relates to an existing user\n    # will be an email if there is no matching user (such as during sign up)\n    Example.Accounts.Emails.deliver_magic_link(\n      user_or_email,\n      url(~p\"/auth/user/magic_link/?token=#{token}\")\n    )\n  end\nend\n```\n\nInside `/lib/example/accounts/emails.ex`\n\n```elixir\n# ...\n\ndef deliver_magic_link(user, url) do\n  if !url do\n    raise \"Cannot deliver reset instructions without a url\"\n  end\n\n  email = case user do\n    %{email: email} -> email\n    email -> email\n  end\n\n  deliver(email, \"Magic Link\", \"\"\"\n   \n     \n      Hi #{email},\n     \n\n     \n       Click here  to login.\n     \n   \n  \"\"\")\nend\n\n# ...\n```","ref":"magic-links.html#create-an-email-sender-and-email-template"},{"type":"extras","title":"Password Authentication","doc":"# Password Authentication","ref":"password.html"},{"type":"extras","title":"With a mix task - Password Authentication","doc":"You can use `mix ash_authentication.add_strategy password` to install this strategy.\nThe rest of the guide is in the case that you wish to proceed manually.","ref":"password.html#with-a-mix-task"},{"type":"extras","title":"Add Bcrypt To your dependencies - Password Authentication","doc":"This step is not strictly necessary, but in the next major version of `AshAuthentication`,\n`Bcrypt` will be an optional dependency. This will make that upgrade slightly easier.\n\n```elixir\n{:bcrypt_elixir, \"~> 3.0\"}\n```","ref":"password.html#add-bcrypt-to-your-dependencies"},{"type":"extras","title":"Add Attributes - Password Authentication","doc":"Add an `email` (or `username`) and `hashed_password` attribute to your user resource.\n\n```elixir\n# lib/my_app/accounts/user.ex\nattributes do\n  ...\n  attribute :email, :ci_string, allow_nil?: false, public?: true\n  attribute :hashed_password, :string, allow_nil?: false, sensitive?: true\nend\n```\n\nEnsure that the `email` (or username) is unique.\n\n```elixir\n# lib/my_app/accounts/user.ex\nidentities do\n  identity :unique_email, [:email]\n  # or\n  identity :unique_username, [:username]\nend\n```","ref":"password.html#add-attributes"},{"type":"extras","title":"Add the password strategy - Password Authentication","doc":"Configure it to use the `:email` or `:username` as the identity field.\n\n```elixir\n# lib/my_app/accounts/user.ex\nauthentication do\n  ...\n  strategies do\n    password :password do\n      identity_field :email\n      # or\n      identity_field :username\n    end\n  end\nend\n```\n\nNow we have enough in place to register and sign-in users using the\n`AshAuthentication.Strategy` protocol.","ref":"password.html#add-the-password-strategy"},{"type":"extras","title":"Slack Tutorial","doc":"# Slack Tutorial\n\nThis is a quick tutorial on how to configure your application to use Slack for authentication.\n\nFirst you need to configure an application in your [Slack app settings](https://api.slack.com/apps):\n\n1. Click the \"Create New App\" button.\n2. Select \"From scratch\"\n3. Set your application name to something that identifies it. You will likely\n   need separate applications for development and production environments, so\n   keep that in mind.\n4. Select a \"development workspace\", which can be used for testing.\n5. Browse to the \"OAuth & Permissions\" page.\n6. In the \"Redirect URLs\" section add your callback URL. The\n   callback URL is generated from the following information:\n\n   - The base URL of the application - in development that would be\n     `http://localhost:4000/` but in production will be your application's\n     URL.\n   - The mount point of the auth routes in your router - we'll assume\n     `/auth`.\n   - The \"subject name\" of the resource being authenticated - we'll assume `user`.\n   - The name of the strategy in your configuration. By default this is\n     `slack`.\n\n   This means that the callback URL should look something like\n   `http://localhost:4000/auth/user/slack/callback`.\n\n   > # HTTPS Required {: .tip}\n   >\n   > Slack won't allow you to register an HTTP URL as the redirect URL, so you\n   > will likely have to add a URL for a service like\n   > [ngrok](https://ngrok.com/)\n\n7. In the \"Scopes\" section, add your user token scopes. You must request `openid`, and may request `email` and\n   `profile` as well.\n8. In the \"OAuth Tokens\" section click \"Install to :workspace:\" where\n   `:workspace:` is the one you selected as the development workspace.\n9. Browse back to the \"Basic Information\".\n10. Copy the \"Client ID\" and \"Client secret\" somewhere safe, we'll need them\n    soon.\n\nNext we can configure our resource (assuming you already have everything else\nset up):\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      slack do\n        client_id MyApp.Secrets\n        redirect_uri MyApp.Secrets\n        client_secret MyApp.Secrets\n      end\n    end\n  end\nend\n```\n\nBecause all the configuration values should be kept secret (ie the\n`client_secret`) or are likely to be different for each environment we use the\n`AshAuthentication.Secret` behaviour to provide them. In this case we're\ndelegating to the OTP application environment, however you may want to use a\nsystem environment variable or some other secret store (eg Vault).\n\n```elixir\ndefmodule MyApp.Secrets do\n  use AshAuthentication.Secret\n\n  def secret_for([:authentication, :strategies, :slack, :client_id], MyApp.Accounts.User, _) do\n    get_config(:client_id)\n  end\n\n  def secret_for([:authentication, :strategies, :slack, :redirect_uri], MyApp.Accounts.User, _) do\n    get_config(:redirect_uri)\n  end\n\n  def secret_for([:authentication, :strategies, :slack, :client_secret], MyApp.Accounts.User, _) do\n    get_config(:client_secret)\n  end\n\n  defp get_config(key) do\n    :my_app\n    |> Application.get_env(:slack, [])\n    |> Keyword.fetch(key)\n  end\nend\n```\n\nThe values for this configuration should be:\n\n- `client_id` - the client ID copied from the Slack settings page.\n- `redirect_uri` - the URL to the generated auth routes in your application\n  (eg `http://localhost:4000/auth`).\n- `client_secret` the client secret copied from the Slack settings page.\n\nLastly, we need to add a register action to your user resource. This is defined\nas an upsert so that it can register new users, or update information for\nreturning users. The default name of the action is `register_with_` followed by\nthe strategy name. In our case that is `register_with_slack`.\n\nThe register action takes two arguments, `user_info` and the `oauth_tokens`.\n\n- `user_info` contains the [`GET /user` response from\n  Slack](https://api.slack.com/authentication/sign-in-with-slack#response)\n  which you can use to populate your user attributes as needed.\n- `oauth_tokens` contains the [`POST /login/oauth/access_token` response from\n  Slack](https://api.slack.com/authentication/sign-in-with-slack#response)\n  - you may want to store these if you intend to call the Slack API on behalf\n    of the user.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  require Ash.Resource.Change.Builtins\n  use Ash.Resource,\n    extensions: [AshAuthentication],\n    domain: MyApp.Accounts\n\n  # ...\n\n  actions do\n    create :register_with_slack do\n      argument :user_info, :map, allow_nil?: false\n      argument :oauth_tokens, :map, allow_nil?: false\n      upsert? true\n      upsert_identity :unique_email\n\n      # Required if you have token generation enabled.\n      change AshAuthentication.GenerateTokenChange\n\n      # Required if you have the `identity_resource` configuration enabled.\n      change AshAuthentication.Strategy.OAuth2.IdentityChange\n\n      change fn changeset, _ ->\n        user_info = Ash.Changeset.get_argument(changeset, :user_info)\n\n        Ash.Changeset.change_attributes(changeset, Map.take(user_info, [\"email\"]))\n      end\n\n      # Required if you're using the password & confirmation strategies\n      upsert_fields []\n      change set_attribute(:confirmed_at, &DateTime.utc_now/0)\n      change after_action(fn _changeset, user, _context ->\n        case user.confirmed_at do\n          nil -> {:error, \"Unconfirmed user exists already\"}\n          _ -> {:ok, user}\n        end\n      end)\n    end\n  end\n\n  # ...\n\nend\n```\n\nEnsure you set the `hashed_password` to `allow_nil?` if you are also using the password strategy.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  # ...\n  attributes do\n    # ...\n    attribute :hashed_password, :string, allow_nil?: true, sensitive?: true\n  end\n  # ...\nend\n```\n\nAnd generate and run migrations in that case.\n\n```bash\nmix ash.codegen make_hashed_password_nullable\nmix ash.migrate\n```","ref":"slack.html"},{"type":"extras","title":"Defining Custom Authentication Strategies","doc":"# Defining Custom Authentication Strategies\n\nAshAuthentication allows you to bring your own authentication strategy without\nhaving to change the Ash Authentication codebase.\n\n> #### Add-on vs Strategy? {:.info}\n>\n> There is functionally no difference between \"add ons\" and \"strategies\" other\n> than where they appear in the DSL. We invented \"add ons\" because it felt\n> weird calling \"confirmation\" an authentication strategy.\n\nThere are several moving parts which must all work together so hold on to your hat!\n\n1. A `Spark.Dsl.Entity` struct. This is used to define the strategy DSL\n   inside the `strategies` (or `add_ons`) section of the `authentication` DSL.\n2. A strategy struct, which stores information about the strategy as\n   configured on a resource which must comply with a few rules.\n3. An optional transformer, which can be used to manipulate the DSL state of\n   the entity and the resource.\n4. An optional verifier, which can be used to verify the DSL state of the\n   entity and the resource after compilation.\n5. The `AshAuthentication.Strategy` protocol, which provides the glue needed\n   for everything to wire up and wrappers around the actions needed to run on\n   the resource.\n\nWe're going to define an extremely dumb strategy which lets anyone with a name\nthat starts with \"Marty\" sign in with just their name. Of course you would\nnever do this in real life, but this isn't real life - it's documentation!","ref":"custom-strategy.html"},{"type":"extras","title":"DSL setup - Defining Custom Authentication Strategies","doc":"Let's start by defining a module for our strategy to live in. Let's call it\n`OnlyMartiesAtTheParty`:\n\n```elixir\ndefmodule OnlyMartiesAtTheParty do\n  use AshAuthentication.Strategy.Custom\nend\n```\n\nSadly, this isn't enough to make the magic happen. We need to define our DSL\nentity by adding it to the `use` statement:\n\n```elixir\ndefmodule OnlyMartiesAtTheParty do\n  @entity %Spark.Dsl.Entity{\n    name: :only_marty,\n    describe: \"Strategy which only allows folks whose name starts with \\\"Marty\\\" to sign in.\",\n    examples: [\n      \"\"\"\n      only_marty do\n        case_sensitive? true\n        name_field :name\n      end\n      \"\"\"\n    ],\n    target: __MODULE__,\n    args: [{:optional, :name, :marty}],\n    schema: [\n      name: [\n        type: :atom,\n        doc: \"\"\"\n        The strategy name.\n        \"\"\",\n        required: true\n      ],\n      case_sensitive?: [\n        type: :boolean,\n        doc: \"\"\"\n        Ignore letter case when comparing?\n        \"\"\",\n        required: false,\n        default: false\n      ],\n      name_field: [\n        type: :atom,\n        doc: \"\"\"\n        The field to check for the users' name.\n        \"\"\",\n        required: true\n      ]\n    ]\n  }\n\n  use AshAuthentication.Strategy.Custom, entity: @entity\nend\n```\n\nIf you haven't you should take a look at the docs for `Spark.Dsl.Entity`, but\nhere's a brief overview of what each field we've set does:\n\n- `name` is the name for which the helper function will be generated in\n  the DSL (ie `only_marty do #... end`).\n- `describe` and `examples` are used when generating documentation.\n- `target` is the name of the module which defines our entity struct. We've\n  set it to `__MODULE__` which means that we'll have to define the struct on\n  this module.\n- `schema` is a keyword list that defines an options schema. See `Spark.Options`.\n\n> By default the entity is added to the `authentication / strategy` DSL, however\n> if you want it in the `authentication / add_ons` DSL instead you can also pass\n> `style: :add_on` in the `use` statement.\n\nNext up, we need to define our struct. The struct should have _at least_ the\nfields named in the entity schema. Additionally, Ash Authentication requires\nthat it have a `resource` field which will be set to the module of the resource\nit's attached to during compilation.\n\n```elixir\ndefmodule OnlyMartiesAtTheParty do\n  defstruct name: :marty, case_sensitive?: false, name_field: nil, resource: nil\n\n  # ...\n\n  use AshAuthentication.Strategy.Custom, entity: @entity\n\n  # other code elided ...\nend\n```\n\nNow it would be theoretically possible to add this custom strategies to your app\nby adding it to the `extensions` section of your resource:\n\n```elixir\ndefmodule MyApp.Accounts.User do\n  use Ash.Resource,\n    extensions: [AshAuthentication, OnlyMartiesAtTheParty],\n    domain: MyApp.Accounts\n\n  authentication do\n    strategies do\n      only_marty do\n        name_field :name\n      end\n    end\n  end\n\n  attributes do\n    uuid_primary_key\n    attribute :name, :string, allow_nil?: false\n  end\nend\n```","ref":"custom-strategy.html#dsl-setup"},{"type":"extras","title":"Implementing the `AshAuthentication.Strategy` protocol - Defining Custom Authentication Strategies","doc":"The Strategy protocol is used to introspect the strategy so that it can\nseamlessly fit in with the rest of Ash Authentication. Here are the key\nconcepts:\n\n- \"phases\" - in terms of HTTP, each strategy is likely to have many phases (eg OAuth 2.0's \"request\" and \"callback\" phases). Essentially you need one phase for each HTTP endpoint you wish to support with your strategy. In our case we just want one sign in endpoint.\n- \"actions\" - actions are exactly as they sound - Resource actions which can be executed by the strategy, whether generated by the strategy (as in the password strategy) or typed in by the user (as in the OAuth 2.0 strategy). The reason that we wrap the strategy's actions this way is that all the built-in strategies (and we hope yours too) allow the user to customise the name of the actions that it uses. At the very least it should probably append the strategy name to the action. Using `Strategy.action/4` allows us to refer these by a more generic name rather than via the user-specified one (eg `:register` vs `:register_with_password`).\n- \"routes\" - `AshAuthentication.Plug` (or [`AshAuthentication.Phoenix.Router.html`](https://hexdocs.pm/ash_authentication_phoenix/AshAuthentication.Phoenix.Router.html)) will generate routes using `Plug.Router` (or [`Phoenix.Router`](https://hexdocs.pm/phoenix/Phoenix.Router.html)) - the `routes/1` callback is used to retrieve this information from the strategy.\n\nGiven this information, let's implement the strategy. It's quite long, so I'm\ngoing to break it up into smaller chunks.\n\n```elixir\ndefimpl AshAuthentication.Strategy, for: OnlyMartiesAtTheParty do\n```\n\nThe `name/1` function is used to uniquely identify the strategy. It _must_ be an\natom and _should_ be the same as the path fragment used in the generated routes.\n\n```elixir\n  def name(strategy), do: strategy.name\n```\n\nSince our strategy only supports sign-in we only need a single `:sign_in` phase\nand action.\n\n```elixir\n  def phases(_), do: [:sign_in]\n  def actions(_), do: [:sign_in]\n```\n\nNext we generate the routes for the strategy. Routes _should_ contain the\nsubject name of the resource being authenticated in case the implementer is\nauthenticating multiple different resources - eg `User` and `Admin`.\n\n```elixir\n  def routes(strategy) do\n    subject_name = AshAuthentication.Info.authentication_subject_name!(strategy.resource)\n\n    [\n      {\"/#{subject_name}/#{strategy.name}\", :sign_in}\n    ]\n  end\n```\n\nWhen generating routes or forms for this phase, what HTTP method should we use?\n\n```elixir\n  def method_for_phase(_, :sign_in), do: :post\n```\n\nNext up, we write our plug. We take the \"name field\" from the input params in\nthe conn and pass them to our sign in action. As long as the action returns\n`{:ok, Ash.Resource.record}` or `{:error, any}` then we can just pass it\nstraight into `store_authentication_result/2` from\n`AshAuthentication.Plug.Helpers`.\n\n```elixir\n  import AshAuthentication.Plug.Helpers, only: [store_authentication_result: 2]\n\n  def plug(strategy, :sign_in, conn) do\n    params = Map.take(conn.params, [to_string(strategy.name_field)])\n    result = action(strategy, :sign_in, params, [])\n    store_authentication_result(conn, result)\n  end\n```\n\nNext, we implement our sign in action. We use `Ash.Query` to find all\nrecords whose name field matches the input, then constrain it to only records\nwhose name field starts with \"Marty\". Depending on whether the name field has a\nunique identity on it we have to deal with it returning zero or more users, or\nan error. When it returns a single user we return that user in an ok tuple,\notherwise we return an authentication failure.\n\nIn this example we're assuming that there is a default `read` action present on\nthe resource.\n\n> #### Warning {: .warning}\n>\n> When it comes to authentication, you never want to reveal to the user what the\n> failure was - this helps prevent [enumeration\n> attacks](https://www.hacksplaining.com/prevention/user-enumeration).\n>\n> You can use `AshAuthentication.Errors.AuthenticationFailed` for this purpose\n> as it will cause `ash_authentication`, `ash_authentication_phoenix`,\n> `ash_graphql` and `ash_json_api` to return the correct HTTP 401 error.\n\n```elixir\n  alias AshAuthentication.Errors.AuthenticationFailed\n  require Ash.Query\n  import Ash.Expr\n\n  def action(strategy, :sign_in, params, options) do\n    name_field = strategy.name_field\n    name = Map.get(params, to_string(name_field))\n    domain = AshAuthentication.Info.domain!(strategy.resource)\n\n    strategy.resource\n    |> Ash.Query.filter(expr(^ref(name_field) == ^name))\n    |> then(fn query ->\n      if strategy.case_sensitive? do\n        Ash.Query.filter(query, like(^ref(name_field), \"Marty%\"))\n      else\n        Ash.Query.filter(query, ilike(^ref(name_field), \"Marty%\"))\n      end\n    end)\n    |> Ash.read(options)\n    |> case do\n      {:ok, [user]} ->\n        {:ok, user}\n\n      {:ok, []} ->\n        {:error, AuthenticationFailed.exception(caused_by: %{reason: :no_user})}\n\n      {:ok, _users} ->\n        {:error, AuthenticationFailed.exception(caused_by: %{reason: :too_many_users})}\n\n      {:error, reason} ->\n        {:error, AuthenticationFailed.exception(caused_by: %{reason: reason})}\n    end\n  end\nend\n```\n\nLastly, we have to implement the `tokens_required?/1` function. This function\nindicates Ash Authentication whether your strategy creates or consumes any\ntokens. Since our strategy does not, we can simply return false:\n\n```elixir\ndef tokens_required?(_), do: false\n```","ref":"custom-strategy.html#implementing-the-ashauthentication-strategy-protocol"},{"type":"extras","title":"Bonus round - transformers and verifiers - Defining Custom Authentication Strategies","doc":"In some cases it may be required for your strategy to modify it's own\nconfiguration or that of the whole resource at compile time. For that you can\ndefine the `transform/2` callback on your strategy module.\n\nAt the very least it is good practice to call\n`AshAuthentication.Strategy.Custom.Helpers.register_strategy_actions/3` so that\nAsh Authentication can keep track of which actions are related to which\nstrategies and `AshAuthentication.Strategy.Custom.Helpers` is automatically\nimported by `use AshAuthentication.Strategy.Custom` for this purpose.","ref":"custom-strategy.html#bonus-round-transformers-and-verifiers"},{"type":"extras","title":"Transformers - Defining Custom Authentication Strategies","doc":"For simple cases where you're just transforming the strategy you can just return\nthe modified strategy and the DSL will be updated accordingly. For example if\nyou wanted to generate the name of an action if the user hasn't specified it:\n\n```elixir\ndef transform(strategy, _dsl_state) do\n  {:ok, Map.put_new(strategy, :sign_in_action_name, :\"sign_in_with_#{strategy.name}\")}\nend\n```\n\nIn some cases you may want to modify the strategy and the resources DSL. In\nthis case you can return the newly mutated DSL state in an ok tuple or an error\ntuple, preferably containing a `Spark.Error.DslError`. For example if we wanted\nto build a sign in action for `OnlyMartiesAtTheParty` to use:\n\n```elixir\ndef transform(strategy, dsl_state) do\n  strategy = Map.put_new(strategy, :sign_in_action_name, :\"sign_in_with_#{strategy.name}\")\n\n  sign_in_action =\n    Spark.Dsl.Transformer.build_entity(Ash.Resource.Dsl, [:actions], :read,\n      name: strategy.sign_in_action_name,\n      accept: [strategy.name_field],\n      get?: true\n    )\n\n  dsl_state =\n    dsl_state\n    |> Spark.Dsl.Transformer.add_entity([:actions], sign_in_action)\n    |> put_strategy(strategy)\n    |> then(fn dsl_state ->\n      register_strategy_actions([strategy.sign_in_action_name], dsl_state, strategy)\n    end)\n\n  {:ok, dsl_state}\nend\n```\n\nTransformers can also be used to validate user input or even directly add code\nto the resource. See the docs for `Spark.Dsl.Transformer` for more information.","ref":"custom-strategy.html#transformers"},{"type":"extras","title":"Verifiers - Defining Custom Authentication Strategies","doc":"We also support a variant of transformers which run in the new `@after_verify`\ncompile hook provided by Elixir 1.14. This is a great place to put checks\nto make sure that the user's configuration makes sense without adding any\ncompile-time dependencies between modules which may cause compiler deadlocks.\n\nFor example, verifying that the \"name\" attribute contains \"marty\" (why you would\ndo this I don't know but I'm running out of sensible examples):\n\n```elixir\ndef verify(strategy, _dsl_state) do\n  if String.contains?(to_string(strategy.name_field), \"marty\") do\n    :ok\n  else\n    {:error,\n      Spark.Error.DslError.exception(\n        path: [:authentication, :strategies, :only_marties],\n        message: \"Option `name_field` must contain \\\"marty\\\"\"\n      )}\n  end\nend\n```","ref":"custom-strategy.html#verifiers"},{"type":"extras","title":"Summary - Defining Custom Authentication Strategies","doc":"You should now have all the tools you need to build custom strategies - and in\nfact the strategies provided by Ash Authentication are built using this system.\n\nIf there is functionality or documentation missing please [raise an\nissue](https://github.com/team-alembic/ash_authentication/issues/new) and we'll\ntake a look at it.\n\nGo forth and strategise!","ref":"custom-strategy.html#summary"},{"type":"extras","title":"Policies on Authenticated Resources","doc":"# Policies on Authenticated Resources\n\nTypically, we want to lock down our `User` resource pretty heavily, which, in Ash, involves writing policies. However, AshAuthentication will be calling actions on your user/token resources. To make this more convenient, all actions run with `AshAuthentication` will set a special context. Additionally a check is provided that will check if that context has been set: `AshAuthentication.Checks.AshAuthenticationInteraction`. Using this you can write a simple bypass policy on your user/token resources like so:\n\n```elixir\npolicies do\n  bypass always() do\n    authorize_if AshAuthentication.Checks.AshAuthenticationInteraction\n  end\n\n  # or, pick your poison\n\n  bypass AshAuthentication.Checks.AshAuthenticationInteraction do\n    authorize_if always()\n  end\nend\n```","ref":"policies-on-authentication-resources.html"},{"type":"extras","title":"Testing","doc":"# Testing\n\nTips and tricks to help test your apps.","ref":"testing.html"},{"type":"extras","title":"When using the Password strategy - Testing","doc":"AshAuthentication uses `bcrypt_elixir` for hashing passwords for secure storage, which by design has a high computational cost. To reduce the cost (make hashing faster), you can reduce the number of computation rounds it performs in tests:\n\n```elixir\n# in config/test.exs\n\n# Do NOT set this value for production\nconfig :bcrypt_elixir, log_rounds: 1\n```","ref":"testing.html#when-using-the-password-strategy"},{"type":"extras","title":"Tokens","doc":"# Tokens","ref":"tokens.html"},{"type":"extras","title":"Token Lifetime - Tokens","doc":"Since refresh tokens are not yet included in `ash_authentication`, you should set the token lifetime to a reasonably long time to ensure a good user experience. Alternatively, refresh tokens can be implemented on your own.","ref":"tokens.html#token-lifetime"},{"type":"extras","title":"Requiring Token Storage - Tokens","doc":"Using `d:AshAuthentication.Dsl.authentication.tokens.require_token_presence_for_authentication?` inverts the token validation behaviour from requiring that tokens are not revoked to requiring any token presented by a client to be present in the token resource to be considered valid.\n\nRequires `store_all_tokens?` to be `true`.\n\n`store_all_tokens?` instructs `AshAuthentication` to keep track of all tokens issued to any user. This is optional behaviour with `ash_authentication` in order to preserve as much performance as possible.","ref":"tokens.html#requiring-token-storage"},{"type":"extras","title":"Sign in Tokens - Tokens","doc":"Enabled with `d:AshAuthentication.Strategy.Password.authentication.strategies.password.sign_in_tokens_enabled?`\n\nSign in tokens can be generated on request by setting the `:token_type` context to `:sign_in` when calling the sign in action. You might do this when you need to generate a short lived token to be exchanged for a real token using the `validate_sign_in_token` route. This is used, for example, by `ash_authentication_phoenix` (since 1.7) to support signing in in a liveview, and then redirecting with a valid token to a controller action, allowing the liveview to show invalid username/password errors.","ref":"tokens.html#sign-in-tokens"},{"type":"extras","title":"Upgrading","doc":"# Upgrading","ref":"upgrading.html"},{"type":"extras","title":"Upgrading to version 4.0.0 - Upgrading","doc":"Version 4.0.0 of AshAuthentication adds support for Ash 3.0 and in line with [a number of changes in Ash](`e:ash:upgrading-to-3.0.html`) there are some corresponding changes to Ash Authentication:\n\n- Token generation is enabled by default, meaning that you will have to explicitly set [`authentication.tokens.enabled?`](documentation/dsls/DSL-AshAuthentication.md#authentication-tokens-enabled?) to `false` if you don't need them.\n\n- Sign in tokens are enabled by default in the password strategy. What this means is that instead of returning a regular user token on sign-in in the user's metadata, we generate a short-lived token which can be used to actually sign the user in. This is specifically to allow live-view based sign-in UIs to display an authentication error without requiring a page-load.","ref":"upgrading.html#upgrading-to-version-4-0-0"},{"type":"extras","title":"Upgrading to version 3.6.0. - Upgrading","doc":"As of version 3.6.0 the `TokenResource` extension adds the `subject` attribute\nwhich allows us to more easily match tokens to specific users. This unlocks\nsome new use-cases (eg sign out everywhere).\n\nThis means that you will need to generate new migrations and migrate your\ndatabase.","ref":"upgrading.html#upgrading-to-version-3-6-0"},{"type":"extras","title":"Upgrade steps: - Upgrading","doc":">","ref":"upgrading.html#upgrade-steps"},{"type":"extras","title":"Warning {: .warning} - Upgrading","doc":">\n> If you already have tokens stored in your database then the migration will\n> likely throw a migration error due to the new `NOT NULL` constraint on\n> `subject`. If this happens then you can either delete all your tokens or\n> explicitly add the `subject` attribute to your resource with `allow_nil?` set\n> to `true`. eg:\n>\n> ```elixir\n> attributes do\n>   attribute :subject, :string, allow_nil?: true\n> end\n> ```\n\n1. Run `mix ash_postgres.generate_migrations --name=add_subject_to_token_resource`\n2. Run `mix ash_postgres.migrate`\n3. 🎉","ref":"upgrading.html#warning-warning"},{"type":"DSL","title":"authentication - dsl-ashauthentication","doc":"Configure authentication for this resource","ref":"dsl-ashauthentication.html#authentication"},{"type":"DSL","title":"authentication.subject_name - dsl-ashauthentication","doc":"The subject name is used anywhere that a short version of your resource name is needed.  Must be unique system-wide and will be inferred from the resource name by default (ie `MyApp.Accounts.User` -> `user`).","ref":"dsl-ashauthentication.html#authentication-subject_name"},{"type":"DSL","title":"authentication.domain - dsl-ashauthentication","doc":"The name of the Ash domain to use to access this resource when doing anything authentication related.","ref":"dsl-ashauthentication.html#authentication-domain"},{"type":"DSL","title":"authentication.get_by_subject_action_name - dsl-ashauthentication","doc":"The name of the read action used to retrieve records. If the action doesn't exist, one will be generated for you.","ref":"dsl-ashauthentication.html#authentication-get_by_subject_action_name"},{"type":"DSL","title":"authentication.select_for_senders - dsl-ashauthentication","doc":"A list of fields that we will ensure are selected whenever a sender will be invoked.  Defaults to `[:email]` if there is an `:email` attribute on the resource, and `[]` otherwise.","ref":"dsl-ashauthentication.html#authentication-select_for_senders"},{"type":"DSL","title":"authentication.tokens - dsl-ashauthentication","doc":"Configure JWT settings for this resource","ref":"dsl-ashauthentication.html#authentication-tokens"},{"type":"DSL","title":"authentication.tokens.enabled? - dsl-ashauthentication","doc":"Should JWTs be generated by this resource?","ref":"dsl-ashauthentication.html#authentication-tokens-enabled?"},{"type":"DSL","title":"authentication.tokens.store_all_tokens? - dsl-ashauthentication","doc":"Store all tokens in the `token_resource`. See the [tokens guide](/documentation/topics/tokens.md) for more.","ref":"dsl-ashauthentication.html#authentication-tokens-store_all_tokens?"},{"type":"DSL","title":"authentication.tokens.require_token_presence_for_authentication? - dsl-ashauthentication","doc":"Require a locally-stored token for authentication. See the [tokens guide](/documentation/topics/tokens.md) for more.","ref":"dsl-ashauthentication.html#authentication-tokens-require_token_presence_for_authentication?"},{"type":"DSL","title":"authentication.tokens.signing_algorithm - dsl-ashauthentication","doc":"The algorithm to use for token signing. Available signing algorithms are; EdDSA, Ed448ph, Ed448, Ed25519ph, Ed25519, PS512, PS384, PS256, ES512, ES384, ES256, RS512, RS384, RS256, HS512, HS384 and HS256.","ref":"dsl-ashauthentication.html#authentication-tokens-signing_algorithm"},{"type":"DSL","title":"authentication.tokens.token_lifetime - dsl-ashauthentication","doc":"How long a token should be valid. See [the tokens guide](/documentation/topics/tokens.md) for more.","ref":"dsl-ashauthentication.html#authentication-tokens-token_lifetime"},{"type":"DSL","title":"authentication.tokens.token_resource - dsl-ashauthentication","doc":"The resource used to store token information, such as in-flight confirmations, revocations, and if `store_all_tokens?` is enabled, authentication tokens themselves.","ref":"dsl-ashauthentication.html#authentication-tokens-token_resource"},{"type":"DSL","title":"authentication.tokens.signing_secret - dsl-ashauthentication","doc":"The secret used to sign tokens.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication.html#authentication-tokens-signing_secret"},{"type":"DSL","title":"authentication.strategies - dsl-ashauthentication","doc":"Configure authentication strategies on this resource","ref":"dsl-ashauthentication.html#authentication-strategies"},{"type":"DSL","title":"authentication.add_ons - dsl-ashauthentication","doc":"Additional add-ons related to, but not providing authentication","ref":"dsl-ashauthentication.html#authentication-add_ons"},{"type":"DSL","title":"authentication.add_ons.confirmation - dsl-ashauthentication-addon-confirmation","doc":"User confirmation flow","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation"},{"type":"DSL","title":"authentication.add_ons.confirmation.name - dsl-ashauthentication-addon-confirmation","doc":"Uniquely identifies the add-on.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-name"},{"type":"DSL","title":"authentication.add_ons.confirmation.token_lifetime - dsl-ashauthentication-addon-confirmation","doc":"How long should the confirmation token be valid.  If no unit is provided, then hours is assumed.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-token_lifetime"},{"type":"DSL","title":"authentication.add_ons.confirmation.monitor_fields - dsl-ashauthentication-addon-confirmation","doc":"A list of fields to monitor for changes. Confirmation will be sent when one of these fields are changed.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-monitor_fields"},{"type":"DSL","title":"authentication.add_ons.confirmation.prevent_hijacking? - dsl-ashauthentication-addon-confirmation","doc":"Whether or not to prevent upserts over unconfirmed uers. See [the confirmation guide](/documentation/topics/confirmation.md) for more.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-prevent_hijacking?"},{"type":"DSL","title":"authentication.add_ons.confirmation.confirmed_at_field - dsl-ashauthentication-addon-confirmation","doc":"The name of the field to store the time that the last confirmation took place. Created if it does not exist.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-confirmed_at_field"},{"type":"DSL","title":"authentication.add_ons.confirmation.confirm_on_create? - dsl-ashauthentication-addon-confirmation","doc":"Generate and send a confirmation token when a new resource is created. Triggers when a create action is executed _and_ one of the monitored fields is being set.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-confirm_on_create?"},{"type":"DSL","title":"authentication.add_ons.confirmation.confirm_on_update? - dsl-ashauthentication-addon-confirmation","doc":"Generate and send a confirmation token when a resource is changed.  Triggers when an update action is executed _and_ one of the monitored fields is being set.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-confirm_on_update?"},{"type":"DSL","title":"authentication.add_ons.confirmation.inhibit_updates? - dsl-ashauthentication-addon-confirmation","doc":"Whether or not to wait until confirmation is received before actually changing a monitored field. See [the confirmation guide](/documentation/topics/confirmation.md) for more.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-inhibit_updates?"},{"type":"DSL","title":"authentication.add_ons.confirmation.auto_confirm_actions - dsl-ashauthentication-addon-confirmation","doc":"A list of actions that should set confirmed_at to `true` automatically. For example, you would likely want to place `:sign_in_with_magic_link` in this list if using magic link.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-auto_confirm_actions"},{"type":"DSL","title":"authentication.add_ons.confirmation.sender - dsl-ashauthentication-addon-confirmation","doc":"How to send the confirmation instructions to the user.","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-sender"},{"type":"DSL","title":"authentication.add_ons.confirmation.confirm_action_name - dsl-ashauthentication-addon-confirmation","doc":"The name of the action to use when performing confirmation. Will be created if it does not already exist. Defaults to confirm_","ref":"dsl-ashauthentication-addon-confirmation.html#authentication-add_ons-confirmation-confirm_action_name"},{"type":"DSL","title":"authentication.strategies.apple - dsl-ashauthentication-strategy-apple","doc":"Provides a pre-configured authentication strategy for [Apple Sign In](https://developer.apple.com/sign-in-with-apple/).\n\nThis strategy is built using the `:oidc` strategy, and thus provides all the same\nconfiguration options should you need them.\n\n## More documentation:\n- The [Apple Sign In Documentation](https://developer.apple.com/documentation/sign_in_with_apple).\n- The [OIDC documentation](`AshAuthentication.Strategy.Oidc`)\n\n#### Strategy defaults:\n\nThe following defaults are applied:\n\n  * `:base_url` is set to `\"https://appleid.apple.com\"`.\n  * `:openid_configuration` is set to `%{\"authorization_endpoint\" => \"https://appleid.apple.com/auth/authorize\", \"issuer\" => \"https://appleid.apple.com\", \"jwks_uri\" => \"https://appleid.apple.com/auth/keys\", \"token_endpoint\" => \"https://appleid.apple.com/auth/token\", \"token_endpoint_auth_methods_supported\" => [\"client_secret_post\"]}`.\n  * `:authorization_params` is set to `[scope: \"email\", response_mode: \"form_post\"]`.\n  * `:client_authentication_method` is set to `\"client_secret_post\"`.\n  * `:openid_default_scope` is set to `nil`.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple"},{"type":"DSL","title":"authentication.strategies.apple.name - dsl-ashauthentication-strategy-apple","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-name"},{"type":"DSL","title":"authentication.strategies.apple.client_id - dsl-ashauthentication-strategy-apple","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-client_id"},{"type":"DSL","title":"authentication.strategies.apple.base_url - dsl-ashauthentication-strategy-apple","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-base_url"},{"type":"DSL","title":"authentication.strategies.apple.site - dsl-ashauthentication-strategy-apple","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-site"},{"type":"DSL","title":"authentication.strategies.apple.prevent_hijacking? - dsl-ashauthentication-strategy-apple","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.apple.auth_method - dsl-ashauthentication-strategy-apple","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-auth_method"},{"type":"DSL","title":"authentication.strategies.apple.client_secret - dsl-ashauthentication-strategy-apple","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-client_secret"},{"type":"DSL","title":"authentication.strategies.apple.trusted_audiences - dsl-ashauthentication-strategy-apple","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.apple.private_key - dsl-ashauthentication-strategy-apple","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-private_key"},{"type":"DSL","title":"authentication.strategies.apple.redirect_uri - dsl-ashauthentication-strategy-apple","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-redirect_uri"},{"type":"DSL","title":"authentication.strategies.apple.code_verifier - dsl-ashauthentication-strategy-apple","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-code_verifier"},{"type":"DSL","title":"authentication.strategies.apple.authorization_params - dsl-ashauthentication-strategy-apple","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-authorization_params"},{"type":"DSL","title":"authentication.strategies.apple.registration_enabled? - dsl-ashauthentication-strategy-apple","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.apple.register_action_name - dsl-ashauthentication-strategy-apple","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-register_action_name"},{"type":"DSL","title":"authentication.strategies.apple.sign_in_action_name - dsl-ashauthentication-strategy-apple","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.apple.identity_resource - dsl-ashauthentication-strategy-apple","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-identity_resource"},{"type":"DSL","title":"authentication.strategies.apple.identity_relationship_name - dsl-ashauthentication-strategy-apple","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.apple.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-apple","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.apple.icon - dsl-ashauthentication-strategy-apple","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-icon"},{"type":"DSL","title":"authentication.strategies.apple.openid_configuration_uri - dsl-ashauthentication-strategy-apple","doc":"The URI for the OpenID provider","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-openid_configuration_uri"},{"type":"DSL","title":"authentication.strategies.apple.client_authentication_method - dsl-ashauthentication-strategy-apple","doc":"The client authentication method to use.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-client_authentication_method"},{"type":"DSL","title":"authentication.strategies.apple.openid_configuration - dsl-ashauthentication-strategy-apple","doc":"The OpenID configuration.  If not set, the configuration will be retrieved from `openid_configuration_uri`.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-openid_configuration"},{"type":"DSL","title":"authentication.strategies.apple.id_token_signed_response_alg - dsl-ashauthentication-strategy-apple","doc":"The `id_token_signed_response_alg` parameter sent by the Client during Registration.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-id_token_signed_response_alg"},{"type":"DSL","title":"authentication.strategies.apple.id_token_ttl_seconds - dsl-ashauthentication-strategy-apple","doc":"The number of seconds from `iat` that an ID Token will be considered valid.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-id_token_ttl_seconds"},{"type":"DSL","title":"authentication.strategies.apple.nonce - dsl-ashauthentication-strategy-apple","doc":"A function for generating the session nonce, `true` to automatically generate it with `AshAuthentication.Strategy.Oidc.NonceGenerator`, or `false` to disable.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-nonce"},{"type":"DSL","title":"authentication.strategies.apple.team_id - dsl-ashauthentication-strategy-apple","doc":"The Apple team ID associated with the application.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-team_id"},{"type":"DSL","title":"authentication.strategies.apple.private_key_id - dsl-ashauthentication-strategy-apple","doc":"The private key ID used for signing the JWT token.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-private_key_id"},{"type":"DSL","title":"authentication.strategies.apple.private_key_path - dsl-ashauthentication-strategy-apple","doc":"The path to the private key file used for signing the JWT token.","ref":"dsl-ashauthentication-strategy-apple.html#authentication-strategies-apple-private_key_path"},{"type":"DSL","title":"authentication.strategies.auth0 - dsl-ashauthentication-strategy-auth0","doc":"Provides a pre-configured authentication strategy for [Auth0](https://auth0.com/).\n\nThis strategy is built using the `:oauth2` strategy, and thus provides all the same\nconfiguration options should you need them.\n\n#### More documentation:\n- The [Auth0 Tutorial](/documentation/tutorial/auth0.md).\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)\n\n#### Strategy defaults:\n\nThe following defaults are applied:\n\n  * `:authorize_url` is set to `\"/authorize\"`.\n  * `:token_url` is set to `\"/oauth/token\"`.\n  * `:user_url` is set to `\"/userinfo\"`.\n  * `:authorization_params` is set to `[scope: \"openid profile email\"]`.\n  * `:auth_method` is set to `:client_secret_post`.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0"},{"type":"DSL","title":"authentication.strategies.auth0.name - dsl-ashauthentication-strategy-auth0","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-name"},{"type":"DSL","title":"authentication.strategies.auth0.client_id - dsl-ashauthentication-strategy-auth0","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-client_id"},{"type":"DSL","title":"authentication.strategies.auth0.base_url - dsl-ashauthentication-strategy-auth0","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-base_url"},{"type":"DSL","title":"authentication.strategies.auth0.site - dsl-ashauthentication-strategy-auth0","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-site"},{"type":"DSL","title":"authentication.strategies.auth0.prevent_hijacking? - dsl-ashauthentication-strategy-auth0","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.auth0.auth_method - dsl-ashauthentication-strategy-auth0","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-auth_method"},{"type":"DSL","title":"authentication.strategies.auth0.client_secret - dsl-ashauthentication-strategy-auth0","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-client_secret"},{"type":"DSL","title":"authentication.strategies.auth0.authorize_url - dsl-ashauthentication-strategy-auth0","doc":"The API url to the OAuth2 authorize endpoint, relative to `site`, e.g `authorize_url fn _, _ -> {:ok, \"https://exampe.com/authorize\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-authorize_url"},{"type":"DSL","title":"authentication.strategies.auth0.token_url - dsl-ashauthentication-strategy-auth0","doc":"The API url to access the token endpoint, relative to `site`, e.g `token_url fn _, _ -> {:ok, \"https://example.com/oauth_token\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-token_url"},{"type":"DSL","title":"authentication.strategies.auth0.trusted_audiences - dsl-ashauthentication-strategy-auth0","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.auth0.user_url - dsl-ashauthentication-strategy-auth0","doc":"The API url to access the user endpoint, relative to `site`, e.g `user_url fn _, _ -> {:ok, \"https://example.com/userinfo\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-user_url"},{"type":"DSL","title":"authentication.strategies.auth0.private_key - dsl-ashauthentication-strategy-auth0","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-private_key"},{"type":"DSL","title":"authentication.strategies.auth0.redirect_uri - dsl-ashauthentication-strategy-auth0","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-redirect_uri"},{"type":"DSL","title":"authentication.strategies.auth0.code_verifier - dsl-ashauthentication-strategy-auth0","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-code_verifier"},{"type":"DSL","title":"authentication.strategies.auth0.authorization_params - dsl-ashauthentication-strategy-auth0","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-authorization_params"},{"type":"DSL","title":"authentication.strategies.auth0.registration_enabled? - dsl-ashauthentication-strategy-auth0","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.auth0.register_action_name - dsl-ashauthentication-strategy-auth0","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-register_action_name"},{"type":"DSL","title":"authentication.strategies.auth0.sign_in_action_name - dsl-ashauthentication-strategy-auth0","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.auth0.identity_resource - dsl-ashauthentication-strategy-auth0","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-identity_resource"},{"type":"DSL","title":"authentication.strategies.auth0.identity_relationship_name - dsl-ashauthentication-strategy-auth0","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.auth0.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-auth0","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.auth0.icon - dsl-ashauthentication-strategy-auth0","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-auth0.html#authentication-strategies-auth0-icon"},{"type":"DSL","title":"authentication.strategies.github - dsl-ashauthentication-strategy-github","doc":"Provides a pre-configured authentication strategy for [GitHub](https://github.com/).\n\nThis strategy is built using the `:oauth2` strategy, and thus provides all the same\nconfiguration options should you need them.\n\n#### More documentation:\n- The [GitHub Tutorial](/documentation/tutorial/github.md).\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)\n\n#### Strategy defaults:\n\nThe following defaults are applied:\n\n  * `:base_url` is set to `\"https://api.github.com\"`.\n  * `:authorize_url` is set to `\"https://github.com/login/oauth/authorize\"`.\n  * `:token_url` is set to `\"https://github.com/login/oauth/access_token\"`.\n  * `:user_url` is set to `\"/user\"`.\n  * `:user_emails_url` is set to `\"/user/emails\"`.\n  * `:authorization_params` is set to `[scope: \"read:user,user:email\"]`.\n  * `:auth_method` is set to `:client_secret_post`.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github"},{"type":"DSL","title":"authentication.strategies.github.name - dsl-ashauthentication-strategy-github","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-name"},{"type":"DSL","title":"authentication.strategies.github.client_id - dsl-ashauthentication-strategy-github","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-client_id"},{"type":"DSL","title":"authentication.strategies.github.base_url - dsl-ashauthentication-strategy-github","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-base_url"},{"type":"DSL","title":"authentication.strategies.github.site - dsl-ashauthentication-strategy-github","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-site"},{"type":"DSL","title":"authentication.strategies.github.prevent_hijacking? - dsl-ashauthentication-strategy-github","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.github.auth_method - dsl-ashauthentication-strategy-github","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-auth_method"},{"type":"DSL","title":"authentication.strategies.github.client_secret - dsl-ashauthentication-strategy-github","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-client_secret"},{"type":"DSL","title":"authentication.strategies.github.authorize_url - dsl-ashauthentication-strategy-github","doc":"The API url to the OAuth2 authorize endpoint, relative to `site`, e.g `authorize_url fn _, _ -> {:ok, \"https://exampe.com/authorize\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-authorize_url"},{"type":"DSL","title":"authentication.strategies.github.token_url - dsl-ashauthentication-strategy-github","doc":"The API url to access the token endpoint, relative to `site`, e.g `token_url fn _, _ -> {:ok, \"https://example.com/oauth_token\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-token_url"},{"type":"DSL","title":"authentication.strategies.github.trusted_audiences - dsl-ashauthentication-strategy-github","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.github.user_url - dsl-ashauthentication-strategy-github","doc":"The API url to access the user endpoint, relative to `site`, e.g `user_url fn _, _ -> {:ok, \"https://example.com/userinfo\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-user_url"},{"type":"DSL","title":"authentication.strategies.github.private_key - dsl-ashauthentication-strategy-github","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-private_key"},{"type":"DSL","title":"authentication.strategies.github.redirect_uri - dsl-ashauthentication-strategy-github","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-redirect_uri"},{"type":"DSL","title":"authentication.strategies.github.code_verifier - dsl-ashauthentication-strategy-github","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-code_verifier"},{"type":"DSL","title":"authentication.strategies.github.authorization_params - dsl-ashauthentication-strategy-github","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-authorization_params"},{"type":"DSL","title":"authentication.strategies.github.registration_enabled? - dsl-ashauthentication-strategy-github","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.github.register_action_name - dsl-ashauthentication-strategy-github","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-register_action_name"},{"type":"DSL","title":"authentication.strategies.github.sign_in_action_name - dsl-ashauthentication-strategy-github","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.github.identity_resource - dsl-ashauthentication-strategy-github","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-identity_resource"},{"type":"DSL","title":"authentication.strategies.github.identity_relationship_name - dsl-ashauthentication-strategy-github","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.github.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-github","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.github.icon - dsl-ashauthentication-strategy-github","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-github.html#authentication-strategies-github-icon"},{"type":"DSL","title":"authentication.strategies.google - dsl-ashauthentication-strategy-google","doc":"Provides a pre-configured authentication strategy for [Google](https://google.com/).\n\nThis strategy is built using the `:oauth2` strategy, and thus provides all the same\nconfiguration options should you need them.\n\n## More documentation:\n- The [Google OAuth 2.0 Overview](https://developers.google.com/identity/protocols/oauth2).\n- The [Google Tutorial](/documentation/tutorial/google.md)\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)\n\n#### Strategy defaults:\n\nThe following defaults are applied:\n\n  * `:base_url` is set to `\"https://www.googleapis.com\"`.\n  * `:authorize_url` is set to `\"https://accounts.google.com/o/oauth2/v2/auth\"`.\n  * `:token_url` is set to `\"/oauth2/v4/token\"`.\n  * `:user_url` is set to `\"/oauth2/v3/userinfo\"`.\n  * `:authorization_params` is set to `[scope: \"https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile\"]`.\n  * `:auth_method` is set to `:client_secret_post`.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google"},{"type":"DSL","title":"authentication.strategies.google.name - dsl-ashauthentication-strategy-google","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-name"},{"type":"DSL","title":"authentication.strategies.google.client_id - dsl-ashauthentication-strategy-google","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-client_id"},{"type":"DSL","title":"authentication.strategies.google.base_url - dsl-ashauthentication-strategy-google","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-base_url"},{"type":"DSL","title":"authentication.strategies.google.site - dsl-ashauthentication-strategy-google","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-site"},{"type":"DSL","title":"authentication.strategies.google.prevent_hijacking? - dsl-ashauthentication-strategy-google","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.google.auth_method - dsl-ashauthentication-strategy-google","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-auth_method"},{"type":"DSL","title":"authentication.strategies.google.client_secret - dsl-ashauthentication-strategy-google","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-client_secret"},{"type":"DSL","title":"authentication.strategies.google.authorize_url - dsl-ashauthentication-strategy-google","doc":"The API url to the OAuth2 authorize endpoint, relative to `site`, e.g `authorize_url fn _, _ -> {:ok, \"https://exampe.com/authorize\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-authorize_url"},{"type":"DSL","title":"authentication.strategies.google.token_url - dsl-ashauthentication-strategy-google","doc":"The API url to access the token endpoint, relative to `site`, e.g `token_url fn _, _ -> {:ok, \"https://example.com/oauth_token\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-token_url"},{"type":"DSL","title":"authentication.strategies.google.trusted_audiences - dsl-ashauthentication-strategy-google","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.google.user_url - dsl-ashauthentication-strategy-google","doc":"The API url to access the user endpoint, relative to `site`, e.g `user_url fn _, _ -> {:ok, \"https://example.com/userinfo\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-user_url"},{"type":"DSL","title":"authentication.strategies.google.private_key - dsl-ashauthentication-strategy-google","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-private_key"},{"type":"DSL","title":"authentication.strategies.google.redirect_uri - dsl-ashauthentication-strategy-google","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-redirect_uri"},{"type":"DSL","title":"authentication.strategies.google.code_verifier - dsl-ashauthentication-strategy-google","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-code_verifier"},{"type":"DSL","title":"authentication.strategies.google.authorization_params - dsl-ashauthentication-strategy-google","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-authorization_params"},{"type":"DSL","title":"authentication.strategies.google.registration_enabled? - dsl-ashauthentication-strategy-google","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.google.register_action_name - dsl-ashauthentication-strategy-google","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-register_action_name"},{"type":"DSL","title":"authentication.strategies.google.sign_in_action_name - dsl-ashauthentication-strategy-google","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.google.identity_resource - dsl-ashauthentication-strategy-google","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-identity_resource"},{"type":"DSL","title":"authentication.strategies.google.identity_relationship_name - dsl-ashauthentication-strategy-google","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.google.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-google","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.google.icon - dsl-ashauthentication-strategy-google","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-google.html#authentication-strategies-google-icon"},{"type":"DSL","title":"authentication.strategies.magic_link - dsl-ashauthentication-strategy-magiclink","doc":"Strategy for authenticating using local users with a magic link","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link"},{"type":"DSL","title":"authentication.strategies.magic_link.name - dsl-ashauthentication-strategy-magiclink","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-name"},{"type":"DSL","title":"authentication.strategies.magic_link.identity_field - dsl-ashauthentication-strategy-magiclink","doc":"The name of the attribute which uniquely identifies the user, usually something like `username` or `email_address`.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-identity_field"},{"type":"DSL","title":"authentication.strategies.magic_link.token_lifetime - dsl-ashauthentication-strategy-magiclink","doc":"How long the sign in token is valid.  If no unit is provided, then `minutes` is assumed.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-token_lifetime"},{"type":"DSL","title":"authentication.strategies.magic_link.prevent_hijacking? - dsl-ashauthentication-strategy-magiclink","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.magic_link.request_action_name - dsl-ashauthentication-strategy-magiclink","doc":"The name to use for the request action. Defaults to `request_ `","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-request_action_name"},{"type":"DSL","title":"authentication.strategies.magic_link.lookup_action_name - dsl-ashauthentication-strategy-magiclink","doc":"The action to use when looking up a user by their identity. Defaults to `get_by_ `","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-lookup_action_name"},{"type":"DSL","title":"authentication.strategies.magic_link.single_use_token? - dsl-ashauthentication-strategy-magiclink","doc":"Automatically revoke the token once it's been used for sign in.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-single_use_token?"},{"type":"DSL","title":"authentication.strategies.magic_link.registration_enabled? - dsl-ashauthentication-strategy-magiclink","doc":"Allows registering via magic link. Signing in with magic link becomes an upsert action instead of a read action.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.magic_link.sign_in_action_name - dsl-ashauthentication-strategy-magiclink","doc":"The name to use for the sign in action. Defaults to `sign_in_with_ `","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.magic_link.token_param_name - dsl-ashauthentication-strategy-magiclink","doc":"The name of the token parameter in the incoming sign-in request.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-token_param_name"},{"type":"DSL","title":"authentication.strategies.magic_link.sender - dsl-ashauthentication-strategy-magiclink","doc":"How to send the magic link to the user.","ref":"dsl-ashauthentication-strategy-magiclink.html#authentication-strategies-magic_link-sender"},{"type":"DSL","title":"authentication.strategies.oauth2 - dsl-ashauthentication-strategy-oauth2","doc":"OAuth2 authentication","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2"},{"type":"DSL","title":"authentication.strategies.oauth2.name - dsl-ashauthentication-strategy-oauth2","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-name"},{"type":"DSL","title":"authentication.strategies.oauth2.client_id - dsl-ashauthentication-strategy-oauth2","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-client_id"},{"type":"DSL","title":"authentication.strategies.oauth2.base_url - dsl-ashauthentication-strategy-oauth2","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-base_url"},{"type":"DSL","title":"authentication.strategies.oauth2.site - dsl-ashauthentication-strategy-oauth2","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-site"},{"type":"DSL","title":"authentication.strategies.oauth2.prevent_hijacking? - dsl-ashauthentication-strategy-oauth2","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.oauth2.auth_method - dsl-ashauthentication-strategy-oauth2","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-auth_method"},{"type":"DSL","title":"authentication.strategies.oauth2.client_secret - dsl-ashauthentication-strategy-oauth2","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-client_secret"},{"type":"DSL","title":"authentication.strategies.oauth2.authorize_url - dsl-ashauthentication-strategy-oauth2","doc":"The API url to the OAuth2 authorize endpoint, relative to `site`, e.g `authorize_url fn _, _ -> {:ok, \"https://exampe.com/authorize\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-authorize_url"},{"type":"DSL","title":"authentication.strategies.oauth2.token_url - dsl-ashauthentication-strategy-oauth2","doc":"The API url to access the token endpoint, relative to `site`, e.g `token_url fn _, _ -> {:ok, \"https://example.com/oauth_token\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-token_url"},{"type":"DSL","title":"authentication.strategies.oauth2.trusted_audiences - dsl-ashauthentication-strategy-oauth2","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.oauth2.user_url - dsl-ashauthentication-strategy-oauth2","doc":"The API url to access the user endpoint, relative to `site`, e.g `user_url fn _, _ -> {:ok, \"https://example.com/userinfo\"} end`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-user_url"},{"type":"DSL","title":"authentication.strategies.oauth2.private_key - dsl-ashauthentication-strategy-oauth2","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-private_key"},{"type":"DSL","title":"authentication.strategies.oauth2.redirect_uri - dsl-ashauthentication-strategy-oauth2","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-redirect_uri"},{"type":"DSL","title":"authentication.strategies.oauth2.code_verifier - dsl-ashauthentication-strategy-oauth2","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-code_verifier"},{"type":"DSL","title":"authentication.strategies.oauth2.authorization_params - dsl-ashauthentication-strategy-oauth2","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-authorization_params"},{"type":"DSL","title":"authentication.strategies.oauth2.registration_enabled? - dsl-ashauthentication-strategy-oauth2","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.oauth2.register_action_name - dsl-ashauthentication-strategy-oauth2","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-register_action_name"},{"type":"DSL","title":"authentication.strategies.oauth2.sign_in_action_name - dsl-ashauthentication-strategy-oauth2","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.oauth2.identity_resource - dsl-ashauthentication-strategy-oauth2","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-identity_resource"},{"type":"DSL","title":"authentication.strategies.oauth2.identity_relationship_name - dsl-ashauthentication-strategy-oauth2","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.oauth2.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-oauth2","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.oauth2.icon - dsl-ashauthentication-strategy-oauth2","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-oauth2.html#authentication-strategies-oauth2-icon"},{"type":"DSL","title":"authentication.strategies.oidc - dsl-ashauthentication-strategy-oidc","doc":"Provides an OpenID Connect authentication strategy.\n\nThis strategy is built using the `:oauth2` strategy, and thus provides\nall the same configuration options should you need them.\n\n#### More documentation:\n- The [OAuth2 documentation](`AshAuthentication.Strategy.OAuth2`)","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc"},{"type":"DSL","title":"authentication.strategies.oidc.name - dsl-ashauthentication-strategy-oidc","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-name"},{"type":"DSL","title":"authentication.strategies.oidc.client_id - dsl-ashauthentication-strategy-oidc","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-client_id"},{"type":"DSL","title":"authentication.strategies.oidc.base_url - dsl-ashauthentication-strategy-oidc","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-base_url"},{"type":"DSL","title":"authentication.strategies.oidc.site - dsl-ashauthentication-strategy-oidc","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-site"},{"type":"DSL","title":"authentication.strategies.oidc.prevent_hijacking? - dsl-ashauthentication-strategy-oidc","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.oidc.auth_method - dsl-ashauthentication-strategy-oidc","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-auth_method"},{"type":"DSL","title":"authentication.strategies.oidc.client_secret - dsl-ashauthentication-strategy-oidc","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-client_secret"},{"type":"DSL","title":"authentication.strategies.oidc.trusted_audiences - dsl-ashauthentication-strategy-oidc","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.oidc.private_key - dsl-ashauthentication-strategy-oidc","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-private_key"},{"type":"DSL","title":"authentication.strategies.oidc.redirect_uri - dsl-ashauthentication-strategy-oidc","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-redirect_uri"},{"type":"DSL","title":"authentication.strategies.oidc.code_verifier - dsl-ashauthentication-strategy-oidc","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-code_verifier"},{"type":"DSL","title":"authentication.strategies.oidc.authorization_params - dsl-ashauthentication-strategy-oidc","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-authorization_params"},{"type":"DSL","title":"authentication.strategies.oidc.registration_enabled? - dsl-ashauthentication-strategy-oidc","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.oidc.register_action_name - dsl-ashauthentication-strategy-oidc","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-register_action_name"},{"type":"DSL","title":"authentication.strategies.oidc.sign_in_action_name - dsl-ashauthentication-strategy-oidc","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.oidc.identity_resource - dsl-ashauthentication-strategy-oidc","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-identity_resource"},{"type":"DSL","title":"authentication.strategies.oidc.identity_relationship_name - dsl-ashauthentication-strategy-oidc","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.oidc.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-oidc","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.oidc.icon - dsl-ashauthentication-strategy-oidc","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-icon"},{"type":"DSL","title":"authentication.strategies.oidc.openid_configuration_uri - dsl-ashauthentication-strategy-oidc","doc":"The URI for the OpenID provider","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-openid_configuration_uri"},{"type":"DSL","title":"authentication.strategies.oidc.client_authentication_method - dsl-ashauthentication-strategy-oidc","doc":"The client authentication method to use.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-client_authentication_method"},{"type":"DSL","title":"authentication.strategies.oidc.openid_configuration - dsl-ashauthentication-strategy-oidc","doc":"The OpenID configuration.  If not set, the configuration will be retrieved from `openid_configuration_uri`.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-openid_configuration"},{"type":"DSL","title":"authentication.strategies.oidc.id_token_signed_response_alg - dsl-ashauthentication-strategy-oidc","doc":"The `id_token_signed_response_alg` parameter sent by the Client during Registration.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-id_token_signed_response_alg"},{"type":"DSL","title":"authentication.strategies.oidc.id_token_ttl_seconds - dsl-ashauthentication-strategy-oidc","doc":"The number of seconds from `iat` that an ID Token will be considered valid.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-id_token_ttl_seconds"},{"type":"DSL","title":"authentication.strategies.oidc.nonce - dsl-ashauthentication-strategy-oidc","doc":"A function for generating the session nonce, `true` to automatically generate it with `AshAuthentication.Strategy.Oidc.NonceGenerator`, or `false` to disable.","ref":"dsl-ashauthentication-strategy-oidc.html#authentication-strategies-oidc-nonce"},{"type":"DSL","title":"authentication.strategies.password - dsl-ashauthentication-strategy-password","doc":"Strategy for authenticating using local resources as the source of truth.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password"},{"type":"DSL","title":"authentication.strategies.password.name - dsl-ashauthentication-strategy-password","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-name"},{"type":"DSL","title":"authentication.strategies.password.identity_field - dsl-ashauthentication-strategy-password","doc":"The name of the attribute which uniquely identifies the user, usually something like `username` or `email_address`.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-identity_field"},{"type":"DSL","title":"authentication.strategies.password.hashed_password_field - dsl-ashauthentication-strategy-password","doc":"The name of the attribute within which to store the user's password once it has been hashed.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-hashed_password_field"},{"type":"DSL","title":"authentication.strategies.password.hash_provider - dsl-ashauthentication-strategy-password","doc":"A module which implements the `AshAuthentication.HashProvider` behaviour, to provide cryptographic hashing of passwords.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-hash_provider"},{"type":"DSL","title":"authentication.strategies.password.confirmation_required? - dsl-ashauthentication-strategy-password","doc":"Whether a password confirmation field is required when registering or changing passwords.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-confirmation_required?"},{"type":"DSL","title":"authentication.strategies.password.register_action_accept - dsl-ashauthentication-strategy-password","doc":"A list of additional fields to be accepted in the register action.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-register_action_accept"},{"type":"DSL","title":"authentication.strategies.password.password_field - dsl-ashauthentication-strategy-password","doc":"The name of the argument used to collect the user's password in plaintext when registering, checking or changing passwords.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-password_field"},{"type":"DSL","title":"authentication.strategies.password.password_confirmation_field - dsl-ashauthentication-strategy-password","doc":"The name of the argument used to confirm the user's password in plaintext when registering or changing passwords.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-password_confirmation_field"},{"type":"DSL","title":"authentication.strategies.password.register_action_name - dsl-ashauthentication-strategy-password","doc":"The name to use for the register action. Defaults to `register_with_ `","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-register_action_name"},{"type":"DSL","title":"authentication.strategies.password.registration_enabled? - dsl-ashauthentication-strategy-password","doc":"If you do not want new users to be able to register using this strategy, set this to false.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.password.sign_in_action_name - dsl-ashauthentication-strategy-password","doc":"The name to use for the sign in action. Defaults to `sign_in_with_ `","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.password.sign_in_enabled? - dsl-ashauthentication-strategy-password","doc":"If you do not want new users to be able to sign in using this strategy, set this to false.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-sign_in_enabled?"},{"type":"DSL","title":"authentication.strategies.password.sign_in_tokens_enabled? - dsl-ashauthentication-strategy-password","doc":"Whether or not to support generating short lived sign in tokens. Requires the resource to have tokens enabled.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-sign_in_tokens_enabled?"},{"type":"DSL","title":"authentication.strategies.password.sign_in_token_lifetime - dsl-ashauthentication-strategy-password","doc":"A lifetime for which a generated sign in token will be valid, if `sign_in_tokens_enabled?`. Unit defaults to `:seconds`.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-sign_in_token_lifetime"},{"type":"DSL","title":"authentication.strategies.password.require_confirmed_with - dsl-ashauthentication-strategy-password","doc":"Whether a new account must be confirmed in order to log in.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-password-require_confirmed_with"},{"type":"DSL","title":"authentication.strategies.resettable.resettable - dsl-ashauthentication-strategy-password","doc":"Configure password reset options for the resource","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-resettable-resettable"},{"type":"DSL","title":"authentication.strategies.resettable.resettable.token_lifetime - dsl-ashauthentication-strategy-password","doc":"How long should the reset token be valid.  If no unit is provided `:hours` is assumed.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-resettable-resettable-token_lifetime"},{"type":"DSL","title":"authentication.strategies.resettable.resettable.request_password_reset_action_name - dsl-ashauthentication-strategy-password","doc":"The name to use for the action which generates a password reset token. Defaults to `request_password_reset_with_ `.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-resettable-resettable-request_password_reset_action_name"},{"type":"DSL","title":"authentication.strategies.resettable.resettable.password_reset_action_name - dsl-ashauthentication-strategy-password","doc":"The name to use for the action which actually resets the user's password. Defaults to `password_reset_with_ `.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-resettable-resettable-password_reset_action_name"},{"type":"DSL","title":"authentication.strategies.resettable.resettable.sender - dsl-ashauthentication-strategy-password","doc":"The sender to use when sending password reset instructions.","ref":"dsl-ashauthentication-strategy-password.html#authentication-strategies-resettable-resettable-sender"},{"type":"DSL","title":"authentication.strategies.slack - dsl-ashauthentication-strategy-slack","doc":"Provides a pre-configured authentication strategy for [Slack](https://slack.com/).\n\nThis strategy is built using the `:oauth2` strategy, and thus provides all the same\nconfiguration options should you need them.\n\n#### More documentation:\n- The [Slack Tutorial](/documentation/tutorial/slack.md).\n- The [OIDC documentation](`AshAuthentication.Strategy.Oidc`)\n\n#### Strategy defaults:\n\nThe following defaults are applied:\n\n  * `:base_url` is set to `\"https://slack.com\"`.\n  * `:authorization_params` is set to `[scope: \"openid email profile\"]`.\n  * `:client_authentication_method` is set to `\"client_secret_post\"`.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack"},{"type":"DSL","title":"authentication.strategies.slack.team_id - dsl-ashauthentication-strategy-slack","doc":"The team id to restrict authorization for.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-team_id"},{"type":"DSL","title":"authentication.strategies.slack.name - dsl-ashauthentication-strategy-slack","doc":"Uniquely identifies the strategy.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-name"},{"type":"DSL","title":"authentication.strategies.slack.client_id - dsl-ashauthentication-strategy-slack","doc":"The OAuth2 client ID.  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-client_id"},{"type":"DSL","title":"authentication.strategies.slack.base_url - dsl-ashauthentication-strategy-slack","doc":"The base URL of the OAuth2 server - including the leading protocol (ie `https://`).  Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-base_url"},{"type":"DSL","title":"authentication.strategies.slack.site - dsl-ashauthentication-strategy-slack","doc":"Deprecated: Use `base_url` instead.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-site"},{"type":"DSL","title":"authentication.strategies.slack.prevent_hijacking? - dsl-ashauthentication-strategy-slack","doc":"Requires a confirmation add_on to be present if the password strategy is used with the same identity_field.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-prevent_hijacking?"},{"type":"DSL","title":"authentication.strategies.slack.auth_method - dsl-ashauthentication-strategy-slack","doc":"The authentication strategy used, optional. If not set, no authentication will be used during the access token request.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-auth_method"},{"type":"DSL","title":"authentication.strategies.slack.client_secret - dsl-ashauthentication-strategy-slack","doc":"The OAuth2 client secret. Required if :auth_method is `:client_secret_basic`, `:client_secret_post` or `:client_secret_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-client_secret"},{"type":"DSL","title":"authentication.strategies.slack.trusted_audiences - dsl-ashauthentication-strategy-slack","doc":"A list of audiences which are trusted. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-trusted_audiences"},{"type":"DSL","title":"authentication.strategies.slack.private_key - dsl-ashauthentication-strategy-slack","doc":"The private key to use if `:auth_method` is `:private_key_jwt`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-private_key"},{"type":"DSL","title":"authentication.strategies.slack.redirect_uri - dsl-ashauthentication-strategy-slack","doc":"The callback URI *base*. Not the whole URI back to the callback endpoint, but the URI to your `AuthPlug`. Takes either a module which implements the `AshAuthentication.Secret` behaviour, a 2 arity anonymous function or a string.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-redirect_uri"},{"type":"DSL","title":"authentication.strategies.slack.code_verifier - dsl-ashauthentication-strategy-slack","doc":"Boolean to generate and use a random 128 byte long url safe code verifier for PKCE flow, optional, defaults to false. When set to true the session params will contain :code_verifier, :code_challenge, and :code_challenge_method params","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-code_verifier"},{"type":"DSL","title":"authentication.strategies.slack.authorization_params - dsl-ashauthentication-strategy-slack","doc":"Any additional parameters to encode in the request phase. eg: `authorization_params scope: \"openid profile email\"`","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-authorization_params"},{"type":"DSL","title":"authentication.strategies.slack.registration_enabled? - dsl-ashauthentication-strategy-slack","doc":"If enabled, new users will be able to register for your site when authenticating and not already present. If not, only existing users will be able to authenticate.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-registration_enabled?"},{"type":"DSL","title":"authentication.strategies.slack.register_action_name - dsl-ashauthentication-strategy-slack","doc":"The name of the action to use to register a user, if `registration_enabled?` is `true`. Defaults to `register_with_ ` See the \"Registration and Sign-in\" section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-register_action_name"},{"type":"DSL","title":"authentication.strategies.slack.sign_in_action_name - dsl-ashauthentication-strategy-slack","doc":"The name of the action to use to sign in an existing user, if `sign_in_enabled?` is `true`. Defaults to `sign_in_with_ `, which is generated for you by default. See the \"Registration and Sign-in\" section of the strategy docs for more information.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-sign_in_action_name"},{"type":"DSL","title":"authentication.strategies.slack.identity_resource - dsl-ashauthentication-strategy-slack","doc":"The resource used to store user identities, or `false` to disable. See the User Identities section of the strategy docs for more.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-identity_resource"},{"type":"DSL","title":"authentication.strategies.slack.identity_relationship_name - dsl-ashauthentication-strategy-slack","doc":"Name of the relationship to the provider identities resource","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-identity_relationship_name"},{"type":"DSL","title":"authentication.strategies.slack.identity_relationship_user_id_attribute - dsl-ashauthentication-strategy-slack","doc":"The name of the destination (user_id) attribute on your provider identity resource. Only necessary if you've changed the `user_id_attribute_name` option of the provider identity.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-identity_relationship_user_id_attribute"},{"type":"DSL","title":"authentication.strategies.slack.icon - dsl-ashauthentication-strategy-slack","doc":"The name of an icon to use in any potential UI. This is a *hint* for UI generators to use, and not in any way canonical.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-icon"},{"type":"DSL","title":"authentication.strategies.slack.openid_configuration_uri - dsl-ashauthentication-strategy-slack","doc":"The URI for the OpenID provider","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-openid_configuration_uri"},{"type":"DSL","title":"authentication.strategies.slack.client_authentication_method - dsl-ashauthentication-strategy-slack","doc":"The client authentication method to use.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-client_authentication_method"},{"type":"DSL","title":"authentication.strategies.slack.openid_configuration - dsl-ashauthentication-strategy-slack","doc":"The OpenID configuration.  If not set, the configuration will be retrieved from `openid_configuration_uri`.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-openid_configuration"},{"type":"DSL","title":"authentication.strategies.slack.id_token_signed_response_alg - dsl-ashauthentication-strategy-slack","doc":"The `id_token_signed_response_alg` parameter sent by the Client during Registration.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-id_token_signed_response_alg"},{"type":"DSL","title":"authentication.strategies.slack.id_token_ttl_seconds - dsl-ashauthentication-strategy-slack","doc":"The number of seconds from `iat` that an ID Token will be considered valid.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-id_token_ttl_seconds"},{"type":"DSL","title":"authentication.strategies.slack.nonce - dsl-ashauthentication-strategy-slack","doc":"A function for generating the session nonce, `true` to automatically generate it with `AshAuthentication.Strategy.Oidc.NonceGenerator`, or `false` to disable.","ref":"dsl-ashauthentication-strategy-slack.html#authentication-strategies-slack-nonce"},{"type":"DSL","title":"token - dsl-ashauthentication-tokenresource","doc":"Configuration options for this token resource","ref":"dsl-ashauthentication-tokenresource.html#token"},{"type":"DSL","title":"token.domain - dsl-ashauthentication-tokenresource","doc":"The Ash domain to use to access this resource.","ref":"dsl-ashauthentication-tokenresource.html#token-domain"},{"type":"DSL","title":"token.expunge_expired_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to remove expired tokens.","ref":"dsl-ashauthentication-tokenresource.html#token-expunge_expired_action_name"},{"type":"DSL","title":"token.read_expired_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action use to find all expired tokens.","ref":"dsl-ashauthentication-tokenresource.html#token-read_expired_action_name"},{"type":"DSL","title":"token.expunge_interval - dsl-ashauthentication-tokenresource","doc":"How often to scan this resource for records which have expired, and thus can be removed.","ref":"dsl-ashauthentication-tokenresource.html#token-expunge_interval"},{"type":"DSL","title":"token.store_token_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action to use to store a token, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"dsl-ashauthentication-tokenresource.html#token-store_token_action_name"},{"type":"DSL","title":"token.get_token_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to retrieve tokens from the store, if `require_tokens_for_authentication?` is enabled in your authentication resource.","ref":"dsl-ashauthentication-tokenresource.html#token-get_token_action_name"},{"type":"DSL","title":"token.revocation - dsl-ashauthentication-tokenresource","doc":"Configuration options for token revocation","ref":"dsl-ashauthentication-tokenresource.html#token-revocation"},{"type":"DSL","title":"token.revocation.revoke_token_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to revoke tokens.","ref":"dsl-ashauthentication-tokenresource.html#token-revocation-revoke_token_action_name"},{"type":"DSL","title":"token.revocation.is_revoked_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to check if a token is revoked.","ref":"dsl-ashauthentication-tokenresource.html#token-revocation-is_revoked_action_name"},{"type":"DSL","title":"token.confirmation - dsl-ashauthentication-tokenresource","doc":"Configuration options for confirmation tokens","ref":"dsl-ashauthentication-tokenresource.html#token-confirmation"},{"type":"DSL","title":"token.confirmation.store_changes_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to store confirmation changes.","ref":"dsl-ashauthentication-tokenresource.html#token-confirmation-store_changes_action_name"},{"type":"DSL","title":"token.confirmation.get_changes_action_name - dsl-ashauthentication-tokenresource","doc":"The name of the action used to get confirmation changes.","ref":"dsl-ashauthentication-tokenresource.html#token-confirmation-get_changes_action_name"},{"type":"DSL","title":"user_identity - dsl-ashauthentication-useridentity","doc":"Configure identity options for this resource","ref":"dsl-ashauthentication-useridentity.html#user_identity"},{"type":"DSL","title":"user_identity.domain - dsl-ashauthentication-useridentity","doc":"The Ash domain to use to access this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-domain"},{"type":"DSL","title":"user_identity.user_resource - dsl-ashauthentication-useridentity","doc":"The user resource to which these identities belong.","ref":"dsl-ashauthentication-useridentity.html#user_identity-user_resource"},{"type":"DSL","title":"user_identity.uid_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `uid` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-uid_attribute_name"},{"type":"DSL","title":"user_identity.strategy_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `strategy` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-strategy_attribute_name"},{"type":"DSL","title":"user_identity.user_id_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `user_id` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-user_id_attribute_name"},{"type":"DSL","title":"user_identity.access_token_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `access_token` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-access_token_attribute_name"},{"type":"DSL","title":"user_identity.access_token_expires_at_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `access_token_expires_at` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-access_token_expires_at_attribute_name"},{"type":"DSL","title":"user_identity.refresh_token_attribute_name - dsl-ashauthentication-useridentity","doc":"The name of the `refresh_token` attribute on this resource.","ref":"dsl-ashauthentication-useridentity.html#user_identity-refresh_token_attribute_name"},{"type":"DSL","title":"user_identity.upsert_action_name - dsl-ashauthentication-useridentity","doc":"The name of the action used to create and update records.","ref":"dsl-ashauthentication-useridentity.html#user_identity-upsert_action_name"},{"type":"DSL","title":"user_identity.destroy_action_name - dsl-ashauthentication-useridentity","doc":"The name of the action used to destroy records.","ref":"dsl-ashauthentication-useridentity.html#user_identity-destroy_action_name"},{"type":"DSL","title":"user_identity.read_action_name - dsl-ashauthentication-useridentity","doc":"The name of the action used to query identities.","ref":"dsl-ashauthentication-useridentity.html#user_identity-read_action_name"},{"type":"DSL","title":"user_identity.user_relationship_name - dsl-ashauthentication-useridentity","doc":"The name of the belongs-to relationship between identities and users.","ref":"dsl-ashauthentication-useridentity.html#user_identity-user_relationship_name"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0-rc.2"}}